<?php
declare(strict_types=1);

/* webEdition CMS
 *
 * $Rev: dcd059745ca7 $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-06-09 11:26 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

/** This class deals with the frames for the liveUpdate
 * Not much functionality here, just show the requested frame
 */
class welu_update_frames{
	/** @var string */
	var $Section;

	/** @var array<string,mixed> */
	var $Data;

	public function __construct(){

		// depending on section variables execute different stuff to gather
		// data for the frame


		$this->Section = we_base_request::string('section', 'frameset');

		switch($this->Section){
			case 'frameset':
				// open frameset
				$this->Data['activeTab'] = self::getValidTab(we_base_request::string('active', ''));

				break;
			case 'tabs':
				// frame with tabs
				$this->Data['activeTab'] = we_base_request::string('active', '');
				break;

			case 'update':
				$this->processUpdateVariables();
				break;

			case 'beta':
				$this->processUpdateVariables();
				$this->processBeta();
				break;

			case 'updatelog':
				$this->processUpdateLogVariables();
				break;

			case 'languages':
				$this->processDeleteLanguages();
				break;
		}
	}

	function getFrame(): string{
		switch($this->Section){
			case 'tabs':
				return welu_update_templates::tabs($this);
			case 'frameset':
				return $this->htmlFrameset();
			case 'beta':
				return welu_update_templates::beta($this);
			case 'update':
				return welu_update_templates::update($this);
			case 'languages':
				return welu_update_templates::languages($this);
			case 'updatelog':
				return welu_update_templates::updateLog($this);
			case 'connect':
				return welu_update_templates::connect();
			default:
				return 'Frame ' . $this->Section . ' is not known!';
		}
	}

	function getData(string $name){
		if(isset($this->Data[$name])){
			return $this->Data[$name];
		}
		return false;
	}

	function processBeta(): void{
		if(($setTestUpdate = we_base_request::int('setTestUpdate', -1)) !== -1){
			$_SESSION['weS']['testUpdate'] = (bool) $setTestUpdate;
		}
	}

	function processUpdateVariables(): void{
		$this->Data['lastUpdate'] = g_l('liveUpdate', '[update][neverUpdated]');

		if(($date = fS('SELECT DATE_FORMAT(datum, "%d.%m.%y - %T ") FROM ' . UPDATE_LOG_TABLE . ' WHERE error=0 ORDER BY ID DESC LIMIT 1'))){
			$this->Data['lastUpdate'] = $date;
		}
	}

	function processDeleteLanguages(): void{
		$deletedLngs = [];
		$notDeletedLngs = [];

		if(($langs = we_base_request::stringList('deleteLanguages'))){
			// update prefs_table

			DB_WE::$inst->query('UPDATE ' . PREFS_TABLE . ' SET value="' . WE_LANGUAGE . '" WHERE `key`="Language" AND value IN ("","' . implode('","', array_map('escape_sql_query', $langs)) . '")');

			// delete folders
			foreach($langs as $lng){
				'@phan-var string $lng';
				if($lng != '' && strpos($lng, '..') === false){
					if(we_base_file::deleteLocalFolder(LIVEUPDATE_SOFTWARE_DIR . '/webEdition/we/include/language/' . $lng, true, true)){
						$deletedLngs[] = $lng;
					} else {
						$notDeletedLngs[] = $lng;
					}
				}
			}
		}
		$this->Data['deletedLngs'] = $deletedLngs;
		$this->Data['notDeletedLngs'] = $notDeletedLngs;
	}

	function processUpdateLogVariables(): void{
		$db = DB_WE::$inst;
		$this->Data['amountPerPage'] = 5;
		$show = [
			'msg' => we_base_request::bool('messages', false),
			'err' => we_base_request::bool('errors', false),
			'notice' => we_base_request::bool('notices', false),
		];
		$errors = [];
		$tmp = 0;
		foreach($show as $cur){
			if($cur){
				$errors[] = $tmp;
			}
			$tmp++;
		}
		$condition = ' WHERE ' . ($errors ? ' error IN (' . implode(',', $errors) . ')' : ' FALSE ');

		/*
		 * process update_cmd
		 */

		switch(we_base_request::string('log_cmd', '')){
			case 'deleteEntries':
				$db->query('DELETE FROM ' . UPDATE_LOG_TABLE . $condition);
				$start = 0;
				break;
			case 'nextEntries':
				$start = we_base_request::int('start') + $this->Data['amountPerPage'];
				break;
			case 'lastEntries':
				$start = we_base_request::int('start') - $this->Data['amountPerPage'];
				break;
			default:
				$start = 0;
				break;
		}

		/*
		 * Check if there are Log-Entries
		 */
		// complete amount

		$this->Data['amountMessages'] = $this->Data['amountNotices'] = $this->Data['amountErrors'] = $this->Data['allEntries'] = $this->Data['amountEntries'] = 0;

		$db->query('SELECT COUNT(ID) as amount, error FROM ' . UPDATE_LOG_TABLE . ' GROUP BY error');
		while($db->next_record()){
			$this->Data['allEntries'] += $db->f('amount');

			switch($db->f('error')){
				case 0:
					$this->Data['amountMessages'] = $db->f('amount');
					if($show['msg']){
						$this->Data['amountEntries'] += $db->f('amount');
					}
					break;
				case 1:
					$this->Data['amountErrors'] = $db->f('amount');
					if($show['err']){
						$this->Data['amountEntries'] += $db->f('amount');
					}
					break;
				case 2:
					$this->Data['amountNotices'] = $db->f('amount');
					if($show['notice']){
						$this->Data['amountEntries'] += $db->f('amount');
					}
			}
		}
		$this->Data['start'] = max(0, $start);

		if($this->Data['allEntries']){
			//There are entries available, get them
			$this->Data['logEntries'] = $db->getAllq('SELECT DATE_FORMAT(datum, "' . str_replace(' ', '&nbsp;/&nbsp;', g_l('date', '[format][mysql]')) . '") AS date, aktion AS action, versionsnummer AS version, error AS `state` FROM ' . UPDATE_LOG_TABLE . ' ' . $condition . ' ORDER BY ID DESC LIMIT ' . $this->Data['start'] . ',' . ((int) $this->Data['amountPerPage']));
		}
	}

	/**
	 * @return string
	 */
	private function htmlFrameset(): string{
		$activeTab = self::getValidTab((string) $this->Data['activeTab']);

		we_html_tools::headerCtCharset('text/html');
		return we_html_tools::getHtmlTop('webEdition Update', ' ', '<body style="overflow:hidden">' .
				we_html_element::iframe('updatetabs', welu_update_const::LOCAL_SCRIPT . '?section=tabs&active=' . $activeTab, 'position: absolute;top:0;left:0;right:0;height:25px;') .
				we_html_element::iframe('updatecontent', welu_update_const::LOCAL_SCRIPT . '?section=' . $activeTab, 'position: absolute;top:25px;left:0;right:0;bottom:0;') .
				we_html_element::iframe('updateload', 'about:blank', 'display:none;') . '</body>');
	}

	static function htmlConnectionSuccess(string $errorMessage = ''): string{
		return welu_update_templates::getHtml(g_l('liveUpdate', '[connect][headline]'), we_html_element::div([], ($errorMessage ? // servers response is error string
					g_l('liveUpdate', '[connect][connectionSuccessError]', false, true) . we_html_element::div(['class' => 'errorDiv'], '<code>' . $errorMessage . '</code>') :
					g_l('liveUpdate', '[connect][connectionSuccess]')
		)));
	}

	static function htmlConnectionError(): string{
		return welu_update_templates::connectError();
	}

	static function htmlStateMessage(string $msg): string{
		$description = /* ($this->State === 'true' ?
			  g_l('liveUpdate', '[state][descriptionTrue]') : */
			g_l('liveUpdate', '[state][descriptionError]');

		$content = we_html_element::div([], $description .
				we_html_element::div(['class' => 'errorDiv'], '<code>' . $msg . '</code>')
		);

		return welu_update_templates::getHtml(g_l('liveUpdate', '[state][headline]'), $content);
	}

	static function getValidTab(string $showTab = ''): string{
		if(in_array($showTab, welu_update_const::UPDATE_CMDS, true)){
			return $showTab;
		}
		return welu_update_const::UPDATE_CMDS[0];
	}

	public static function getJSLangConsts(): string{
		return we_base_util::setLangString([
				'confirmDelete' => g_l('liveUpdate', '[updatelog][confirmDelete]'),
				'languagesDeleted' => g_l('liveUpdate', '[languages][languagesDeleted]'),
				'languagesNotDeleted' => g_l('liveUpdate', '[languages][languagesNotDeleted]'),
		]);
	}

}
