<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: aacab26fe68a $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-05-05 10:25 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

/**
 * Class we_backup_binary
 *
 * Provides functions for exporting and importing backups.
 */
class we_backup_binary extends we_backup_contentBase implements we_backup_restoreIF, we_exim_xmlIF{
	public const BACKUP_XML_TAG = 'we:binary';

	/** @var we_database_base */
	protected $db;

	/** @var array<string,string> */
	var $persistent_slots = [
		'ID' => we_base_request::INT,
		'Path' => we_base_request::STRING,
		'Data' => we_base_request::RAW,
	];

	/** @var int */
	var $ID = 0;

	/** @var string */
	var $Path = '';

	/** @var string */
	var $Data = '';

	/** @var int */
	var $SeqN = 0;

	/** @var string */
	public $targetDir = '';

	function __construct(int $id = 0){
		$this->db = new DB_WE();
		if($id){
			$this->load($id, false);
		}
	}

	function load(int $id = 0, bool $loadData = true): bool{
		$this->ID = $id ?: 0;
		if($id && ($path = fS('SELECT Path FROM ' . FILE_TABLE . ' WHERE ID=' . $id))){
			$this->Path = $path;
			if($this->Path && $loadData){
				return $this->loadFile($this->Path);
			}
		}
		return false;
	}

	function loadFile(string $file): bool{
		$path = str_replace([WE_DOCUMENT_ROOT, WE_SITE_DIR], '', $file);
		$this->Path = $path;
		try{
			return !empty($this->Data = we_base_file::load($file));
		} catch (we_exception_file $e){
			t_e($e);
			return false;
		}
	}

	function we_save(bool $force = false, bool $rebuild = false, array $skipWrite = []): bool{
		//FIXME: check this
		/* $path = WE_DOCUMENT_ROOT . ($this->ID ? rtrim(WE_SITE_DIR, '/') : '') . $this->Path;
		  if(file_exists($path) && !$force){
		  return false;
		  }
		  if(!is_dir(dirname($path))){
		  we_base_file::createLocalFolderByPath(dirname($path));
		  }
		  if($this->Data === ''){//new backups don't store this attribute
		  we_backup_settings::i()->class->extractBinFile(we_backup_exportBase::FILES_DIR, $this->Path);
		  } else {
		  we_base_file::save($path, $this->Data, ($this->SeqN == 0 ? 'wb' : 'ab'));
		  } */
		return true;
	}

	public function getFilesize(): int{
		$path = WE_DOCUMENT_ROOT . $this->Path;
		return (file_exists($path) ?
			(filesize($path) ?: 0) :
			(file_exists(($path = WE_DOCUMENT_ROOT . rtrim(WE_SITE_DIR, '/') . $this->Path)) ?
			(filesize($path) ?: 0) :
			0)
			);
	}

	public function getLogString(string $prefix = ''): string{
		return $prefix . FILE_TABLE . $this->ID . ':' . $this->Path;
	}

	public function toXML(array $attribs = array(), array $extraSlots = array(), string $content = ''): string{
		return $this->binary2xml($content);
	}

	/**
	 *
	 * @return array{id:int,path:string}
	 */
	protected function getBackupData(): array{
		return ['id' => $this->ID, 'path' => $this->Path];
	}

	public function setBackupData(array $data): bool{
		$this->ID = (int) $data['id'];
		$this->Path = (string) $data['path'];
		$path = WE_DOCUMENT_ROOT . ($this->ID ? rtrim(WE_SITE_DIR, '/') : '') . $this->Path;

		if(!is_dir(dirname($path))){
			we_base_file::createLocalFolderByPath(dirname($path));
		}
		$class = we_backup_settings::i()->class;
		assert($class instanceof we_backup_importBase);
		$class->extractBinFile(we_backup_exportBase::FILES_DIR, $this->Path);
		return true;
	}

	protected function getTable(): string{
		return FILE_TABLE;
	}

	/**
	 *
	 * @staticvar int $id
	 * @staticvar string $attribs
	 * @param string $content
	 * @return string
	 */
	protected function binary2xml(string $content = ''): string{
		static $id = 0;
		static $attribs = '';
		$tag = we_exim_contentProvider::getTagName($this);
		if($id === $this->ID){
			return '<' . $tag . '>' . $attribs . $content . '</' . $tag . '>';
		}
		$id = $this->ID;
		foreach(array_keys($this->persistent_slots) as $v){
			assert(is_string($v));
			switch($v){
				case 'Data':
				case 'SeqN':
					break;
				default:
					$content = (property_exists($this, $v) ? $this->{$v} : '');
					list($content, $coding) = we_exim_contentProvider::formatXMLData($content);
					$attribs .= we_xml_composer::we_xmlElement($v, (string) $content, $coding);
			}
		}
		return '<' . $tag . '>' . $attribs . $content . '</' . $tag . '>';
	}

	public function binary2file(/* resource */ $file): bool{
		if(!($this->Path)){
			return false;
		}
		$offset = 0;
		$rsize = we_base_util::MBYTE;
		do{
			//prefer doc_root over site.
			//FIXME: this must be changed, if parking of documents is implemented
			$path = WE_DOCUMENT_ROOT . $this->Path;
			if($this->Path == ''){
				break;
			}
			if(!file_exists($path)){
				$path = WE_DOCUMENT_ROOT . WE_SITE_DIR . $this->Path;
			}
			$data = we_base_file::loadPart($path, $offset, $rsize);
			if($data){
				fwrite($file, $this->binary2xml(we_xml_composer::we_xmlElement('SeqN', (string) $this->SeqN) .
						we_xml_composer::we_xmlElement('Data', we_exim_contentProvider::encode($data), [we_exim_contentProvider::CODING_ATTRIBUTE => we_exim_contentProvider::CODING_ENCODE])) .
					we_backup_util::backupMarker . "\n");
				$offset += $rsize;
				$this->SeqN++;
			}
		} while($data);
		return true;
	}

	public function getPersistent(): array{
		return $this->persistent_slots;
	}

}
