<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 560042758e6a $
 * $Author: Lukas Imhof $
 * $Date: 2020-05-07 21:16 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_dialog_image extends we_dialog_base{
	/** @var bool */
	protected $isTinyDialog = true;

	/** @var string */
	protected $editor = '';

	/** @var bool */
	protected $isFrontend = false;

	/** @var bool */
	protected $outsideWE = false;
	protected $disableWidtheight = false;

	/** @var string */
	protected $initialSrc = '';

	/** @var int */
	protected $initialID = 0;

	/** @var string */
	protected $initialLongdescSrc = '';

	/** @var string */
	protected $extSrc = we_base_link::EMPTY_EXT;

	/** @var string */
	protected $type = we_base_link::TYPE_INT;

	/** @var string */
	protected $intSrc = '';

	/** @var int */
	protected $intID = 0;

	/** @var int */
	protected $selectorStartID = 0;

	/** @var int */
	protected $thumbnailID = 0;

	/** @var string */
	protected $longdescSrc = '';

	/** @var int */
	protected $longdescID = 0;

	/** @var int */
	protected $height = 0;

	/** @var int */
	protected $width = 0;

	/** @var int */
	protected $ratio = 1;

	/** @var int */
	protected $hspace = 0;

	/** @var int */
	protected $vspace = 0;

	/** @var int */
	protected $border = 0;

	/** @var string */
	protected $name = '';

	/** @var string */
	protected $alt = '';

	/** @var string */
	protected $class = '';

	/** @var string */
	protected $title = '';

	/** @var string */
	protected $align = '';

	/** @var int */
	protected $rendered_width = 0;

	/** @var int */
	protected $rendered_heigh = 0;

	const TASK_OPENDIALOG = 'openDialog';
	const TASK_WRITEBACK = 'writeBack';
	const TASK_CHANGETYPE = 'changeType';
	const TASK_CHANGEIMAGE = 'changeImage';
	const TASK_CHANGETHUMBNAIL = 'changeThumbnail';

	protected function __construct(bool $noInternals = true){
		parent::__construct($noInternals);
		$this->argTypes = array_merge($this->argTypes, ['type' => we_base_request::STRING,
			'initialSrc' => we_base_request::URL,
			'initialID' => we_base_request::INT,
			'initialLongdescSrc' => we_base_request::URL,
			'editor' => we_base_request::STRING,
			'outsideWE' => we_base_request::BOOL,
			'isFrontend' => we_base_request::BOOL,
			'extSrc' => we_base_request::URL,
			'intSrc' => we_base_request::URL,
			'intID' => we_base_request::INT,
			'selectorStartID' => we_base_request::INT,
			'thumbnailID' => we_base_request::INT,
			'longdescSrc' => we_base_request::URL,
			'longdescID' => we_base_request::INT,
			'height' => we_base_request::INT,
			'width' => we_base_request::INT,
			'ratio' => we_base_request::INT,
			'hspace' => we_base_request::INT,
			'vspace' => we_base_request::INT,
			'border' => we_base_request::INT,
			'name' => we_base_request::STRING,
			'alt' => we_base_request::STRING,
			'class' => we_base_request::STRING,
			'title' => we_base_request::STRING,
			'align' => we_base_request::STRING,
			'rendered_width' => we_base_request::INT,
			'rendered_height' => we_base_request::INT,
		]);

		$this->dialogTitle = g_l('wysiwyg', '[insert_image]');
		$this->bodyId = 'weImageDialog';

		$this->initByHttp();
	}

	protected function getHiddenArgs(): string{
		return we_html_element::hiddens([
				'we_cmd[task]' => 'writeBack'
		]);
	}

	protected function initByHttp(): void{
		parent::initByHttp();
		if($this->task === self::TASK_OPENDIALOG){
			if($this->initialSrc){
				$this->initFromSrc($this->initialSrc);
				$this->initLongdescFromSrc($this->initialLongdescSrc);
				//$this->jsCmd->addCmd('we_cmd', ['fill_dialog_from_html']);
			} elseif($this->initialID){
				$this->intID = $this->initialID;
			}
		}

		switch($this->type){
			case we_base_link::TYPE_INT:
				if($this->intID && ($this->intSrc = id2path((int) $this->intID))){
					switch($this->task){
						case self::TASK_OPENDIALOG:
							if(!$this->initialID){
								break;
							}
						case self::TASK_CHANGETYPE:
						case self::TASK_CHANGEIMAGE:
							$this->setAttributesFromImage();
							break;
						case self::TASK_CHANGETHUMBNAIL:
							if(!$this->thumbnailID){
								$this->setAttributesFromImage(true);
							}
							break;
					}
					if($this->thumbnailID){
						$this->width = 0;
						$this->height = 0;
						$this->disableWidtheight = true;
					}
				}
				break;
			case we_base_link::TYPE_EXT:
				$this->extSrc = $this->extSrc ? preg_replace(['|^https?://' . $_SERVER['SERVER_NAME'] . '(/.*)$|i',
						'|^' . WEBEDITION_DIR . 'we_cmd.php[^"\'#]+(#.*)$|',
						'|^' . WEBEDITION_DIR . '|',
						], ['${1}', '${1}', ''], (string) $this->extSrc) : we_base_link::EMPTY_EXT;
				$this->disableWidtheight = true;

				switch($this->task){
					case self::TASK_CHANGETYPE:
						$this->resetAttributes();
					case self::TASK_CHANGEIMAGE:
						if($this->extSrc === we_base_link::EMPTY_EXT){
							$this->resetAttributes();
						}
				}
				break;
		}

		$this->longdescSrc = id2path((int) $this->longdescID);
		$this->longdescID = $this->longdescSrc ? $this->longdescID : 0;
	}

	private function initFromSrc(string $src): void{
		$this->type = we_base_link::TYPE_INT;
		$this->intID = 0;
		$this->intSrc = '';
		$this->thumbnailID = 0;
		$this->extSrc = we_base_link::EMPTY_EXT;

		if($src){
			$tokkens = explode('?', $src);
			if(count($tokkens) == 2){
				$this->type = we_base_link::TYPE_INT;
				$foo = [];
				parse_str($tokkens[1], $foo);
				if(isset($foo['id'])){
					$this->intID = (int) $foo['id'];
				} else if(isset($foo['thumb'])){
					$foo = explode(',', $foo['thumb']);
					$this->intID = (int) $foo[0];
					$this->thumbnailID = (int) $foo[1];
				}
			} else {
				$this->type = we_base_link::TYPE_EXT;
			}
		}
	}

	private function setAttributesFromImage($dimensionOnly = false): void{
		$imgObj = new we_document_image($this->intID);
		$this->width = $imgObj->getElementI('width');
		$this->height = $imgObj->getElementI('height');

		if(!$dimensionOnly){
			$this->alt = $imgObj->getElementS('alt');
			// ?? // $this->alt = fS('SELECT c.Dat FROM ' . CONTENT_TABLE . ' c WHERE c.DocumentTable="' . stripTblPrefix(FILE_TABLE) . '" AND c.DID=' . intval($this->intID) . ' AND c.nHash=x\'' . md5('alt') . '\'');
			$this->hspace = $imgObj->getElementI('hspace');
			$this->vspace = $imgObj->getElementI('vspace');
			$this->title = ($imgObj->getElementB('useMetaTitle') && $imgObj->getElementS('Title') ? $imgObj->getElementS('Title') : $imgObj->getElementS('title'));
			$this->name = $imgObj->getElementS('name');
			$this->align = $imgObj->getElementS('align');
			$this->border = $imgObj->getElementI('border');
			$this->longdescID = $imgObj->getElementI('longdescid');
		}
	}

	private function resetAttributes(): void{
		$this->width = 0;
		$this->height = 0;
		$this->alt = '';
		$this->title = '';
		$this->border = 0;
		$this->hspace = 0;
		$this->vspace = 0;
		$this->align = '';
		$this->longdescID = 0;
	}

	private function initLongdescFromSrc(string $src = ''): void{
		$this->longdescID = 0;
		$this->longdescSrc = '';

		if($src && count(($parts = explode('?id=', $src)))){
			$this->longdescID = (int) $parts[1];
		}
	}

	protected function getDialogPartsHTML(): we_html_multiIconBoxContent{
		$weSuggest = we_gui_suggest::getInstance();
		if($this->noInternals || $this->outsideWE){
			$this->type = we_base_link::TYPE_EXT;
			$extSrc = we_html_tools::formElementTable('', we_html_tools::textInput('we_dialog_args[extSrc]', str_replace('%20', ' ', (string) $this->extSrc), 0, [], 'text', '410px'));
			$typeSelect = we_html_element::hidden('we_dialog_args[type]', we_base_link::TYPE_EXT);
			$intSrc = $thumbnails = '';
			$longdesc = we_html_tools::formElementTable(g_l('weClass', '[longdesc_text]'), we_html_tools::textInput('we_dialog_args[longdescSrc]', str_replace('"', '&quot;', (string) ($this->longdescSrc ?: '')), 0, [
						], 'text', '520px'));
		} else {
			$typeSelect = we_html_tools::select('we_dialog_args[type]', array_filter([we_base_link::TYPE_INT => g_l('wysiwyg', '[internal_image]'),
					we_base_link::TYPE_EXT => g_l('wysiwyg', '[external_image]'),
					]), 1, $this->type, false, ['onchange' => "WE().util.dialogTop(window).we_cmd('dialog_setType',this.value)",
					'class' => 'big',
					'style' => 'margin-bottom:5px;width:300px;'
			]);

			$btnExt = we_base_permission::hasPerm([we_base_perm::WORK_EXTERNAL_FILES_CAN_SELECT]) ?
				we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('browse_server','we_dialog_args[extSrc]','" . we_base_ContentTypes::IMAGE . "',document.we_form.elements['we_dialog_args[extSrc]'].value,'dialog_extSrcOnchange')") : '';
			$btnOpen = we_html_button::create_button(we_html_button::OPEN_TO_EDIT, "javascript:WE().util.dialogTop(window).openExtSource('extHref');", '', '', ($this->extSrc === we_base_link::EMPTY_EXT ? true : false), false, '_ext', false, g_l('wysiwyg', '[openNewWindow]'));
			$textInput = we_html_tools::textInput('we_dialog_args[extSrc]', $this->extSrc, 0, ['onchange' => "WE().util.dialogTop(window).extSrc_doOnchange;"], 'text', '315px');
			$extSrc = we_html_tools::formElementTable('', $textInput, $btnExt, $btnOpen);

			$startID = (int) ($this->selectorStartID ?: (IMAGESTARTID_DEFAULT ?: 0));
			$weSuggest->setAcId('Image')
				->setContentType([we_base_ContentTypes::IMAGE])
				->setInput('we_dialog_args[intSrc]', str_replace('"', '&quot;', (string) ($this->intSrc ?: id2path($startID))))
				->setJsCommandOnItemSelect('dialog_imageChanged')
				->setLabel('')
				->setMaxResults(10)
				->setResult('we_dialog_args[intID]', (strval($this->intID ?: $startID)))
				->setSelector(we_gui_suggest::DocSelector)
				->setWidth(315)
				->setSelectButton(we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('we_selector_image',{id:document.we_form.elements['we_dialog_args[intID]'].value,table:WE().consts.tables.FILE_TABLE,targetID:'we_dialog_args[intID]',targetName:'we_dialog_args[intSrc]',open_doc:" . (we_base_permission::hasPerm([
							we_base_perm::WORK_WORKSPACE_CAN_SELECT_OTHER_FILES]) ? 0 : 1) . "});"));
			$parent = empty($this->intID) ? $startID : f('SELECT ParentID FROM ' . FILE_TABLE . ' WHERE ID=' . $this->intID);
			$weSuggest->setOpenButton(we_html_button::create_button(we_html_button::OPEN_TO_EDIT, "javascript:WE().layout.openToEdit(WE().consts.tables.FILE_TABLE,document.we_form.elements.yuiAcResultImage.value,WE().consts.contentTypes.IMAGE);"))
				->setAdditionalButton(we_html_button::create_button(we_html_button::ADD_IMAGE, "javascript:we_cmd('we_fileupload_editor', {accept: WE().consts.contentTypes.IMAGE, import: true, parentid: " . $parent . ", writeback: 'we_suggest_processExternalSelection,Image'})"))
				->setIsDropFromTree(true)
				->setIsDropFromExt(true);
			$intSrc = $weSuggest->getHTML();

			$thumbdata = (!$this->thumbnailID ? '' : $this->thumbnailID);
			$thumbnails = '<select id="selectThumbnail" name="we_dialog_args[thumbnailID]" onchange="thumbnailChanged();"' . ($this->getDisplayThumbsSel() === 'none' ? ' disabled="disabled"' : '') . '>';
			$thumbnails .= '<option value="0"' . (($thumbdata == 0) ? ' selected="selected"' : '') . '>' . g_l('wysiwyg', '[nothumb]') . '</option>';
			foreach($this->db->getAllFirstq('SELECT ID,Text,description FROM ' . THUMBNAILS_TABLE . ' ORDER BY Text', true, MYSQLI_ASSOC) as $id => $item){
				$thumbnails .= '<option title="' . $item['description'] . '" value="' . $id . '"' . (($thumbdata == $id) ? (' selected="selected"') : "") . '>' . $item['Text'] . '</option>';
			}
			$thumbnails .= '</select>';

			$weSuggest->setAcId('Longdesc')
				->setContentType([we_base_ContentTypes::FOLDER_DOCUMENT, we_base_ContentTypes::WEDOCUMENT, we_base_ContentTypes::HTML])
				->setInput('we_dialog_args[longdescSrc]', str_replace('"', '&quot;', (string) ($this->longdescSrc ?: '')))
				->setLabel(g_l('weClass', '[longdesc_text]'))
				->setJsCommandOnItemSelect('')
				->setMaxResults(7)
				->setResult('we_dialog_args[longdescID]', (strval($this->longdescID) ?: ''))
				->setSelector(we_gui_suggest::DocSelector)
				->setWidth(315)
				->setSelectButton(we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('we_selector_document',{id:document.we_form.elements['we_dialog_args[longdescID]'].value,table:WE().consts.tables.FILE_TABLE,targetID:'we_dialog_args[longdescID]',targetName:'we_dialog_args[longdescSrc]',open_doc:" . (we_base_permission::hasPerm([
							we_base_perm::WORK_WORKSPACE_CAN_SELECT_OTHER_FILES]) ? 0 : 1) . "});"))
				->setTrashButton(we_html_button::create_button(we_html_button::TRASH, "javascript:we_cmd('dialog_emptyLongdesc');"))
				->setAdditionalButton('');

			$longdesc = $weSuggest->getHTML();
		}

		$height = we_html_tools::formElementTable(g_l('wysiwyg', '[height]'), we_html_tools::textInput('we_dialog_args[height]', (string) ($this->height ?: ''), 0, [
					'onkeypress' => "return WE().util.IsDigitPercent(event);", 'onkeyup' => "return checkWidthHeight(this);"], 'text', '140px', false, !empty($this->thumbnailID)));
		$width = we_html_tools::formElementTable(g_l('wysiwyg', '[width]'), we_html_tools::textInput('we_dialog_args[width]', (string) ($this->width ?: ''), 0, [
					'onkeypress' => "return WE().util.IsDigitPercent(event);",
					'onkeyup' => "return checkWidthHeight(this);"], 'text', '140px', false, !empty($this->thumbnailID)));
		$onclick = "checkWidthHeight(document.we_form.elements['we_dialog_args[width]']);";
		$ratio = we_html_forms::checkboxWithHidden(boolval($this->ratio) ?: false, 'we_dialog_args[ratio]', g_l('thumbnails', '[ratio]'), false, "defaultfont", $onclick);
		$hspace = we_html_tools::formElementTable(g_l('wysiwyg', '[hspace]'), we_html_tools::textInput('we_dialog_args[hspace]', (string) ($this->hspace ?: ''), 0, [], 'number', '140px'));
		$vspace = we_html_tools::formElementTable(g_l('wysiwyg', '[vspace]'), we_html_tools::textInput('we_dialog_args[vspace]', (string) ($this->vspace ?: ''), 0, [], 'number', '140px'));
		$border = we_html_tools::formElementTable(g_l('wysiwyg', '[border]'), we_html_tools::textInput('we_dialog_args[border]', (string) ($this->border ?: ''), 0, [], 'number', '140px'));
		//$name = we_html_tools::formElementTable(we_html_tools::textInput('we_dialog_args[name]', (isset($this->name) ? $this->name : ""), '', '', "number", 315), "Name");
		$alt = we_html_tools::formElementTable(g_l('wysiwyg', '[altText]'), we_html_tools::textInput('we_dialog_args[alt]', (string) ($this->alt ?: ''), 0, [], 'text', '315px'));
		$title = we_html_tools::formElementTable(g_l('global', '[title]'), we_html_tools::textInput('we_dialog_args[title]', (string) ($this->title ?: ''), 0, [], 'text', '315px'));

		$alignSelect = we_html_tools::getAlignSelect('we_dialog_args[align]', $this->align, g_l('wysiwyg', '[alignment]'), '', '', 120);
		$classSelect = we_html_tools::formElementTable(g_l('wysiwyg', '[css_style]'), $this->getClassSelect('width: 140px;'));

		return (new we_html_multiIconBoxContent())
				->add(new we_html_multiIconBoxContentLine('', we_html_element::div([], $typeSelect) .
						we_html_element::div(['id' => 'imageExt', 'class' => 'changetype ' . we_base_link::TYPE_EXT . '_row' . ($this->type !== we_base_link::TYPE_EXT ? ' hidden' : '')], $extSrc) .
						we_html_element::div(['id' => 'imageInt', 'class' => 'changetype ' . we_base_link::TYPE_INT . '_row' . ($this->type !== we_base_link::TYPE_INT ? ' hidden' : '')], $intSrc)
				))
				->add(new we_html_multiIconBoxContentLine(g_l('wysiwyg', '[thumbnail]'),
																									$thumbnails
					)
				)
				->add((new we_html_multiIconBoxContentLine(g_l('wysiwyg', '[image][formatting]'),
																									 $classSelect . we_html_element::br() .
						$alignSelect
					))->fold()
				)
				->add((new we_html_multiIconBoxContentLine(g_l('global', '[attributes]'), '<table class="default withSpace" style="width:560px">
					<tr><td>' . $width . '</td><td>' . $height . '</td><td>' . $ratio . '</td></tr>
					<tr><td>' . $border . '</td><td>' . $hspace . '</td><td>' . $vspace . '</td></tr>
					<tr><td colspan="3">' . $alt . '</td></tr>
					<tr><td colspan="3">' . $title . '</td></tr>
					<tr><td colspan="3">' . $longdesc . '</td></tr>
				</table>
				<div></div>' .
						we_html_element::hiddens(['we_dialog_args[name]' => ($this->name ?: ''),
							'we_dialog_args[rendered_width]' => '0',
							'we_dialog_args[rendered_height]' => '0',
							'tinyMCEInitRatioH' => '0',
							'tinyMCEInitRatioW' => '0'
						]) .
						we_html_element::jsScript(WE_JS_TINYMCE_DIR . 'plugins/weimage/js/weimage.js') . we_html_element::jsScript(WE_JS_TINYMCE_DIR . 'weTinyMce_plugin.js')
					))->fold()
		);
	}

	private function getDisplayThumbsSel(): string{
		$p = (string) ($this->intSrc ?: '');
		$tmp = $p ? explode('.', $p) : [];
		$extension = count($tmp) > 1 ? '.' . $tmp[count($tmp) - 1] : '';
		unset($p);

		return (we_base_imageEdit::gd_version() > 0 && we_base_imageEdit::is_imagetype_supported(array_key_exists(strtolower($extension), we_base_imageEdit::GDIMAGE_TYPE) ? we_base_imageEdit::GDIMAGE_TYPE[strtolower($extension)] : "") && $this->type == we_base_link::TYPE_INT) ? 'block' : 'none';
	}

	protected function cmdFunction(array $args): string{
		$attributes = [
			'height' => $this->height ?: '',
			'width' => $this->width ?: '',
			'hspace' => $this->hspace ?: '',
			'vspace' => $this->vspace ?: '',
			'border' => $this->border ?: '',
			'alt' => $this->alt,
			'title' => $this->title,
			'align' => $this->align,
			'longdescSrc' => $this->longdescSrc,
			'longdescID' => $this->longdescID,
		];

		switch($this->task){
			case self::TASK_CHANGETYPE:
			case self::TASK_CHANGEIMAGE:
			case self::TASK_CHANGETHUMBNAIL:
				$payload = ['args' => $attributes,
					'displayThumbnailSel' => $this->getDisplayThumbsSel(),
					'disableWidthHeight' => $this->disableWidtheight];
				$js = we_html_element::jsScript(JS_DIR . 'dialogs/we_dialog_cmdFrame.js', '', [
						'id' => 'loadVarDialog_cmdFrame',
						'data-payload' => setDynamicVar($payload)
				]);
				$this->jsCmd->addCmd('update_dialog_image');
				break;
			case self::TASK_WRITEBACK:
			default:
				switch($this->type){
					case we_base_link::TYPE_INT:
						$attributes['src'] = '';
						if($this->intID){
							if($this->thumbnailID){
								try{
									$thumbObj = new we_contents_thumbnail();
									$thumbObj->initByImageIDAndThumbID($this->intID, $this->thumbnailID);
									if(!file_exists($thumbObj->getOutputPath(true))){
										$thumbObj->getThumb();
									}
									$thumbpath = $thumbObj->getOutputPath();
									$attributes['src'] = $thumbpath . '?thumb=' . $this->intID . ',' . $this->thumbnailID;
								} catch (we_exception_base $e){
									//disable thumb add normal img
									$attributes['src'] = $this->intSrc . '?id=' . $this->intID;
								}
							} else {
								$attributes['src'] = $this->intSrc . '?id=' . $this->intID;
							}
						}
						break;
					case we_base_link::TYPE_EXT:
						$attributes['src'] = $this->extSrc;
						break;
				}
				$attributes['longdesc'] = $this->longdescID ? $this->longdescSrc . '?id=' . $this->longdescID : '';

				$js = we_html_element::jsScript(JS_DIR . 'dialogs/we_dialog_cmdFrame.js', "we_cmd('image_writeback')", [
						'id' => 'loadVarDialog_cmdFrame',
						'data-payload' => setDynamicVar(['attributes' => $attributes])
				]);
		}

		return we_html_tools::getHtmlTop('', $js . $this->jsCmd->getCmds(), we_html_element::body());
	}

	protected function getJs(): string{
		//$weSuggest = & weSuggest::getInstance();
		//$css = !empty($this->cssClasses) ? explode(',', $this->cssClasses' : [];

		return parent::getJs() .
			//we_html_element::jsScript(WE_JS_TINYMCE_DIR . 'weTinyMce_plugin.js') .
			we_html_element::jsScript(JS_DIR . 'multiIconBox.js') .
			we_html_element::jsScript(JS_DIR . 'dialogs/we_dialog_image.js');
	}

	protected function getDialogButtons(): string{
		return we_html_button::position_yes_no_cancel(we_html_button::create_button(we_html_button::OK, "javascript:document.forms.we_form.submit();", '', "", false, false), '', $this->getBtnCancel());
	}

}
