<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 39e7b777091a $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2019-12-20 15:43 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_document_html extends we_document_textContent{
	/** @var string ContentType of the Object  */
	public $ContentType = we_base_ContentTypes::HTML;

	protected function saveContentDataInDB(string $table): bool{
		if(($code = $this->getElementS('data'))){
			$metas = $this->getMetas($code);
			if(!empty($metas['title'])){
				$this->setElementS(self::TITLE_FIELD, $metas['title']);
			}
			if(!empty($metas['description'])){
				$this->setElementS(self::DESCRIPTION_FIELD, $metas['description']);
			}
			if(!empty($metas['keywords'])){
				$this->setElementS(self::KEYWORD_FIELD, $metas['keywords']);
			}
		}
		return parent::saveContentDataInDB($table);
	}

	public function getDocumentCode(): string{
		$code = $this->getElementS('data');
		return preg_replace('/<meta http-equiv="Content-Type" content=".*>|<meta charset=".*>/i', we_html_tools::metaCtCharset(), $code);
	}

	public function getPropertyPage(we_base_jsCmd $jsCmd): string{
		return (new we_html_multiIconBox((new we_html_multiIconBoxContent())
					->add((new we_html_multiIconBoxContentLine(g_l('weClass', '[path]'), $this->formPath(), we_html_multiIconBox::PROP_PATH)))
					->add((new we_html_multiIconBoxContentLine(g_l('weClass', '[document]'), $this->formDocTypeTempl(), we_html_multiIconBox::PROP_DOC)))
					->add(we_modules_info::isActive(we_modules_info::CATEGORY) ? (new we_html_multiIconBoxContentLine(g_l('global', '[categorys]'), $this->formCategory($jsCmd), we_html_multiIconBox::PROP_CATEGORIES))->fold() : null)
					->add((new we_html_multiIconBoxContentLine(g_l('global', '[navigation]'), $this->formNavigation($jsCmd), we_html_multiIconBox::PROP_NAVI))->fold())
					->add((new we_html_multiIconBoxContentLine(g_l('weClass', '[copy' . $this->ContentType . ']'), $this->formCopyDocument(), we_html_multiIconBox::PROP_COPY)))
					->add((new we_html_multiIconBoxContentLine(g_l('weClass', '[owners]'), $this->formCreatorOwners($jsCmd), we_html_multiIconBox::PROP_USER))->fold())
					, 'PropertyPage'))
				->marginLeft()
				->get();
	}

}
