<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: a8f4e663d6cb $
 * $Author: Marc Krämer <marc.kraemer@aldente-it.de> $
 * $Date: 2019-11-27 20:12 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
/*  a class for handling flashDocuments. */

class we_document_pdf extends we_document_other{
	/** @var string ContentType of the Object  */
	public $ContentType = we_base_ContentTypes::PDF;

	/**
	 * create instance of weMetaData to access metadata functionality:
	 */
	protected function getMetaDataReader(bool $force = false): we_metadata_metaData{
		return we_document_binary::getMetaDataReader(true);
	}

	function getDocumentData(string $path = '', int $size = -1): string{
		$name = $this->getElementS('data');
		if(file_exists($name) && (filesize($name) * 2 < we_base_util::convertIniSizes(ini_get('memory_limit') ?: 0))){
			$pdf = new we_helpers_pdf2text($name);
			return str_replace(['\\n', '\\r'], '', $pdf->processText());
		}
		return '';
	}

	public function setMetaDataFromFile(string $file): void{
		if($this->Extension === '.pdf' && file_exists($file)){
			$pdf = new we_helpers_pdf2text($file);
			$metaData = $pdf->getInfo();
			if($metaData){
				if(isset($metaData['Title']) && !$this->getElementS(self::TITLE_FIELD)){
					$this->setElementS(self::TITLE_FIELD, $metaData['Title']);
				}
				if(isset($metaData['Keywords']) && !$this->getElementS(self::KEYWORD_FIELD)){
					$this->setElementS(self::KEYWORD_FIELD, $metaData['Keywords']);
				}
				if(isset($metaData['Subject']) && !$this->getElementS(self::DESCRIPTION_FIELD)){
					$this->setElementS(self::DESCRIPTION_FIELD, $metaData['Subject']);
				}
			}
		}
	}

}
