<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: b6c8f7b54af9 $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-06-05 15:22 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_editor_checkDocument extends we_editor_base{

	public function show(): string{
		$this->we_doc->InWebEdition = false;
		assert($this->we_doc instanceof we_document_files);
		$content = $this->we_doc->getDocument();

		$host = we_base_request::string('host');
		if($host != 'validator.w3.org' && !fB('SELECT 1 FROM ' . VALIDATION_SERVICES_TABLE . ' WHERE host="' . DB_WE::$inst->escape($host) . '" LIMIT 1')){
			exit($host . ' not in allowed hosts!');
		}

		$path = we_base_request::_(we_base_request::FILE, 'path');
		$s_method = we_base_request::string('s_method');
		$varname = we_base_request::string('varname');
		$contentType = we_base_request::string('ctype');

		$http_request = new we_http_request($path, $host, $s_method);
		$http_request->addHeader('User-Agent', $_SERVER['HTTP_USER_AGENT']);

		//  add additional parameters to the request
		if(($add = we_base_request::string('additionalVars'))){
			$args = explode('&', $add);
			foreach($args as $pair){
				$keyValue = explode('=', $pair);
				$http_request->addVar($keyValue[0], $keyValue[1]);
			}
		}

		//  generate name of file.  - must be .html because of <?xml and short-open tags
		$extension = $this->we_doc->Extension;
		$filename = '/' . $this->transaction . $extension;

		//  check what should happen with document
		if(we_base_request::string('checkvia') === 'fileupload'){ //  submit via fileupload
			$http_request->addFileByContent($varname, $content, $contentType, $filename);
		} else { //  submit via onlinecheck - site must be available online
			// when it is a dynamic document, remove <?xml when short_open_tags are allowed.
			if(ini_get('short_open_tag') == 1 && $this->we_doc->IsDynamic && $contentType === we_base_ContentTypes::HTML){
				$content = str_replace("<?xml", '<?= "<?xml"; ?>', $content);
			}

			//  save file - submit URL to service
			$tmpFile = WE_DOCUMENT_ROOT . $filename;
			we_base_file::save($tmpFile, $content);
			we_base_file::insertIntoCleanUp($tmpFile, 0);

			$url = we_http_util::getServerUrl() . $filename;
			$http_request->addVar($varname, $url);
		}

		$http_request->executeHttpRequest();

		//  check if all worked well..
		if(($err = $http_request->getError())){
			return $this->getPage($err);
		}

		$http_response = new we_http_response($http_request->getHttpResponseStr());
		return ($http_response->getHttp_answer('code') == '200' ?
			//  change base href -> css of included page is loaded correctly
			str_replace('<head>', '<head><base href="http://' . $host . '" />', $http_response->http_body) :
//  no correct answer
			we_html_tools::getHtmlTop(g_l('validation', '[connection_problems]', false, true), '', '<body>' .
				we_html_tools::alertAttentionBox(sprintf(g_l('validation', '[connection_problems]', false, true), $http_response->getHttp_answer()), we_html_tools::TYPE_ALERT, 0, false) .
				'</body>')
			);
	}

}
