<?php

declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: d080b64d292f $
 * $Author: Lukas Imhof $
 * $Date: 2020-05-04 00:36 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
abstract class we_editor_script{

	public static function get(we_contents_root $we_doc): string{
		$ret = '';
		switch($we_doc->ContentType){
			case we_base_ContentTypes::TEMPLATE:
				switch($we_doc->EditPageNr){
					default:
						break;
					case we_base_constants::WE_EDITPAGE_PREVIEW:
					case we_base_constants::WE_EDITPAGE_PREVIEW_TEMPLATE:
				}
			case we_base_ContentTypes::WEDOCUMENT:
			case we_base_ContentTypes::OBJECT_FILE:
			case we_base_ContentTypes::OBJECT:
				switch($we_doc->EditPageNr){
					case we_base_constants::WE_EDITPAGE_PREVIEW:
					case we_base_constants::WE_EDITPAGE_PREVIEW_TEMPLATE:
					case we_base_constants::WE_EDITPAGE_CONTENT:
						$ret = we_wysiwyg_editor::getHTMLHeader(false, (in_array($we_doc->ContentType, [we_base_ContentTypes::OBJECT_FILE,
											we_base_ContentTypes::OBJECT], true)
						));
						break;
				}
		}

		$doc = [
			'we_transaction' => we_base_request::_(we_base_request::TRANSACTION, 'we_transaction', 0),
			'docId' => intval($we_doc->ID),
			'docPath' => $we_doc->Path,
			'docText' => $we_doc->getText(),
			'oldparentid' => intval($we_doc->ParentID),
			'docName' => $we_doc->Name,
			'isFolder' => intval($we_doc instanceof we_folder_base),
			'docTable' => $we_doc->Table,
			'hasCustomerFilter' => intval(we_modules_info::isActive(we_modules_info::CUSTOMER) && in_array(we_base_constants::WE_EDITPAGE_WEBUSER, $we_doc->EditPageNrs, true) && ($we_doc instanceof we_document_base)),
			'hasGlossary' => intval(we_modules_info::isActive(we_modules_info::GLOSSARY) && (in_array($we_doc->ContentType, [we_base_ContentTypes::WEDOCUMENT, we_base_ContentTypes::OBJECT_FILE]))),
			'gdType' => '',
			'gdSupport' => 0,
			'isWEObject' => intval(($we_doc->ContentType === we_base_ContentTypes::OBJECT/* FIXME: only supported for type object || $we_doc->ContentType === we_base_ContentTypes::OBJECT_FILE */)),
			'WE_EDIT_IMAGE' => intval(defined('WE_EDIT_IMAGE')),
			'useSEE_MODE' => false,
			'cmd' => false,
		];


		if($we_doc instanceof we_document_image){
			$doc['gdType'] = $we_doc->getGDType();
			$doc['gdSupport'] = (int) $we_doc->gd_support();
			$doc['filesize'] = $we_doc->getElementI('filesize');
		}
		if($we_doc instanceof we_folder_base && $we_doc->ApplyWeDocumentCustomerFiltersToChilds && $we_doc->ParentID){
			$doc['cmd'] = ['we_customer_copyWeDocumentFilter', $we_doc->ID, $we_doc->Table, $we_doc->ParentID];
		}

		$useSeeModeJS = [
			we_base_ContentTypes::WEDOCUMENT => [we_base_constants::WE_EDITPAGE_CONTENT, we_base_constants::WE_EDITPAGE_PREVIEW],
			we_base_ContentTypes::TEMPLATE => [we_base_constants::WE_EDITPAGE_PREVIEW, we_base_constants::WE_EDITPAGE_PREVIEW_TEMPLATE],
			we_base_ContentTypes::OBJECT_FILE => [we_base_constants::WE_EDITPAGE_CONTENT, we_base_constants::WE_EDITPAGE_PREVIEW]
		];

		if(isset($useSeeModeJS[$we_doc->ContentType]) && in_array($we_doc->EditPageNr, $useSeeModeJS[$we_doc->ContentType], true)){
			$doc['useSEE_MODE'] = true;
		}

		$ret .= we_html_element::cssLink(CSS_DIR . 'editor.css') .
				we_html_element::cssLink(CSS_DIR . 'overlay.css') .
				we_html_element::jsScript(JS_DIR . 'we_editor_script.js', '', ['id' => 'loadVarEditor_script', 'data-doc' => setDynamicVar($doc)]);

		unset($doc);

		return $ret;
	}

}
