<?php

declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 6976d8e0552f $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-05-05 13:54 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
//@phan-file-suppress PhanUndeclaredProperty,PhanAccessPropertyProtected

class we_exim_XMLImport extends we_exim_ExIm{

	/** @var we_exim_XMLParser */
	private $xmlBrowser;

	function import(string $chunk_file): bool{
		!defined('IMPORT_RUNNING') || define('IMPORT_RUNNING', 1);
		$db = new DB_WE();

		$objects = [];

		$data = we_base_file::load($chunk_file);
		$this->xmlBrowser = new we_exim_XMLParser();
		$this->xmlBrowser->parse($data, $this->options['xml_encoding']);
		unset($data);
		$this->xmlBrowser->normalize();
		if(($node_set = $this->xmlBrowser->getChildren(0))){
			foreach($node_set as $node){
				'@phan-var int $node';
				$this->xmlBrowser->seek($node);
				if($this->handleTag(($tag = $this->xmlBrowser->getNodeName()))){
					if(($tmp = $this->importNodeSet($node))){
						$objects[] = [$tag, $tmp];
					}
				}
			}
		}

		$save = false;
		foreach($objects as $tmp){
			list($tag, $object) = $tmp;
			'@phan-var we_exim_element|we_backup_object|we_contents_objectFile|we_folder_objectFile|we_navigation_navigation|we_navigation_rule|we_exim_thumbnailExport $object';
			$save = true;
			$extra = [
				'OldID' => $object->ID ?? 0,
				'OldParentID' => $object->ParentID ?? 0,
				'OldPath' => $object->Path ?? '',
				'OldTemplatePath' => $object instanceof we_document_webEdition ? $object->getTemplatePath() : '',
				'OldDocTypeName' => $object instanceof we_document_webEdition ? $object->DocTypeName : '',
				'Examined' => true,
			];

			if($object instanceof we_contents_root){
				$extra['elements'] = $object->elements;
			}

			$object->ID = 0;
			$object->Table = $this->getTable($tag);

			switch($tag){
				case we_category_category::BACKUP_XML_TAG:
					$extra['ContentType'] = we_base_ContentTypes::CATEGORY;
					break;
				case we_doctype_docTypes::BACKUP_XML_TAG:
					$extra['ContentType'] = we_base_ContentTypes::DOCTYPE;
					assert($object instanceof we_doctype_docTypes);
					$dtid = fI('SELECT ID FROM ' . DOC_TYPES_TABLE . ' WHERE Text="' . $db->escape($object->DocType) . '"');
					if($dtid && !$this->handleCollission($tag, $object, $dtid, 'DocType')){
						$save = false;
						continue 2;
					}
					break;
				case we_navigation_rule::BACKUP_XML_TAG:
					assert($object instanceof we_navigation_rule);
					$nid = fI('SELECT ID FROM ' . NAVIGATION_RULE_TABLE . ' WHERE NavigationName="' . $db->escape($object->NavigationName) . '"');
					if($nid && !$this->handleCollission($tag, $object, $nid, 'NavigationName')){
						$save = false;
						continue 2;
					}
					break;
				case we_contents_thumbnail::BACKUP_XML_TAG:
					assert($object instanceof we_contents_thumbnail);
					$nid = fI('SELECT ID FROM ' . THUMBNAILS_TABLE . ' WHERE Text="' . $db->escape($object->getText()) . '"');
					if($nid && !$this->handleCollission($tag, $object, $nid, 'Name')){
						$save = false;
						continue 2;
					}
					break;
			}

			if(isset($object->Path)){
				if(isset($object->Table) && !empty($object->Table)){
					$prefix = '/';
					switch($object->Table){
						case FILE_TABLE:
							assert($object instanceof we_document_files || $object instanceof we_folder_document);
							if($this->options['document_path']){
								$prefix = id2path((int) $this->options['document_path'], FILE_TABLE);
							}
							$object->Path = $prefix . ($this->options['restore_doc_path'] ?
									$object->Path :
									'/' . $object->getText());
							break;
						case TEMPLATES_TABLE:
							assert($object instanceof we_contents_template || $object instanceof we_folder_template);
							if($this->options['template_path']){
								$prefix = id2path((int) $this->options['template_path'], TEMPLATES_TABLE);
							}
							$object->Path = $prefix . ($this->options['restore_tpl_path'] ?
									$object->Path :
									'/' . $object->getText());
							break;
						case NAVIGATION_TABLE:
							assert($object instanceof we_navigation_navigation);
							if($this->options['navigation_path']){
								$prefix = id2path((int) $this->options['navigation_path'], NAVIGATION_TABLE);
							}
							$object->Path = $prefix . $object->Path;
							break;
					}

					$object->Path = we_base_file::clearPath($object->Path);

					//fix Path if there is a conflict
					$id = path_to_id($object->Path, $object->Table);

					if($id){
						if($this->options['handle_collision'] === we_exim_ExIm::COLLISION_REPLACE ||
								(in_array($object->ContentType, we_base_ContentTypes::FOLDER_TYPES) && $this->RefTable->exists(['OldID' => $object->ID, 'Table' => $object->Table]))
						){
							$object->ID = $id;
							if(isset($object->isnew)){
								$object->isnew = false;
							}
						} else if($this->options['handle_collision'] === we_exim_ExIm::COLLISION_RENAME){
							$this->getNewName($tag, $object, $id, 'Path');
						} else {
							$save = false;
							continue;
						}
					}
				}
				//fix Path ends
				// set OldPath
				if(isset($object->OldPath)){
					$object->OldPath = $object->Path;
				}

				// assign ParentID and ParentPath based on Path
				if(isset($object->Table)){
					$pathids = [];
					$old_pid = $object->ParentID;
					$owner = ($this->options['owners_overwrite'] && $this->options['owners_overwrite_id']) ? $this->options['owners_overwrite_id'] : 0;
					if(we_modules_info::isActive(we_modules_info::OBJECT) && $object->ContentType === we_base_ContentTypes::OBJECT_FILE){
						//dont create Path in objects if the class doesn't exist
						$match = [];
						if(preg_match('|(/+[a-zA-Z0-9_+-\.]*)|', $object->Path, $match) && isset($match[0]) && !fB('SELECT 1 FROM ' . OBJECT_TABLE . ' WHERE Path="' . $db->escape($match[0]) . '" LIMIT 1')){
							return false;
						}
					}
					$object->ParentID = self::makePath(dirname($object->Path), $object->Table, $pathids, $owner);
					if(isset($object->ParentPath)){
						$object->ParentPath = id2path($object->ParentID, $object->Table);
					}
					// insert new created folders in ref table
					foreach($pathids as $pid){

						$h = getHash('SELECT ParentID,Path FROM ' . $db->escape($object->Table) . ' WHERE ID=' . intval($pid), $db);
						if(!$this->RefTable->exists(['ID' => $pid, 'ContentType' => we_base_ContentTypes::FOLDER_DOCUMENT])){
							$this->RefTable->add2([
								'ID' => $pid,
								'ParentID' => $h['ParentID'],
								'Path' => $h['Path'],
								'Table' => $object->Table,
								'ContentType' => we_base_ContentTypes::FOLDER_DOCUMENT,
								'OldID' => ($pid == $object->ParentID) ? $old_pid : null,
								'OldParentID' => null,
								'OldPath' => null,
								'OldTemplatePath' => null,
								'Examined' => false,
									]
							);
						}
					}
				}

				if($tag === we_backup_binary::BACKUP_XML_TAG){
					if(is_file(WE_DOCUMENT_ROOT . $object->Path)){
						switch($this->options['handle_collision']){
							case we_exim_ExIm::COLLISION_REPLACE:
								$save = true;
								break;
							case we_exim_ExIm::COLLISION_RENAME:
								$c = 1;
								do{
									$path = $object->Path . '_' . $c;
									$c++;
								}while(is_file(WE_DOCUMENT_ROOT . $path));
								$object->Path = $path;
								unset($path);
								unset($c);
								break;
							default:
								$save = false;
						}
					}

					if($save && !$this->RefTable->exists(['ID' => $object->ID, 'Path' => $object->Path, 'ContentType' => we_base_ContentTypes::BACKUP_BINARY])){
						$this->RefTable->add2([
							'ID' => $object->ID,
							'ParentID' => 0,
							'Path' => $object->Path,
							'Table' => $object->Table,
							'ContentType' => we_base_ContentTypes::BACKUP_BINARY,
						]);
					}
				}
			}

			switch($tag){
				case we_contents_objectFile::BACKUP_XML_TAG:
				case we_folder_objectFile::BACKUP_XML_TAG:
					if(!we_modules_info::isActive(we_modules_info::OBJECT)){
						break;
					}
					assert($object instanceof we_contents_objectFile || $object instanceof we_folder_objectFile);
					$ref = $this->RefTable->getRef(['OldID' => $object->TableID, 'ContentType' => 'object']);
					if($ref){
						// assign TableID and ParentID from reference
						$object->TableID = $ref->ID;
					} else {
						//assign TableID based on Path
						// evaluate root dir for object
						$match = [];
						preg_match('|(/+[a-zA-Z0-9_+-\.]*)|', $object->Path, $match);
						if(isset($match[0])){
							$object->TableID = fI('SELECT ID FROM ' . OBJECT_TABLE . ' WHERE Path="' . $db->escape($match[0]) . '"');
						}
					}
			}

			// update owners data
			$this->refreshOwners($object);

			if($save){
				$save = $this->saveObject($object);
			}
			$this->RefTable->add($object, $extra);
		}
		return $save;
	}

	private function handleCollission(string $tag, we_base_mainmodel $object, int $id, string $prop): bool{
		switch($this->options['handle_collision']){
			case we_exim_ExIm::COLLISION_REPLACE:
				$object->ID = $id;
				return true;
			case we_exim_ExIm::COLLISION_RENAME:
				$this->getNewName($tag, $object, $id, $prop);
				return true;
			default:
				return false;
		}
	}

	private function getTable(string $tag): string{
		switch($tag){
			case we_contents_template::BACKUP_XML_TAG:
				return TEMPLATES_TABLE;
			case we_doctype_docTypes::BACKUP_XML_TAG:
				return DOC_TYPES_TABLE;
			case we_category_category::BACKUP_XML_TAG:
				return CATEGORY_TABLE;
			case we_navigation_navigation::BACKUP_XML_TAG:
				return NAVIGATION_TABLE;
			case we_navigation_rule::BACKUP_XML_TAG:
				return NAVIGATION_RULE_TABLE;
			case we_thumbnail_thumbnail::BACKUP_XML_TAG:
				return THUMBNAILS_TABLE;
			case we_backup_binary::BACKUP_XML_TAG:
				return '';
			case we_contents_object::BACKUP_XML_TAG:
				return OBJECT_TABLE;
			case we_contents_objectFile::BACKUP_XML_TAG:
				return OBJECT_FILES_TABLE;
			default:
				return FILE_TABLE;
		}
	}

	private function getNewName(string $tag, we_base_mainmodel $object, int $id, string $prop): void{
		$c = 0;
		$newid = $id;
		do{
			$c++;

			switch($tag){
				case we_doctype_docTypes::BACKUP_XML_TAG:
				case we_navigation_rule::BACKUP_XML_TAG:
				case we_contents_thumbnail::BACKUP_XML_TAG:
					$newname = $object->$prop;
					break;
				default:
					$newname = basename($object->$prop);
			}

			if($newid){
				$newname = $c . '_' . $newname;
			}
			switch($tag){
				case we_doctype_docTypes::BACKUP_XML_TAG:
					$newid = fI('SELECT ID FROM ' . DOC_TYPES_TABLE . ' dt WHERE dt.Text="' . DB_WE::$inst->escape($newname) . '"');
					break;
				case we_navigation_rule::BACKUP_XML_TAG:
					$newid = fI('SELECT ID FROM ' . NAVIGATION_RULE_TABLE . ' nr WHERE nr.NavigationName="' . DB_WE::$inst->escape($newname) . '"');
					break;
				case we_contents_thumbnail::BACKUP_XML_TAG:
					$newid = fI('SELECT ID FROM ' . THUMBNAILS_TABLE . ' t WHERE t.Text="' . DB_WE::$inst->escape($newname) . '"');
					break;
				default:
					$newid = path_to_id(we_base_file::clearPath(dirname($object->Path) . '/' . $newname), $object->Table);
			}
		}while($newid);
		$this->renameObject($tag, $object, $newname);
	}

	private function renameObject(string $tag, we_base_mainmodel $object, string $new_name): void{
		switch($tag){
			case we_doctype_docTypes::BACKUP_XML_TAG:
				$object->Text = $new_name;
				return;
			case we_navigation_rule::BACKUP_XML_TAG:
				$object->NavigationName = $new_name;
				return;
			case we_contents_thumbnail::BACKUP_XML_TAG:
				$object->Name = $new_name;
				return;
		}
		if(isset($object->Path)){
//			$path = dirname($object->Path);
			$ref = $this->RefTable->getRef([
				'OldID' => $object->ParentID,
				'ContentType' => we_base_ContentTypes::NAVIGATION
			]);
			if($ref){
				$object->ParentID = $ref->ID;
				$object->Path = $ref->Path . '/' . $new_name;
			} else {
				$object->Path = we_base_file::clearPath(dirname($object->Path) . '/' . $new_name);
			}
		}
		if($object instanceof we_base_model){
			$object->Text = $new_name;
		}
		if(isset($object->Filename)){
			$object->Filename = str_replace($object->Extension, '', $new_name);
		}
	}

	/**
	 *
	 * @param int $node_id
	 * @return null|we_exim_element|we_backup_object|we_contents_objectFile|we_folder_objectFile|we_navigation_navigation|we_navigation_rule|we_exim_thumbnailExport
	 */
	private function importNodeSet(int $node_id): ?we_base_mainmodel{
		$node_data = [];

		if(($node_props = $this->xmlBrowser->getChildren($node_id))){
			$object = null;
			foreach($node_props as $node){
				'@phan-var int $node';
				$this->xmlBrowser->seek($node);
				$nodname = $this->xmlBrowser->getNodeName();
				$noddata = $this->xmlBrowser->getNodeData();
				$attributes = $this->xmlBrowser->getNodeAttributes();

				switch($nodname){
					case self::INFO_TAG:
						$this->importNodeSet($node);
						break;
					case self::MAP_TAG:
						$this->RefTable->Users[$attributes['user']] = $attributes;
						break;
					case we_exim_element::BACKUP_XML_TAG:
						$this->xmlBrowser->addMark(we_exim_element::BACKUP_XML_TAG);
						$content = $this->importNodeSet($node);
						$this->xmlBrowser->gotoMark(we_exim_element::BACKUP_XML_TAG);
						if(!is_object($object)){
							t_e($this->xmlBrowser, $nodname, $noddata, $attributes);
						} else {
							assert($content instanceof we_exim_element);
							$object->elements = array_merge($object->elements, $content->getElement());
						}
						break;
					case we_contents_object::BACKUP_XML_TAG:
						$object = new we_backup_object();
						break;
					case we_contents_objectFile::BACKUP_XML_TAG:
						$object = new we_contents_objectFile();
						break;
					case we_folder_objectFile::BACKUP_XML_TAG://Bug 3857 sonderbehandlung hinzugefuegt, da es sonst hier beim letzten else zum Absturz kommt, es wird nichts geladen, da eigentlich alles geladen ist
						$object = new we_folder_objectFile();
						break;
					case we_navigation_navigation::BACKUP_XML_TAG:
						$object = new we_navigation_navigation();
						break;
					case we_navigation_rule::BACKUP_XML_TAG:
						$object = new we_navigation_rule();
						break;
					case we_contents_thumbnail::BACKUP_XML_TAG:
						$object = new we_exim_thumbnailExport();
						break;
					default:
						//FIXME: check if this is handled correct - it is not: $node_data is never used.
						/*if(!in_array($nodname, we_exim_contentProvider::DONT_IMEXPORT_PROPS, true)){
							$node_coding = (empty($attributes[we_exim_contentProvider::CODING_ATTRIBUTE]) ? we_exim_contentProvider::CODING_NONE : $attributes[we_exim_contentProvider::CODING_ATTRIBUTE] );
							$node_data[$nodname] = we_exim_contentProvider::getDecodedData($node_coding, $noddata);
						}*/
				}
			}
			return $object;
		}

		return self::populateInstance($node_data);
	}

	private static function populateInstance(array $content): we_base_mainmodel{
		$object = new we_exim_element();
		$reflect = new ReflectionClass($object);
		$props = $reflect->getProperties(ReflectionProperty::IS_PRIVATE | ReflectionProperty::IS_PROTECTED);
		foreach($props as $prop){
			unset($content[$prop->getName()]);
		}

		foreach($content as $k => $v){
			$object->$k = $v;
		}
		if(isset($object->persistent_slots) && empty($object->persistent_slots)){
			t_e('this call will cause errors', array_keys($content), $object);
			$object->persistent_slots = array_keys($content);
		}
		return $object;
	}

	function changeEncoding(string $value): string{
		if($this->options['change_encoding']){
			if($this->options['target_encoding'] != '' && $this->options['xml_encoding'] != '' && $this->options['target_encoding'] != $this->options['xml_encoding']){

				if($value == $this->options['xml_encoding']){
					return $this->options['target_encoding'];
				}
				$usv = we_unserialize($value, $value);
				if(is_array($usv)){
					foreach($usv as &$av){
						'@phan-var string $av';
						if($this->options['xml_encoding'] === 'ISO-8859-1'){
							$av = utf8_encode($av);
						} else {
							$av = utf8_decode($av);
						}
					}
					$sv = we_serialize($usv, SERIALIZE_JSON);
					return $sv;
				}

				return ($this->options['xml_encoding'] === 'ISO-8859-1' ?
						utf8_encode($value) :
						utf8_decode($value));
			}
		}
		return $value;
	}

	function refreshOwners(object &$object): void{
		if(isset($object->CreatorID) && ($this->options['handle_owners'] || $this->options['owners_overwrite'])){
			$userid = $object->CreatorID;
			if($this->options['handle_owners']){
				$userid = $this->RefTable->getNewOwnerID($userid);
				if($userid == 0 && $this->options['owners_overwrite'] && $this->options['owners_overwrite_id']){
					$userid = $this->options['owners_overwrite_id'];
				}
			} else if($this->options['owners_overwrite'] && $this->options['owners_overwrite_id']){
				$userid = $this->options['owners_overwrite_id'];
			} else {
				$userid = 0;
			}
			$object->CreatorID = $userid;
			if(isset($object->ModifierID)){
				$object->ModifierID = $userid;
			}
		} else {
			if(isset($object->CreatorID)){
				$object->CreatorID = 0;
			}
			if(isset($object->ModifierID)){
				$object->ModifierID = 0;
			}
		}

		if(isset($object->Owners) && ($this->options['handle_owners'] || $this->options['owners_overwrite'])){
			$owners = we_base_request::filterIntList($object->Owners);
			$newowners = [];
			foreach($owners as $owner){
				'@phan-var int $owner';
				if($this->options['handle_owners']){
					$own = $this->RefTable->getNewOwnerID($owner);
					if($own == 0 && $this->options['owners_overwrite'] && $this->options['owners_overwrite_id']){
						$own = $this->options['owners_overwrite_id'];
					}
				} else if($this->options['owners_overwrite'] && $this->options['owners_overwrite_id']){
					$own = $this->options['owners_overwrite_id'];
				}
				if(!empty($own) && !in_array($own, $newowners)){
					if(!$object->CreatorID){
						$object->CreatorID = $own;
					}
					if(!$object->ModifierID){
						$object->ModifierID = $own;
					}
					$newowners[] = $own;
				}
			}
			$object->Owners = $newowners;
			if(isset($object->OwnersReadOnly)){
				$readonly = $object->OwnersReadOnly;
				$readonly_new = [];
				if(is_array($readonly)){
					foreach($readonly as $key => $value){
						$newkey = 0;
						if($this->options['handle_owners']){
							$newkey = $this->RefTable->getNewOwnerID($key);
							if($newkey == 0 && $this->options['owners_overwrite'] && $this->options['owners_overwrite_id']){
								$newkey = $this->options['owners_overwrite_id'];
							}
						} else if($this->options['owners_overwrite'] && $this->options['owners_overwrite_id']){
							$newkey = $this->options['owners_overwrite_id'];
						}
						if($newkey){
							$readonly_new[$newkey] = $value;
						}
					}
					$object->OwnersReadOnly = $readonly_new;
				}
			}
		} else {
			if(isset($object->Owners)){
				$object->Owners = [];
			}
			if(isset($object->RestrictOwners)){
				$object->RestrictOwners = false;
			}
			if(isset($object->OwnersReadOnly)){
				$object->OwnersReadOnly = [];
			}
		}
	}

	public static function splitFile(string $filename, string $tmppath, int $count): int{//FIXME:remove
		if(!$filename){
			return -1;
		}

		$path = $tmppath;
		$pattern = basename($filename) . "_%s";

		$head = we_base_file::loadPart($filename, 0, 256);

		$encoding = we_xml_parser::getEncoding('', $head);
		$_SESSION['weS']['weXMLimportCharset'] = $encoding;
		$header = ''; //weXMLExIm::getHeader($encoding);
		$footer = we_exim_ExIm::getFooter();

		$buff = '';
		$fh = fopen($filename, 'rb');

		$num = -1;
		$elnum = $fh_temp = 0;

		$marker_size = strlen(we_backup_util::backupMarker);

		if(!$fh){
			return -1;
		}
		$line = '';
		'@phan-var string $line';
		while(!feof($fh)){
			$line = '';
			$findline = false;

			while($findline == false && !feof($fh)){
				$line .= fgets($fh, 4096);
				if(substr($line, -1) === "\n"){
					$findline = true;
				}
			}

			if(!$fh_temp && $line && trim($line) != we_backup_util::weXmlExImFooter){
				$num++;
				$filename_tmp = sprintf($path . $pattern, $num);
				$fh_temp = fopen($filename_tmp, 'wb');
				if(!$fh_temp){
					return -1;
				}
				if($header){
					fwrite($fh_temp, $header);
				}
			}

			if($fh_temp){
				if((substr($line, 0, 2) != "<?") && (substr($line, 0, 11) != we_backup_util::weXmlExImHead) && (substr($line, 0, 12) != we_backup_util::weXmlExImFooter)){

					$buff .= $line;
					$write = (
							(substr($buff, (0 - ($marker_size + 1))) == we_backup_util::backupMarker . "\n") ||
							(substr($buff, (0 - ($marker_size + 2))) == we_backup_util::backupMarker . "\r\n")
							);

					if($write){
						//$fsize += strlen($buff);
						fwrite($fh_temp, $buff);
						$elnum++;
						if($elnum >= $count){
							$elnum = 0;
							fwrite($fh_temp, $footer);
							fclose($fh_temp);
							$fh_temp = 0;
						}
						//$fsize = 0;
						$buff = '';
					}
				} else {
					if(((substr($line, 0, 2) === '<?') || (substr($line, 0, 11) == we_backup_util::weXmlExImHead)) && $num == 0){
						$header .= $line;
						fwrite($fh_temp, $line);
					}
				}
			}
		}

		if($fh_temp && trim($line) != we_backup_util::weXmlExImFooter){
			if($buff){
				fwrite($fh_temp, $buff);
			}
			fwrite($fh_temp, $footer);
			fclose($fh_temp);
		}
		fclose($fh);

		return $num + 1;
	}

	private function handleTag(string $tag): bool{
		switch($tag){
			case we_document_base::BACKUP_XML_TAG:
				return boolval($this->options['handle_documents']);
			case boolval(we_contents_template::BACKUP_XML_TAG):
				return boolval($this->options['handle_templates']);
			case we_contents_object::BACKUP_XML_TAG:
				return boolval($this->options['handle_classes']);
			case we_contents_objectFile::BACKUP_XML_TAG:
				return boolval($this->options['handle_objects']);
			case we_doctype_docTypes::BACKUP_XML_TAG:
				return boolval($this->options['handle_doctypes']);
			case we_category_category::BACKUP_XML_TAG:
				return boolval($this->options['handle_categorys']);
			case we_exim_element::BACKUP_XML_TAG:
				return boolval($this->options['handle_content']);
			case we_backup_dbTable::BACKUP_XML_TAG:
				return boolval($this->options['handle_table']);
			case we_backup_tableItem::BACKUP_XML_TAG:
				return boolval($this->options['handle_tableitems']);
			case we_backup_binary::BACKUP_XML_TAG:
				return boolval($this->options['handle_binarys']);
			case we_navigation_navigation::BACKUP_XML_TAG:
				return boolval($this->options['handle_navigation']);
			case we_navigation_rule::BACKUP_XML_TAG:
				return boolval($this->options['handle_navigation']);
			case we_contents_thumbnail::BACKUP_XML_TAG:
				return boolval($this->options['handle_thumbnails']);
			case self::INFO_TAG://internal
			case self::MAP_TAG:
				return false;
			default:
				return true;
		}
	}

	/**
	 * This function creates the given path in the repository and returns the id of the last created folder
	 *
	 * @param          string				$path
	 * @param          string				$table
	 * @param          array				$pathids
	 *
	 * @return         string
	 */
	private static function makePath(string $path, string $table, array &$pathids, int $owner = 0): int{
		$path = str_replace('\\', '/', $path);
		$patharr = explode('/', $path);
		$mkpath = '';
		$pid = 0;
		foreach($patharr as $elem){
			if($elem != '' && $elem != '/'){
				$mkpath .= '/' . $elem;
				$id = path_to_id($mkpath, $table);
				if(!$id){
					$new = we_folder_base::instance($table);
					$new->we_new($table, $pid, $elem);
					$new->CreatorID = $owner;
					$new->ModifierID = $owner;
					$new->Owners = [$owner];
					$new->OwnersReadOnly = [$owner => 0];
					$new->we_save();
					$id = $new->ID;
					$pathids[] = $id;
				}
				$pid = $id;
			}
		}

		return $pid;
	}

}
