<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 4f156532507d $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-05-06 23:12 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_glossary_search{
	/** @var object Database Object DB_WE */
	var $db = null;

	/** @var string Tablename */
	var $Table = '';

	/** @var string[] fields which have to be selected */
	var $Fields = [];

	/** @var string where clause */
	var $Where = '';

	/** @var string group by clause	 */
	var $GroupBy = '';

	/** @var string having clause	 */
	var $Having = '';

	/** @var string order clause	 */
	var $Order = '';

	/** @var string	 */
	var $Sort = '';

	/** @var int offset of the queryresult */
	var $Offset = 0;

	/** @var int Rows of the result */
	var $Rows = 10;

	function __construct(string $table){
		$this->db = new DB_WE();
		$this->Table = $table;
	}

	/**
	 * set the fields wich have to be selected
	 *
	 * @param string[] $fields
	 */
	function setFields(array $fields = []): void{
		$this->Fields = $fields;
	}

	/**
	 * set the where clause
	 *
	 * @param string $where
	 */
	function setWhere(string $where = ''): void{
		$this->Where = $where;
	}

	/**
	 * set the group by clause
	 *
	 * @param string $groupBy
	 */
	function setGroupBy(string $groupBy = ''): void{
		$this->GroupBy = $groupBy;
	}

	/**
	 * set the where clause
	 *
	 * @param string $having
	 */
	function setHaving(string $having = ''): void{
		$this->Having = $having;
	}

	/**
	 * set the order clause
	 *
	 * @param string $order
	 */
	function setOrder(string $order = '', string $sort = 'ASC'): void{
		$this->Order = $order;
		$this->Sort = $sort;
	}

	/**
	 * set the offset and the count
	 *
	 * @param integer $offset
	 * @param integer $rows
	 */
	function setLimit(int $offset = 0, int $rows = 10): void{
		$this->Offset = $offset;
		$this->Rows = $rows;
	}

	/**
	 * get statement
	 *
	 * @param boolean $countStmt
	 * @return string
	 */
	function _getStatement(bool $countStmt = false): string{
		return 'SELECT ' .
			($countStmt ? 'COUNT(1)' : implode(', ', $this->Fields)) . '
FROM ' .
			DB_WE::$inst->escape($this->Table) . '
WHERE ' . ($this->Where ?: '1') .
			($this->GroupBy ? ' GROUP BY ' . $this->GroupBy : '') .
			($this->Having ? ' HAVING ' . $this->Having : '') .
			($countStmt ? '' :
			($this->Order ? ' ORDER BY ' . $this->Order . ' ' . $this->Sort : '') .
			' LIMIT ' . $this->Offset . ', ' . $this->Rows
			);
	}

	/**
	 * count the items
	 *
	 * @return integer
	 */
	function countItems(): int{
		return fI($this->_getStatement(true));
	}

	/**
	 * execute the saerch query
	 *
	 */
	function execute(): we_rpc_response{
		$this->db->query($this->_getStatement());
		return new we_rpc_response();
	}

	/**
	 * iterate over the whole resultset
	 *
	 * @return mixed
	 */
	function next(): bool{
		return $this->db->next_record();
	}

	/**
	 * get the value of a field
	 *
	 * @param string $field
	 * @return mixed
	 */
	function getField(string $field): string{
		return (string) $this->db->f($field);
	}

	/**
	 * get the pages as array (key = pageNr, value = start)
	 *
	 * @return array
	 */
	function getPages(): array{
		$count = $this->countItems();
		$numPages = ceil($count / $this->Rows);

		$pages = [];
		for($i = 1; $i <= $numPages; $i++){
			$pages[($i - 1) * $this->Rows] = $i;
		}

		return $pages;
	}

	/**
	 * get the number of the active page
	 *
	 * @return integer
	 */
	function getActivePage(): int{
		return intval(ceil(($this->Offset - 1) / $this->Rows));
	}

}
