<?php

declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 6917b9add631 $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-04-21 13:24 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

/**
 * Klasse f�r Autocomleter
 *
 * $weSuggest =& weSuggest::getInstance();																								// Die Kalsse instanzieren.
 * echo $weSuggest->createAutocompleter(																								// GUI-Element mit Input-Feld und Auswahl-Button
 * 			"Doc", 																														// AC-Id
 * 			we_button::create_button(we_html_button::SELECT, "javascript:select_seem_start()", , '', false, false),					// Auswahl-Button
 * 			we_html_tools::textInput("seem_start_document_name", $document_path, '', " id='yuiAcInputDoc'", "text", 190, 0, '', false),		// Input-Feld
 * 			'yuiAcInputDoc',																											// Input-Feld-Id. Die Id besteht aus 'yuiAcInput' und AC-Id
 * 			rray('name' => "seem_start_document", 'value' => $document_id, "id"=>"yuiAcResultDoc")),		// Result-Field (hidden) für die Document-, Folder-, Object-,...ID
 * 			'yuiAcResultDoc', 																											// Result-Feld-Id. Die Id besteht aus 'yuiAcResult' und AC-Id
 * 			'',																															// Label: steht über dem Inputfeld
 * 			FILE_TABLE, 																												// Name der Tabele in für die Query
 * 			"folder,text/webedition,image/*,text/js,text/css,text/html,application/*", 													// ContentTypen für die Query: sie entsprechende Tabele
 * 			"docSelector", 																												// docSelector | dirSelector : ob nach folder oder doc gesucht wird
 * 			20, 																														// Anzahl der Vorschläge
 * 			0, 																															// Verzögerung für das auslösen des AutoCompletion
 * 			true, 																														// Soll eine Ergebnisüberprüfung stattfinden
 * 			"190", 																														// Container-Breite
 * 			"true",																														// Feld darf leer bleiben
 * 			10																															// Abstand zwischen Input-Feld und Button
 * 		);
 */
class we_gui_suggest{

	/** @var string */
	public const CustomerSelector = 'customerSelector';

	/** @var string */
	public const DirSelector = 'dirSelector';

	/** @var string */
	public const DocSelector = 'docSelector';
	private const BTN_EDIT = true;
	private const BTN_SELECT = 0;
	private const BTN_ADDITIONAL = 1;
	private const BTN_TRASH = 2;
	private const BTN_OPEN = 3;
	private const BTN_CREATE = 4;

	/** @var bool */
	private $noautoinit = false;

	/** @var string */
	private $acId = '';

	/** @var bool */
	private $checkFieldValue = true;

	/** @var string */
	private $contentType = we_base_ContentTypes::FOLDER_DOCUMENT;

	/** @var array<string,string> */
	private $inputAttribs = [];

	/** @var string */
	private $inputId = '';

	/** @var string */
	private $inputName = '';

	/** @var string */
	private $inputValue = '';

	/** @var string */
	private $label = '';

	/** @var int */
	private $maxResults = 20;

	/** @var bool */
	private $required = false;

	/** @var string */
	private $resultName = '';

	/** @var string */
	private $resultValue = '';

	/** @var string */
	private $resultId = '';

	/** @var string */
	private $rootDir = '';

	/** @var string */
	private $selector = self::DirSelector;

	/** @var string[] */
	private $buttons = [
		self::BTN_SELECT => '',
		self::BTN_ADDITIONAL => '',
		self::BTN_TRASH => '',
		self::BTN_OPEN => '',
		self::BTN_CREATE => '',
	];

	/** @var string */
	private $table = FILE_TABLE;

	/** @var int */
	private $width = 0;

	/** @var string */
	private $jsCommandOnItemSelect = '';

	/** @var bool */
	private $isDropFromTree = false;

	/** @var bool */
	private $isDropFromExt = false;

	/** @var string */
	private $doOnDropFromTree = '';

	/** @var string */
	private $doOnDropFromExt = '';

	/** @var bool */
	private static $giveStatic = true;

	public static function getInstance(): self{
		static $inst = null;
		if(!is_object($inst)){
			$inst = new self();
		}
		if(self::$giveStatic){
			return $inst;
		}
		$void = new self();
		return $void;
	}

	public function getHTML(bool $reset = true): string{
		$inputId = $this->inputId ?: 'yuiAcInput' . $this->acId;
		$resultId = $this->resultId ?: 'yuiAcResult' . $this->acId;

		if($this->buttons[self::BTN_OPEN] === self::BTN_EDIT){
			$this->buttons[self::BTN_OPEN] = we_html_button::create_button(we_html_button::OPEN_TO_EDIT, "javascript:WE().layout.weSuggest.openSelectionToEdit(window,'" . $inputId . "')");
		}

		if(!$this->noautoinit){
			$this->inputAttribs['class'] .= ' weSuggest';
		}
		$this->inputAttribs['data-table'] = $this->table;
		$this->inputAttribs['data-contenttype'] = $this->contentType;
		$this->inputAttribs['data-basedir'] = ($this->rootDir ?: '/');
		$this->inputAttribs['data-result'] = $resultId;
		$this->inputAttribs['data-max'] = (string) $this->maxResults;
		$this->inputAttribs['data-selector'] = $this->selector;
		$this->inputAttribs['data-currentDocumentType'] = we_base_WE::$we_doc ? we_base_WE::$we_doc->ContentType : '';
		$this->inputAttribs['data-currentDocumentID'] = (string) (we_base_WE::$we_doc ? we_base_WE::$we_doc->ID : 0);
		$this->inputAttribs['data-onSelect'] = $this->jsCommandOnItemSelect;

		if($this->required){
			$this->inputAttribs['required'] = 'required';
		}

		$html = we_html_tools::formElementTable($this->label, [
					'text' => $this->htmlTextInput($this->inputName, $this->inputValue, $this->inputAttribs, 'text', $this->width) .
					we_html_element::hidden($this->resultName, $this->resultValue, $resultId, [
						//FIXME: we need to know this
						'data-contenttype' => '',
					])
						], we_html_element::span(['class'=>'buttons'],implode('',[$this->buttons[self::BTN_SELECT], $this->buttons[self::BTN_OPEN], $this->buttons[self::BTN_TRASH], $this->buttons[self::BTN_ADDITIONAL]
						, $this->buttons[self::BTN_CREATE]]))
		);
		$file = [];

		if(($this->isDropFromTree || $this->isDropFromExt)){
			$this->isDropFromExt = $this->table === FILE_TABLE ? $this->isDropFromExt : false;

			$dropzoneContent = g_l('global', '[dragndrop][selection]') . '<br/>' . g_l('global', '[dragndrop][dnd_text][' . (($this->isDropFromTree ? 1 : 0) + ($this->isDropFromExt ? 2 : 0)) . ']');
			$dropzoneStyle = 'width:auto;padding-top:14px;height:60px;';

			$img = '';
			$eventAttribs = ['ondragover' => 'handleDragOver(event, \'' . $this->acId . '\');', 'ondragleave' => 'handleDragLeave(event, \'' . $this->acId . '\');'];

			if($this->contentType === we_base_ContentTypes::IMAGE){ // FIXME: add code for icons so we can have preview for all cts
				if($this->resultValue){
					$DE_WE = new DB_WE;
					$file = $DE_WE->getHash('SELECT Path,Extension,ContentType FROM ' . FILE_TABLE . ' WHERE ID=' . $this->resultValue);

					if($file['ContentType'] === we_base_ContentTypes::IMAGE){
						$url = WEBEDITION_DIR . 'we_cmd.php?we_cmd[0]=we_base_showThumbnail&id=' . $this->resultValue . '&size[width]=100&size[heihjt]=100&path=' . urlencode($file['Path']) . '&extension=' . $file['Extension'];
						$img = we_html_element::img(['src' => $url, 'style' => 'vertical-align:middle;']);
					}
				}

				$imgDiv = we_html_element::div(array_merge($eventAttribs, ['style' => 'float:left;height:100%;']), we_html_element::span(['style' => 'display:inline-block;height: 100%;vertical-align: middle;']) .
								we_html_element::span(['id' => 'preview_' . $this->acId], $img)
				);
				$dropzoneContent = $imgDiv . we_html_element::div(array_merge($eventAttribs, ['class' => 'textExtDropzone we_suggest']), $dropzoneContent);
				$dropzoneStyle = 'width:auto;padding:0 0 0 12px;';
			}
			$dropzone = we_fileupload_ui_base::getExternalDropZone($this->acId, $dropzoneContent, $dropzoneStyle, $this->isDropFromTree, $this->isDropFromExt, 'we_suggest_processExternalSelection,' . $this->acId, 'we_suggest_processExternalSelection,' . $this->acId, explode(',', $this->contentType), $this->table);

			$html = we_html_element::div([], we_html_element::div([], $html) .
							we_html_element::div(['style' => 'margin-top:-4px;'], $dropzone)
			);
			$this->isDropFromTree = $this->isDropFromExt = false; //reset default for other instances on the same site
		}

		$html .= we_html_element::hidden('yuiAcContentType' . $this->acId, $file['ContentType'] ?? '', 'yuiAcContentType' . $this->acId);

		if($reset){
			$this->reset();
		}
		$this->acId = '';
		$this->maxResults = 20;
		$this->resultName = '';
		$this->resultValue = '';
		$this->resultId = '';
		$this->buttons = [
			self::BTN_SELECT => '',
			self::BTN_ADDITIONAL => '',
			self::BTN_TRASH => '',
			self::BTN_OPEN => '',
			self::BTN_CREATE => '',
		];
		return $html;
	}

	public function reset(): void{
		$this->contentType = we_base_ContentTypes::FOLDER_DOCUMENT;
		$this->required = false;
		$this->label = '';
		$this->selector = self::DirSelector;
		$this->table = FILE_TABLE;
		$this->width = 0;
		$this->jsCommandOnItemSelect = '';
	}

	function getInputId(): string{
		return $this->inputId;
	}

	/**
	 * Set id and value for the input field
	 *
	 * @param String $name
	 * @param String $value
	 * @param array<string,string> $attribs
	 * @param Boolean $disabled
	 */
	public function setInput(string $name, string $value = '', array $attribs = [], bool $disabled = false, bool $markHot = false): self{
		$this->inputId = '';
		$this->inputName = $name;
		$this->inputValue = $value;
		$this->inputAttribs = [
			'class' => 'wetextinput',
			'onchange' => ($markHot ? 'hot=true;we_cmd(\'setHot\')' : ''),
		];
		if($disabled){
			$this->inputAttribs['disabled'] = 'disabled';
		}
		foreach($attribs as $key => $val){
			$key = strtolower($key);
			switch($key){
				case 'id':
					$this->inputId = $key;
					$this->inputAttribs[$key] = $val;
					break;
				case 'onchange':
					$this->inputAttribs[$key] .= $val;
					break;
				case 'class':
					$this->inputAttribs[$key] .= ' ' . $val;
//				case 'onfocus':
				default:
					$this->inputAttribs[$key] = $val;
			}
		}

		if(!$this->inputId){
			$this->setInputId();
		}
		return $this;
	}

	private function htmlTextInput(string $name, string $value = '', array $attribs = [], string $type = 'text', /* mixed */ $width = 0): string{
		$attribs['type'] = $type;
		$attribs['name'] = trim($name);
		$attribs['value'] = htmlspecialchars($value);
		$attribs['style'] = (empty($attribs['style']) ? '' : $attribs['style']) .
				($width ? ('width: ' . $width . (is_string($width) && (strpos($width, 'px') || strpos($width, '%')) ? '' : 'px') . ';') : '');

		return we_html_element::input($attribs);
	}

	//setter

	public function setAcId(string $val, string $rootDir = ''): self{
		$this->acId = str_replace('-', '_', $val);
		$this->rootDir = $rootDir;
		return $this;
	}

	public function setNoAutoInit(bool $noautoinit = false): self{
		$this->noautoinit = $noautoinit;
		return $this;
	}

	/**
	 * Set the content tye to filter result
	 *
	 * @param string[] $val
	 */
	public function setContentType(array $val): self{
		$this->contentType = implode(',', $val);
		return $this;
	}

	/* public function setDocType(string $type):void{
	  //FIXME: implement
	  } */

	/* if some other item is selected this we-cmd is called */

	public function setJsCommandOnItemSelect(string $val): self{
		$this->jsCommandOnItemSelect = $val;
		return $this;
	}

	public function setDoOnDropFromExt(string $val = ''): self{
		$this->doOnDropFromExt = $val;
		return $this;
	}

	public function setDoOnDropFromTree(string $val = ''): self{
		$this->doOnDropFromTree = $val;
		return $this;
	}

	public function setIsDropFromExt(bool $val = false): self{
		$this->isDropFromExt = $val;
		return $this;
	}

	public function setIsDropFromTree(bool $val = false): self{
		$this->isDropFromTree = $val;
		return $this;
	}

	private function setInputId(string $val = ''): self{
		$this->inputId = ($val ?: 'yuiAcInput' . $this->acId);
		$this->inputAttribs['id'] = $this->inputId;
		return $this;
	}

	public function setMaxResults(int $val): self{
		$this->maxResults = $val;
		return $this;
	}

	public function setCheckFieldValue(bool $val): self{
		$this->checkFieldValue = $val;
		return $this;
	}

	/**
	 * Flag if the autocompleter my be empty
	 *
	 * @param boolean $val
	 */
	public function setRequired(bool $val): self{
		$this->required = $val;
		return $this;
	}

	public function setLabel(string $val): self{
		$this->label = $val;
		return $this;
	}

	/**
	 * Set name, value and id for the result field
	 * @param string $resultName
	 * @param string $resultValue
	 * @param string $resultID
	 */
	public function setResult(string $resultName, string $resultValue = '', string $resultID = ''): self{
		$this->resultName = $resultName;
		$this->resultId = $resultID;
		$this->resultValue = $resultValue;
		return $this;
	}

	/**
	 * Set the selector
	 *
	 * @param String $val
	 */
	public function setSelector(string $val): self{
		$this->selector = $val;
		return $this;
	}

	/**
	 * Set the table for query result
	 *
	 * @param string $val
	 */
	public function setTable(string $val): self{
		$this->table = $val;
		return $this;
	}

	public function setSelectButton(string $val): self{
		$this->buttons[self::BTN_SELECT] = $val;
		return $this;
	}

	public function setTrashButton(string $val): self{
		$this->buttons[self::BTN_TRASH] = $val;
		return $this;
	}

	public function setOpenButton(string $val): self{
		$this->buttons[self::BTN_OPEN] = $val;
		return $this;
	}

	public function setAdditionalButton(string $val): self{
		$this->buttons[self::BTN_ADDITIONAL] = $val;
		return $this;
	}

	public function setCreateButton(string $val): self{
		$this->buttons[self::BTN_CREATE] = $val;
		return $this;
	}

	public function setWidth(int $var): self{
		$this->width = $var;
		return $this;
	}

	/**
	 * This function sets the values for the autocompletion fields
	 *
	 * @param unknown_type $inputFieldId
	 * @param unknown_type $containerFieldId
	 * @param unknown_type $table
	 * @param unknown_type $contentType
	 * @param unknown_type $maxResults
	 * @param unknown_type $queryDelay
	 * @param unknown_type $layerId
	 * @param unknown_type $setOnSelectFields
	 * @param unknown_type $checkFieldsValue
	 * @param unknown_type $containerwidth
	 */

	/**
	 * needed to suppress giving the same instance
	 * If sth. is included & the main instance should not be modified, set this to false
	 * @param bool $staticInstance false, if the results should be omitted; don't forget to reset
	 */
	public static function setStaticInstance(bool $staticInstance): void{
		self::$giveStatic = $staticInstance;
	}

}
