<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 53a08b3d6828 $
 * $Author: Marc Krämer <marc.kraemer@aldente-it.de> $
 * $Date: 2019-09-09 17:57 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
abstract class we_gui_tagWizzard{

	public static function getHTML(): string{
		$tagName = we_base_request::string('we_cmd', '', 1);
		$openAtCursor = we_base_request::bool('we_cmd', false, 2);

// include wetag depending on we_cmd[1]
		try{
			$weTag = we_weTag_twbase::getTagData($tagName);
		} catch (we_exception_file $e){
			return sprintf(g_l('taged', '[tag_not_found]'), $tagName);
		}

		$tw = [
			'tagName' => $weTag->getName(),
			'needsEndTag' => $weTag->needsEndTag(),
			'openAtCursor' => $openAtCursor,
			'reqAttributes' => [],
			'typeAttributeId' => '',
			'typeAttributeAllows' => [],
			'typeAttributeRequires' => [],
		];
// needed javascript for the individual tags
// #1 - all attributes of this we:tag (ids of attributes)
		$tw['attributes'] = $weTag->getAllAttributes(true);

// #2 all required attributes
		$reqAttributes = $weTag->getRequiredAttributes();
		foreach($reqAttributes as $attribName){
			$tw['reqAttributes'][$attribName] = 1;
		}

// #3 all neccessary stuff for typeAttribute
		if(($typeAttribute = $weTag->getTypeAttribute())){
			// name of the attribute
			$tw['typeAttributeId'] = $typeAttribute->getIdName();

			// allowed attributes
			$typeOptions = $weTag->getTypeAttributeOptions();

			if($typeOptions){
				foreach($typeOptions as $option){
					$allowedAttribs = $option->getAllowedAttributes();
					$tw['typeAttributeAllows'][$option->getName()] = (empty($allowedAttribs) ? [] : $allowedAttribs);

					$reqAttribs = $option->getRequiredAttributes($tw['attributes']);
					$tw['typeAttributeRequires'][$option->getName()] = (empty($reqAttribs) ? [] : $reqAttribs);
				}
			}
		}

// get all attributes
		$typeAttribCode = $weTag->getTypeAttributeCodeForTagWizard();
		$attributesCode = $weTag->getAttributesCodeForTagWizard();
		$defaultValueCode = ($weTag->needsEndTag() ? $weTag->getDefaultValueCodeForTagWizard() : '');

// additional javascript for the individual tags - end
// print html header of page
		return we_html_tools::getHtmlTop('', we_html_element::cssLink(CSS_DIR . 'tagWizard.css') .
				we_html_element::jsScript(JS_DIR . 'tagWizard.js', '', ['id' => 'loadVarTagWizard', 'data-tw' => setDynamicVar($tw)]), we_html_element::body(['onload' => 'self.focus();',
					'class' => 'defaultfont tagWizzard'], we_html_element::form(['name' => we_base_constants::WE_FORM, 'onsubmit' => "we_cmd('saveTag');return false;"], we_html_element::div([
							'id' => 'divTagName'],
																																																																																							 we_html_element::h(1, '&lt;we:' . $weTag->getName() . '&gt;' . ($weTag->isDeprecated() ? ' (' . g_l('taged', '[deprecated][title]') . ')' : ''))
						) . '
		<div id="divContent">
		<fieldset>' .
						//<div class="legend"><strong>' . g_l('taged', '[description]') . '</strong></div>' .
						($weTag->isDeprecated() ? we_html_tools::alertAttentionBox(g_l('taged', '[deprecated][description]'), we_html_tools::TYPE_ALERT, '98%') : '') . $weTag->getDescription() .
						'</fieldset>' . ($typeAttribCode ?
						'<hr /><fieldset>
		<div class="legend"><strong>' . g_l('taged', '[type_attribute]') . '</strong></div>' .
						$typeAttribCode .
						'</fieldset>' : '') . ' ' . ($attributesCode ?
						'<hr/><fieldset>
		<div class="legend"><strong>' . g_l('taged', '[attributes]') . '</strong></div>
		' . ($typeAttribCode ? '<ul id="no_type_selected_attributes"><li><table class="attribute"><tr><td class="attributeName"></td><td class="attributeField">' . g_l('taged', '[no_type_selected]') . '</td></tr></table></li></ul>' : '' ) . '
		' . ($typeAttribCode ? '<ul id="no_attributes_for_type" style="display: none;"><li><table class="attribute"><tr><td class="attributeName"></td><td class="attributeField">' . g_l('taged', '[no_attributes_for_type]') . '</td></tr></table></li></ul>' : '' ) .
						$attributesCode .
						'</fieldset>' : '') . ' ' .
						($defaultValueCode ?
						'<hr/><fieldset>
		<div class="legend"><strong>' . we_html_element::label(['id' => 'label_weTagData_defaultValue', 'for' => 'weTagData_defaultValue'], g_l('taged', '[defaultvalue]') . ':<br/>') . '</strong></div>' .
						$defaultValueCode .
						'</fieldset>' : '') . '
			<br/>
		</div>
		<div id="divButtons">
			<div style="padding-top: 8px;">' .
						we_html_button::position_yes_no_cancel(we_html_button::create_button(we_html_button::OK, "javascript:we_cmd('saveTag');"), '', we_html_button::create_button(we_html_button::CANCEL, "javascript:WE().util.dialogTop(window).doClose();") . '
			</div>
		</div>
		<input type="submit" style="width:1px; height:1px; padding:0; margin:0; color:#fff; background-color:#fff; border:0;" />'
						)
		)));
	}

}
