<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: aa01842ef312 $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-04-15 10:54 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

/**
 * class    we_customer_listview
 * @desc    class for tag <we:listview type="customer">
 *
 */
class we_listview_customer extends we_listview_base{
	/** @var string */
	var $Path = '';

	/** @var int */
	public $docID = 0;

	/** @var bool */
	var $hidedirindex = false;

	/**
	 * @desc    constructor of class
	 *
	 * @param   $name          string - name of listview
	 * @param   $rows          integer - number of rows to display per page
	 * @param   $order         string - field name(s) to order by
	 * @param   $desc		   string - if desc order
	 * @param   $condition	   string - condition of listview
	 * @param   $cols		   string - number of cols (default = 1)
	 * @param   $docID	   	   string - id of a document where a we:customer tag is on
	 *
	 */
	function __construct(string $name, int $rows, int $offset, string $order, bool $desc, string $condition, int $cols, int $docID, bool $hidedirindex){
		parent::__construct($name, $rows, $offset, $order, $desc, [], false, [], $cols);

		$this->docID = $docID;
		$this->condition = $condition;
		$this->hidedirindex = $hidedirindex;

		$this->Path = ($this->docID ?
			id2path($this->docID, FILE_TABLE) :
			we_base_WE::$we_doc->Path
			);


		// IMPORTANT for seeMode !!!! #5317
		$this->LastDocPath = (isset($_SESSION['weS']['last_webEdition_document']) ? $_SESSION['weS']['last_webEdition_document']['Path'] : '');

		if($this->desc && $this->order != '' && (!preg_match('|.+ desc$|i', $this->order))){
			$this->order .= ' DESC';
		}

		$orderstring = $extra = '';
		if($this->order === self::RANDOM_FIELD){
			$extra = ', RAND() AS RANDOM';
			$orderstring = ' ORDER BY RANDOM';
		} else if($this->order != ''){
			$orderstring = ' ORDER BY ' . $this->order;
		}

		$where = $this->condition ? (' WHERE ' . $this->condition) : '';
//FIXME use window function COUNT(*) OVER() if mariadb 10.2/mysql 8
		$this->anz_all = fI('SELECT COUNT(1) FROM ' . CUSTOMER_TABLE . $where);

		$this->DB_WE->query('SELECT * ' . $extra . ' FROM ' . CUSTOMER_TABLE . $where . ' ' . $orderstring . ' ' . (($this->maxItemsPerPage > 0) ? (' LIMIT ' . $this->start . ',' . $this->maxItemsPerPage) : ''));
		$this->anz = $this->DB_WE->num_rows();
	}

	public function next_record(): bool{
		$ret = $this->DB_WE->next_record();
		if($ret){
			$this->Record = array_change_key_case($this->DB_WE->getRecord());
			$this->Record[self::PATH_FIELD] = $this->Path . '?we_cid=' . $this->Record['id'];
			$this->Record[self::TEXT_FIELD] = $this->Record['username'];
			$this->Record[self::ID_FIELD] = $this->Record['id'];
			$this->Record[self::LASTPATH_FIELD] = $this->LastDocPath . '?we_cid=' . $this->Record['id'];
			$this->count++;
			return true;
		}
		$this->stop_next_row = $this->shouldPrintEndTR();
		if($this->cols && ($this->count <= $this->maxItemsPerPage) && !$this->stop_next_row){
			$this->Record = [];
			$this->count++;
			return true;
		}

		return false;
	}

	function f(string $key, bool $stripWE = false): string{
		return parent::f($key, $stripWE);
	}

}
