<?php
declare(strict_types=1);

/** webEdition CMS
 *
 * $Rev: 08f2bc05ecec $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-05-19 12:15 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
abstract class we_modules_info{
	public const BANNER = 'banner';
	public const CATEGORY = 'category';
	public const COLLECTION = 'collection';
	//const CORE = 'core';
	public const CORE = 'core';
	public const CUSTOMER = 'customer';
	public const DOCTYPE = 'doctype';
	public const EDITOR = 'editor';
	public const EXPORT = 'export';
	public const GLOSSARY = 'glossary';
	public const METAFIELDS = 'metafields';
	public const NAVIGATION = 'navigation';
	public const NEWSLETTER = 'newsletter';
	public const OBJECT = 'object';
	public const SCHEDULER = 'schedule';
	public const SEARCH = 'search';
	public const SHOP = 'shop';
	public const THUMB = 'thumbnail';
	public const USERS = 'users';
	public const VOTING = 'voting';
	public const WORKFLOW = 'workflow';
	//FIXME: move to const! check how to migrate langs
	/** @var array<string,array{name:string,perm:string[],icon:string,parentMenu:string,inModuleMenu:bool,alwaysActive:bool,hasSettings:bool,text:string[],text_short:string[],tables:array<string,string>}>	 */
	private const AVAILABLE_MODULES = [
		self::USERS => [
			'name' => self::USERS,
			'perm' => [we_base_perm::ADMINISTRATOR], //[we_base_perm::USER_NEW, we_base_perm::USER_GROUP_NEW, we_base_perm::USER_SAVE, we_base_perm::USER_GROUP_SAVE, we_base_perm::USER_DELETE, we_base_perm::USER_GROUP_DELETE],
			'icon' => 'fa-user',
			'parentMenu' => we_gui_mainMenu::E_SETTINGS,
			'inModuleMenu' => true,
			'alwaysActive' => true,
			'hasSettings' => false,
			'text' => ['menu_moduleInformation', '[users][text]'],
			'text_short' => ['menu_moduleInformation', '[users][text_short]'],
			//'dependson' => '',
			//'childmodule' => '',
			'tables' => ['USER_TABLE' => USER_TABLE]
		],
		self::CUSTOMER => [
			'name' => self::CUSTOMER,
			'perm' => [we_base_perm::CUSTOMER__SHOW_ADMIN, we_base_perm::CUSTOMER_DELETE, we_base_perm::CUSTOMER_EDIT, we_base_perm::CUSTOMER_NEW],
			'icon' => 'fa-users',
			'parentMenu' => we_gui_mainMenu::E_MODULE,
			'inModuleMenu' => true,
			'alwaysActive' => false,
			'hasSettings' => true,
			'text' => ['menu_moduleInformation', '[customer][text]'],
			'text_short' => ['menu_moduleInformation', '[customer][text_short]'],
			//'dependson' => '',
			'childmodule' => self::SHOP,
			'tables' => ['CUSTOMER_TABLE' => CUSTOMER_TABLE, 'CUSTOMER_FILTER_TABLE' => CUSTOMER_FILTER_TABLE, 'CUSTOMER_AUTOLOGIN_TABLE' => CUSTOMER_AUTOLOGIN_TABLE, 'CUSTOMER_SESSION_TABLE' => CUSTOMER_SESSION_TABLE],
		],
		self::NAVIGATION => [
			'name' => self::NAVIGATION,
			'perm' => [we_base_perm::NAVIGATION_EDIT],
			'text' => ['menu_global', '[navigation]'],
			'text_short' => ['menu_global', '[navigation]'],
			'icon' => 'fa-compass',
			'parentMenu' => we_gui_mainMenu::E_MODULE,
			'inModuleMenu' => true,
			'alwaysActive' => true,
			'hasSettings' => false,
			//'dependson' => '',
			//'childmodule' => '',
			'tables' => ['NAVIGATION_TABLE' => NAVIGATION_TABLE, 'NAVIGATION_RULE_TABLE' => NAVIGATION_RULE_TABLE],
		],
		self::COLLECTION => [
			'name' => self::COLLECTION,
			'perm' => [we_base_perm::WORK_COLLECTIONS_CAN_SEE, we_base_perm::WORK_COLLECTION_DELETE, /* we_base_perm::EDIT_COLLECTIONS, */ we_base_perm::WORK_COLLECTION_NEW,
				we_base_perm::WORK_COLLECTION_FOLDER_NEW],
			'text' => ['menu_moduleInformation', '[collection][text]'],
			'text_short' => ['menu_moduleInformation', '[collection][text_short]'],
			'icon' => '',
			'parentMenu' => '',
			'inModuleMenu' => false,
			'alwaysActive' => false,
			'hasSettings' => false,
			//'dependson' => '',
			//'childmodule' => '',
			'tables' => ['VFILE_TABLE' => VFILE_TABLE],
		],
		self::SHOP => [
			'name' => self::SHOP,
			'perm' => [we_base_perm::SHOP_NEW_ARTICLE, we_base_perm::SHOP_DELETE_ARTICLE, we_base_perm::SHOP_EDIT_ORDER, we_base_perm::SHOP_DELETE_ORDER, we_base_perm::SHOP_EDIT_PREFS],
			'text' => ['menu_moduleInformation', '[shop][text]'],
			'text_short' => ['menu_moduleInformation', '[shop][text_short]'],
			'icon' => 'fa-shopping-cart',
			'parentMenu' => we_gui_mainMenu::E_MODULE,
			'inModuleMenu' => true,
			'alwaysActive' => false,
			'hasSettings' => true,
			'dependson' => self::CUSTOMER,
			//'childmodule' => '',
			'tables' => ['WE_SHOP_VAT_TABLE' => WE_SHOP_VAT_TABLE, 'SHOP_ORDER_DATES_TABLE' => SHOP_ORDER_DATES_TABLE, 'SHOP_ORDER_DOCUMENT_TABLE' => SHOP_ORDER_DOCUMENT_TABLE,
				'SHOP_ORDER_ITEM_TABLE' => SHOP_ORDER_ITEM_TABLE, 'SHOP_ORDER_TABLE' => SHOP_ORDER_TABLE]
		],
		self::SCHEDULER => ['name' => self::SCHEDULER,
			'text' => ['menu_moduleInformation', '[schedule][text]'],
			'text_short' => ['menu_moduleInformation', '[schedule][text_short]'],
			'icon' => '',
			'parentMenu' => '',
			'inModuleMenu' => false,
			'alwaysActive' => false,
			'hasSettings' => false,
			//'dependson' => '',
			//'childmodule' => '',
			'tables' => ['SCHEDULE_TABLE' => SCHEDULE_TABLE],
		],
		self::EDITOR => [
			'name' => self::EDITOR,
			'text' => ['menu_moduleInformation', '[editor][text]'],
			'text_short' => ['menu_moduleInformation', '[editor][text_short]'],
			'icon' => '',
			'parentMenu' => '',
			'inModuleMenu' => false,
			'alwaysActive' => false,
			'hasSettings' => true,
		//'dependson' => '',
		//'childmodule' => '',
		//'tables' => []
		],
		self::OBJECT => [
			'name' => self::OBJECT,
			'text' => ['menu_moduleInformation', '[object][text]'],
			'text_short' => ['menu_moduleInformation', '[object][text_short]'],
			'icon' => '',
			'parentMenu' => '',
			'inModuleMenu' => false,
			'alwaysActive' => false,
			'hasSettings' => false,
			//'dependson' => '',
			//'childmodule' => '',
			'tables' => ['OBJECT_FILES_TABLE' => OBJECT_FILES_TABLE, 'OBJECT_TABLE' => OBJECT_TABLE]
		],
		self::WORKFLOW => [
			'name' => self::WORKFLOW,
			'perm' => [we_base_perm::WORKFLOW_NEW, we_base_perm::WORKFLOW_DELETE, we_base_perm::WORKFLOW_EDIT, we_base_perm::WORKFLOW_EMPTY_LOG],
			'text' => ['menu_moduleInformation', '[workflow][text]'],
			'text_short' => ['menu_moduleInformation', '[workflow][text_short]'],
			'icon' => 'fa-gears',
			'parentMenu' => we_gui_mainMenu::E_MODULE,
			'alwaysActive' => false,
			'inModuleMenu' => true,
			'hasSettings' => false,
			//'dependson' => '',
			//'childmodule' => '',
			'tables' => ['WORKFLOW_TABLE' => WORKFLOW_TABLE, 'WORKFLOW_DOC_TABLE' => WORKFLOW_DOC_TABLE, 'WORKFLOW_DOC_STEP_TABLE' => WORKFLOW_DOC_STEP_TABLE, 'WORKFLOW_DOC_TASK_TABLE' => WORKFLOW_DOC_TASK_TABLE,
				'WORKFLOW_LOG_TABLE' => WORKFLOW_LOG_TABLE, 'WORKFLOW_STEP_TABLE' => WORKFLOW_STEP_TABLE, 'WORKFLOW_TASK_TABLE' => WORKFLOW_TASK_TABLE]
		],
		self::NEWSLETTER => [
			'name' => self::NEWSLETTER,
			'perm' => [we_base_perm::NEWSLETTER_NEW, we_base_perm::NEWSLETTER_DELETE, we_base_perm::NEWSLETTER_EDIT, we_base_perm::NEWSLETTER_SEND, we_base_perm::NEWSLETTER_SEND_TEST_EMAIL],
			'text' => ['menu_moduleInformation', '[newsletter][text]'],
			'text_short' => ['menu_moduleInformation', '[newsletter][text_short]'],
			'icon' => 'fa-newspaper-o',
			'parentMenu' => we_gui_mainMenu::E_MODULE,
			'inModuleMenu' => true,
			'alwaysActive' => false,
			'hasSettings' => true,
			//'dependson' => '',
			//'childmodule' => '',
			'tables' => ['NEWSLETTER_BLOCK_TABLE' => NEWSLETTER_BLOCK_TABLE, 'NEWSLETTER_CONFIRM_TABLE' => NEWSLETTER_CONFIRM_TABLE, 'NEWSLETTER_GROUP_TABLE' => NEWSLETTER_GROUP_TABLE,
				'NEWSLETTER_LOG_TABLE' => NEWSLETTER_LOG_TABLE, 'NEWSLETTER_TABLE' => NEWSLETTER_TABLE]
		],
		self::EXPORT => [
			'name' => self::EXPORT,
			'perm' => [we_base_perm::EXIM_EXPORT_NEW, we_base_perm::EXIM_EXPORT_DELETE, we_base_perm::EXIM_EXPORT_EDIT, we_base_perm::EXIM_EXPORT_MAKE],
			'text' => ['menu_moduleInformation', '[export][text]'],
			'text_short' => ['menu_moduleInformation', '[export][text_short]'],
			'icon' => 'fa-download',
			'parentMenu' => we_gui_mainMenu::E_MODULE,
			'inModuleMenu' => true,
			'alwaysActive' => true,
			'hasSettings' => false,
			//'dependson' => '',
			//'childmodule' => '',
			'tables' => ['EXPORT_TABLE' => EXPORT_TABLE]
		],
		self::VOTING => [
			'name' => self::VOTING,
			'perm' => [we_base_perm::VOTING_NEW, we_base_perm::VOTING_DELETE, we_base_perm::VOTING_EDIT],
			'text' => ['menu_moduleInformation', '[voting][text]'],
			'text_short' => ['menu_moduleInformation', '[voting][text_short]'],
			'icon' => 'fa-thumbs-up',
			'parentMenu' => we_gui_mainMenu::E_MODULE,
			'inModuleMenu' => true,
			'alwaysActive' => false,
			'hasSettings' => false,
			//'dependson' => '',
			//'childmodule' => '',
			'tables' => ['VOTING_LOG_TABLE' => VOTING_LOG_TABLE, 'VOTING_TABLE' => VOTING_TABLE]
		],
		self::GLOSSARY => [
			'name' => self::GLOSSARY,
			'perm' => [we_base_perm::GLOSSARY_NEW, we_base_perm::GLOSSARY_DELETE, we_base_perm::GLOSSARY_EDIT],
			'text' => ['menu_moduleInformation', '[glossary][text]'],
			'text_short' => ['menu_moduleInformation', '[glossary][text_short]'],
			'icon' => 'fa-commenting',
			'parentMenu' => we_gui_mainMenu::E_SETTINGS,
			'inModuleMenu' => true,
			'alwaysActive' => false,
			'hasSettings' => true,
			//'dependson' => '',
			//'childmodule' => '',
			'tables' => ['GLOSSARY_TABLE' => GLOSSARY_TABLE]
		],
		self::SEARCH => [
			'name' => self::SEARCH,
			'text' => ['searchtool', '[weSearch]'],
			'text_short' => ['searchtool', '[weSearch]'],
			'icon' => 'fa-search',
			'parentMenu' => we_gui_mainMenu::E_MODULE,
			'inModuleMenu' => true,
			'alwaysActive' => true,
			'hasSettings' => false,
			//'dependson' => '',
			//'childmodule' => '',
			'tables' => ['SEARCH_TABLE' => SEARCH_TABLE]
		],
		self::THUMB => [
			'name' => self::THUMB,
			'perm' => [we_base_perm::ADMINISTRATOR],
			'text' => ['menu_global', '[thumbnails]'],
			'text_short' => ['menu_global', '[thumbnails]'],
			'icon' => 'fa-image',
			'parentMenu' => we_gui_mainMenu::E_SETTINGS,
			'inModuleMenu' => true,
			'alwaysActive' => true,
			'hasSettings' => false,
			//'dependson' => '',
			//'childmodule' => '',
			'tables' => ['THUMBNAILS_TABLE' => THUMBNAILS_TABLE]
		],
		self::DOCTYPE => [
			'name' => self::DOCTYPE,
			'perm' => [we_base_perm::WORK_DOCTYPE_EDIT],
			'text' => ['menu_global', '[document_types]'],
			'text_short' => ['menu_global', '[document_types]'],
			'icon' => 'fa-flask',
			'parentMenu' => we_gui_mainMenu::E_SETTINGS,
			'inModuleMenu' => true,
			'alwaysActive' => true,
			'hasSettings' => false,
			//'dependson' => '',
			//'childmodule' => '',
			'tables' => ['DOC_TYPES_TABLE' => DOC_TYPES_TABLE]
		],
		self::CATEGORY => [
			'name' => self::CATEGORY,
			'perm' => [we_base_perm::WORK_CATEGORY_EDIT],
			'text' => ['menu_global', '[categories]'],
			'text_short' => ['menu_global', '[categories]'],
			'icon' => 'fa-tags',
			'parentMenu' => we_gui_mainMenu::E_SETTINGS,
			'inModuleMenu' => true,
			'alwaysActive' => false,
			'hasSettings' => false,
			//'dependson' => '',
			//'childmodule' => '',
			'tables' => ['CATEGORY_TABLE' => CATEGORY_TABLE]
		],
		/* self::METAFIELDS => [
		  'name' => self::METAFIELDS,
		  'perm' => [we_base_perm::ADMINISTRATOR],
		  'text' => ['menu_global', '[metadata]'],
		  'text_short' => ['menu_global', '[metadata]'],
		  'icon' => 'fa-plus-circle',
		  'parentMenu' => we_gui_mainMenu::E_SETTINGS,
		  'inModuleMenu' => true,
		  'alwaysActive' => true,
		  'hasSettings' => false,
		  //'dependson' => '',
		  //'childmodule' => '',
		  'tables' => ['METADATA_TABLE' => METADATA_TABLE, 'METAVALUES_TABLE' => METAVALUES_TABLE]
		  ], */
	];

	private static $userEnabledModules = [];
	private static $activeModules = [];

	private static function init(): void{
		if(self::$userEnabledModules){
			return;
		}

		if(file_exists(WE_CUSTOM_PATH . 'conf/we_active_integrated_modules.inc.php')){
			self::$userEnabledModules = include_once(WE_CUSTOM_PATH . 'conf/we_active_integrated_modules.inc.php');
		} elseif(file_exists(WE_INCLUDES_PATH . 'conf/we_active_integrated_modules.inc.php')){
			$_we_active_integrated_modules = [];
			include_once(WE_INCLUDES_PATH . 'conf/we_active_integrated_modules.inc.php');
			self::$userEnabledModules = $_we_active_integrated_modules;
		}
		if(empty(self::$userEnabledModules) || !is_array(self::$userEnabledModules)/* if old style globals is used */){
			self::$userEnabledModules = include_once(WE_CUSTOM_PATH . 'conf/we_active_integrated_modules.inc.php.default');
		}

		foreach(self::AVAILABLE_MODULES as $modul => $conf){
			if(!empty($conf['alwaysActive']) || in_array($modul, self::$userEnabledModules, true)){
				self::$activeModules[] = $modul;
				if(!empty($conf['tables'])){
					we_base_request::registerTables($conf['tables']);
				}
			}
		}
	}

	public static function getUserEnabledModules(): array{
		self::init();
		return self::$userEnabledModules;
	}

	/**
	 * returns hash with All modules
	 *
	 * @return array<string,array{name:string,perm:string[],icon:string,parentMenu:string,inModuleMenu:bool,alwaysActive:bool,hasSettings:bool,text:string,text_short:string,dependson?:string,childmodule?:string,tables:array<string,string>}>
	 */
	public static function getAllModules(): array{
		self::init();
		$ret = [];
		foreach(self::AVAILABLE_MODULES as $key => $mod){
			$ret[$key] = self::setGLFields($mod);
		}
		return $ret;
	}

	/**
	 * returns hash of all modules
	 * @return array
	 */
	public static function getIntegratedModules(string $active): array{
		self::init();
		$ret = [];
		foreach(self::AVAILABLE_MODULES as $key => $modInfo){
			if(self::isActive($key) == $active){
				$ret[$key] = g_l($modInfo['text_short'][0], $modInfo['text_short'][1]);
			}
		}

		return $ret;
	}

	private static function setGLFields(array $mod): array{
		foreach(['text', 'text_short'] as $field){
			$mod[$field] = g_l($mod[$field][0], $mod[$field][1]);
		}
		return $mod;
	}

	/**
	 * returns whether a module is in the menu or not
	 * @param string $key key in module must eval true to return the entry
	 * @return array<string,array>
	 */
	public static function showModuleInMenu(string $menu, string $key): array{
		self::init();
		// show a module, if
		// - it is active
		// - if it is in module window
		$ret = [];
		foreach(self::AVAILABLE_MODULES as $modulekey => $mod){
			if((!$menu || self::AVAILABLE_MODULES[$modulekey]['parentMenu'] === $menu) && (!$key || !empty(self::AVAILABLE_MODULES[$modulekey][$key])) && self::isActive($modulekey)){
				if($modulekey === self::EDITOR && empty($_SESSION['weS']['hasJava'])){
					continue;
				}
				$mod = self::setGLFields($mod);
				$ret[$mod['text']] = $mod;
			}
		}
		ksort($ret);
		return $ret;
	}

	public static function isActive(string $modul): bool{
		self::init();
		return in_array($modul, self::$activeModules, true);
	}

	public static function getAllActiveModules(): array{
		return self::$activeModules;
	}

	public static function getAllInactiveModules(): array{
		return array_diff(array_keys(self::AVAILABLE_MODULES), self::$activeModules);
	}

	public static function loadConfigs(): void{
		self::init();
	}

	public static function getModuleField(string $module, string $field = ''){
		self::init();
		if(!empty(self::AVAILABLE_MODULES[$module])){
			$mod = self::setGLFields(self::AVAILABLE_MODULES[$module]);
			return $mod[$field] ?? '';
		}
		return '';
	}

}
