<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 1bf7a97ca6dc $
 * $Author: Marc Krämer <marc.kraemer@aldente-it.de> $
 * $Date: 2019-10-31 17:45 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

/**
 * base class for rpc commands
 *

 * @abstract
 */
abstract class we_rpc_cmd{
	const STATUS_OK = 0;
	const STATUS_NO_PERMISSION = 1;
	const STATUS_NOT_ALLOWED_VIEW = 2;
	const STATUS_LOGIN_FAILED = 3;
	const STATUS_REQUEST_MALFORMED = 4;
	const STATUS_NO_CMD = 5;
	const STATUS_NO_CIEW = 6;
	const STATUS_NO_SESSION = 7;
	const STATUS_NO_VIEW = 8;

	/** @var we_rpc_cmdShell */
	protected $CmdShell;

	/** @var array */
	protected $Permissions = [];

	/** @var int */
	protected $Status = self::STATUS_OK;

	/** @var string[] */
	protected $Parameters = [];

	/** @var we_rpc_response */
	protected $resp;

	public function __construct(){
		$this->resp = new we_rpc_response();
	}

	function executeRpcCmd(we_rpc_cmdShell $shell): void{
		$this->checkSession();
		$this->checkParameters();
		$this->Status = we_base_permission::hasPerm($this->Permissions, false) ? $this->Status : self::STATUS_NO_PERMISSION;
		$this->CmdShell = $shell;
	}

	function execute(): we_rpc_response{
		return $this->resp;
	}

	public function cleanUp(): void{
		//overwrite
	}

	function checkSession(): bool{
		if(!we_users_user::getID()){
			$this->Status = self::STATUS_NO_SESSION;
			return false;
		}

		return true;
	}

	function checkParameters(): bool{
		foreach($this->Parameters as $par){
			assert(is_string($par));
			if(we_base_request::string($par, we_base_request::NOT_VALID_S) === we_base_request::NOT_VALID_S){
				$this->Status = self::STATUS_REQUEST_MALFORMED;
				return false;
			}
		}
		return true;
	}

	public function getStatus(): int{
		return $this->Status;
	}

}
