<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: e6ba594552ca $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-05-11 23:31 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_search_tree extends we_tree_base{
	static $treeItems = [];

	protected function customJSFile(): string{
		return we_html_element::jsScript(JS_DIR . 'search_tree.js', 'initTree();');
	}

	public static function getItems(int $ParentID, int $offset = 0, int $segment = 500, bool $sort = false): array{
		$db = new DB_WE();
		$openFolders = [];

		if(isset($_SESSION['weS']['search']['modelidForTree'])){
			$id = $_SESSION['weS']['search']['modelidForTree'];
			$pid = fI('SELECT ParentID FROM `' . SEARCH_TABLE . '` WHERE ID=' . intval($id));
			$openFolders[] = $pid;
			while($pid > 0){
				$pid = fI('SELECT ParentID FROM `' . SEARCH_TABLE . '` WHERE ID=' . $pid);
				$openFolders[] = $pid;
			}
		}

		$wsQuery = '';
		$prevoffset = max(0, $offset - $segment);
		if($offset && $segment){
			self::$treeItems[] = [
				'id' => 'prev_' . $ParentID,
				'parentid' => $ParentID,
				'text' => 'display (' . $prevoffset . '-' . $offset . ')',
				'contenttype' => self::CT_UP,
				'table' => SEARCH_TABLE,
				'typ' => self::TYP_DOTS,
				'open' => 0,
				'published' => 0,
				'disabled' => 0,
				'tooltip' => '',
				'offset' => $prevoffset
			];
		}

		$where = $wsQuery . ' ParentID=' . intval($ParentID) . ' ';

		$all = $db->getAllq('SELECT ID,ParentID,Path,Text,IsFolder FROM `' . SEARCH_TABLE . '` WHERE ' . $where . ' ORDER BY (Text REGEXP "^[0-9]") DESC,abs(Text),Text' . ($segment ? ' LIMIT ' . abs($offset) . ',' . abs($segment) : ''));
		'@phan-var array{ID:int,ParentID:int,Path:string,Text:string,IsFolder:int}[] $all';
		foreach($all as $rec){
			switch($rec['Path']){
				case '/_PREDEF_/object':
				case '/_PREDEF_/object/unpublished':
					if(!we_modules_info::isActive(we_modules_info::OBJECT) || !we_base_permission::hasPerm([we_base_perm::OBJECTFILES_CAN_SEE])){
						continue 2;
					}
				case '/_PREDEF_/document':
				case '/_PREDEF_/document/unpublished':
				case '/_PREDEF_/document/static':
				case '/_PREDEF_/document/dynamic':
					if(!we_base_permission::hasPerm([we_base_perm::WORK_DOCUMENTS_CAN_SEE])){
						continue 2;
					}
				default:
					if(strpos($rec['Path'], '/_VERSION_') === 0 && !we_base_permission::hasPerm([we_base_perm::VERSIONS_CAN_SEE])){
						continue 2;
					}

					$isOpen = (in_array($rec['ID'], $openFolders) ? 1 : 0);

					$typ = [
						'typ' => ($rec['IsFolder'] ? self::TYP_FOLDER : self::TYP_ITEM),
						'contenttype' => ($rec['IsFolder'] ? we_base_ContentTypes::FOLDER_COMMON : 'we/search'),
						'open' => $isOpen,
						'parentid' => $rec['ParentID'],
						'disabled' => 0,
						'tooltip' => $rec['ID'],
						'offset' => $offset,
						//'order' => $rec['Ordn'],
						'published' => 1,
						'text' => htmlspecialchars(we_search_model::getLangText($rec['Path'], $rec['Text'])),
					];
					$fields = array_change_key_case($rec);


					self::$treeItems[] = array_merge($fields, $typ);

					if($rec['IsFolder'] && $isOpen){
						static::getItems((int) $rec['ID'], 0, $segment);
					}
			}
		}

		$total = fI('SELECT COUNT(1) FROM `' . SEARCH_TABLE . '` WHERE ' . $where);
		$nextoffset = $offset + $segment;
		if($segment && ($total > $nextoffset)){
			self::$treeItems[] = [
				'id' => 'next_' . $ParentID,
				'parentid' => $ParentID,
				'text' => 'display (' . $nextoffset . '-' . ($nextoffset + $segment) . ')',
				'contenttype' => self::CT_DOWN,
				'table' => SEARCH_TABLE,
				'typ' => self::TYP_DOTS,
				'open' => 0,
				'disabled' => 0,
				'tooltip' => '',
				'offset' => $nextoffset
			];
		}

		return self::$treeItems;
	}

}
