<?php
declare(strict_types=1);

/* webEdition CMS
 *
 * $Rev: a9645d657b67 $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-04-03 19:49 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

abstract class we_tag_ifEmpty extends we_weTag_ifTag{

	public static function tag(array $attribs = [], string $content = '', bool $internal = false): bool{
		if(we_base_WE::$editmode){
			return false;
		}

		$docAttr = self::getAttribute('doc', $attribs, '', we_base_request::STRING);
		$name = (string) (self::getAttribute('name', $attribs, '', we_base_request::STRING) ?: self::getPostName(self::getAttribute('match', $attribs, '', we_base_request::STRING)));
		$doc = self::getDocForTag($docAttr, false);
		$regs = [];

		if(!$name && ($foo = self::attributeMissing($attribs, ['name' => false], __CLASS__))){
			echo $foo;
			return false;
		}

		if(preg_match('|^\\\\?\$([^\[]+)(\[.*\])?|', $name, $regs)){
			$name = (string) (isset($regs[2]) ?
				self::getArrayValue($GLOBALS, $regs[1], $regs[2]) :
				($GLOBALS[$regs[1]] ?? ''));
		}

		switch(self::getAttribute('type', $attribs, '', we_base_request::STRING)){
			case 'checkbox':
			case 'object':
				$val = $doc->getElementS($name);
				return empty($val);
			case 'binary' :
			case 'img':
			case 'flashmovie' :
				$val = $doc->getElementI($name);
				return !($val && id2path($val, FILE_TABLE, false, true));
			case 'href':
				if($doc instanceof we_document_triggerDocument){
					$hrefA = $doc->getElementA($name);
					if(!$hrefA){
						return true;
					}
					$hreftmp = trim(we_document_base::getHrefByArray($hrefA));

					return ($hreftmp && $hreftmp[0] === '/' ?
						(!file_exists(WEBEDITION_PATH . '../' . $hreftmp)) :
						($hreftmp ? false : true)
						);
				}
				if($doc->getElementI($name . we_base_link::MAGIC_INT_LINK)){ // for type = href int
					$intID = $doc->getElementI($name . we_base_link::MAGIC_INT_LINK_ID);
					return !($intID && id2path($intID, FILE_TABLE, false, true));
				}
				$hreftmp = $doc->getElementS($name);
				if($hreftmp && $hreftmp[0] === '/'){
					return (!file_exists(WEBEDITION_PATH . '../' . $hreftmp));
				}

			default:
				$dat = $doc->getElementS($name);
				//   #3938 added this - some php version crashed, when unserialize started with a ?,?,?
				if(we_base_util::isSerialized($dat)){ //  only unserialize, when $match could be an array
					// Added @-operator in front of the unserialze function because there
					// were some PHP notices that had no effect on the output of the function
					// remark holeg: when it is a serialized array, the function looks if it is not empty
					if(is_array($arr = we_unserialize($dat))){
						return empty($arr);
					}
				}

			//   end of #3938
		}

		return $doc->isEmptyElement($name);
	}

}
