<?php
declare(strict_types=1);

/* webEdition CMS
 *
 * $Rev: f389fd111aab $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-02-17 01:17 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

abstract class we_tag_ifVarEmpty extends we_weTag_ifTag{

	public static function tag(array $attribs = [], string $content = '', bool $internal = false): bool{
		$name = self::getAttribute('name', $attribs, '', we_base_request::STRING);
		$orgName = (string) self::getAttribute('_name_orig', $attribs, '', we_base_request::STRING);
		self::$doc = we_base_WE::getWeDoc();

		if(!$name){
			$attribs['_name_orig'] = $orgName = (string) self::getAttribute('match', $attribs, '', we_base_request::STRING);
			$attribs['name'] = $name = self::getPostName($orgName);
		}

		if(!$orgName && ($foo = self::attributeMissing($attribs, ['name' => false], __CLASS__))){
			echo $foo;
			return false;
		}

		$type = self::getAttribute('type', $attribs, '', we_base_request::STRING);

		switch($type){
			case 'cookie' :
				$ret = self::getArrayValue($_COOKIE, null, $orgName);
				return empty($ret);
			case 'request' :
				$ret = self::getArrayValue($_REQUEST, null, $orgName);
				return empty($ret);
			case 'post' :
				$ret = self::getArrayValue($_POST, null, $orgName);
				return empty($ret);
			case 'get' :
				$ret = self::getArrayValue($_GET, null, $orgName);
				return empty($ret);
			case 'global' :
				$ret = self::getArrayValue($GLOBALS, null, $orgName);
				return empty($ret);
			case 'session' :
				$ret = self::getArrayValue($_SESSION, null, $orgName);
				return empty($ret);
			case 'sessionfield' :
				return empty($_SESSION['webuser'][$orgName]);
		}

		$docAttr = self::getAttribute('doc', $attribs, '', we_base_request::STRING);
		switch($docAttr){
			case 'object' :
			case 'document' :
				$formname = self::getAttribute('formname', $attribs, we_base_constants::DEFAULT_FORM, we_base_request::STRING);
				$doc = $GLOBALS['we_' . $docAttr][$formname] ?? false;
				break;
			case 'top':
				$doc = !we_base_WE::$mainDoc['doc'] ? false : we_base_WE::$mainDoc['doc'];
				break;
			default:
				$doc = self::$doc ?? false;
		}
		if(!$doc){
			return true;
		}

		if(self::getAttribute('property', $attribs, false, we_base_request::BOOL)){
			return empty($doc->$orgName);
		}

		$type = $type ?: $doc->getElementType($orgName);
		switch($type){
			case 'href' :
				$foo = $doc->getField($attribs, $type, true);
				return empty($foo) || $foo === '/';
			case 'multiobject':
				//FIXME: this makes no sense
				/* $attribs['name'] = $name;
				  $attribs['_name_orig'] = $orgName; */
				$data = we_unserialize((string) $doc->getField($attribs, $type, true));
				$objects = $data['objects'] ?? $data;
				return empty($objects);
			default:
				$foo = we_tag_var::getElementData($doc, $name) ?: we_tag_var::getElementData($doc, $orgName);
				return empty($foo);
		}
	}

}
