<?php
declare(strict_types=1);

/* webEdition CMS
 *
 * $Rev: b6c8f7b54af9 $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-06-05 15:22 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

abstract class we_tag_include extends we_weTag_tag{
	/** @var array<int,array{we_doc:object,GLOBAL:array,REQUEST:array,we:array}> */
	private static $backvars = [];
	private static $we_unique = 0;

	public static function parse(array $attribs, string $content): string{
		$type = self::getParserAttribute('type', $attribs, 'document', we_base_request::STRING);
		$once = (self::getParserAttribute('once', $attribs, false, we_base_request::BOOL) ? '_once' : '');
		$merge = self::getParserAttribute('merge', $attribs, false, we_base_request::BOOL);
		if($type === 'template'){
			$attribs['_parsed'] = 'true';
		}

		return ($type === 'template' ?
			'<?php if(($we_inc=' . we_weTag_parser::printTag('include', $attribs) . ')){include' . $once . '($we_inc);}; ?>' : //include templates of ID's
			'<?php if(($we_inc=' . we_weTag_parser::printTag('include', $attribs) . ')){we_tag_include::setBackVar(we_tag_include::newBackVarID());include' . $once . '($we_inc);we_tag_include::resetBackVar(-1,' . ($merge ? 'true' : 'false') . ');}?>' //include documents
			);
	}

	public static function tag(array $attribs = [], string $content = '', bool $internal = false): string{
		$id = intval(self::getAttribute('id', $attribs, 0, we_base_request::INT));
		$path = (string) self::getAttribute('path', $attribs, '', we_base_request::FILE);
		self::$doc = we_base_WE::getWeDoc();

		if(self::getAttribute('type', $attribs, '', we_base_request::STRING) === 'template'){
			if(!isset($attribs['_parsed'])){
				echo 'cannot use we:include with type="template" dynamically';
				return '';
			}
			$ret = rtrim(preg_replace('/.tmpl$/i', '.php', ($id ? id2path($id, TEMPLATES_TABLE) : str_replace('..', '', $path))), '/'); //filter rel. paths
			return (empty($ret) ? '' : TEMPLATES_PATH . $ret);
		}

		$name = self::getAttribute('name', $attribs, '', we_base_request::STRING);
		$gethttp = self::getAttribute('gethttp', $attribs, false, we_base_request::BOOL);
		$seeMode = self::getAttribute((isset($attribs['seem']) ? 'seem' : 'seeMode'), $attribs, true, we_base_request::BOOL);

		if((!$id) && (!$path) && (!$name)){
			t_e('we:include - missing id, path or name');
			echo '<!-- we:include - missing id, path or name !!-->';
			return '';
		}

		if(we_weTag_util::tag('ifEditmode')){
			if($name && !($id || $path)){
				self::edit($attribs);
				return '';
			}
		} else //notEditmode
		if($name && !($id || $path)){
			$type = self::getAttribute('kind', $attribs, we_base_link::TYPE_ALL, we_base_request::STRING);
			$nameOrig = self::getAttribute('_name_orig', $attribs, '', we_base_request::STRING);
			$nint = $name . we_base_link::MAGIC_INT_LINK;
			$int = self::$doc->getElementI($nint);
			if($int && ($id = self::$doc->getElementI($nint . 'ID'))){
				$path = '';
			} else {
				$path = (string) we_weTag_util::tag('href', ['name' => $nameOrig, 'hidedirindex' => 'false', 'type' => $type, 'isInternal' => 1]);
			}
		}

		if(
			(!$id && !$path) ||
			(!empty(we_base_WE::$mainDoc['id']) && we_base_WE::$mainDoc['id'] == $id)//don't include same id
		){
			return '';
		}

		$realPath = ($id ? self::getRealPath($id) : $path);
		if(!$realPath){
			return '';
		}

		$isSeemode = (we_weTag_util::tag('ifSeeMode'));
		// check early if there is a document - if not the rest is never needed
		if($gethttp){
			try{
				echo ($isSeemode && $seeMode && ($id || $path) ? we_gui_SEEM::getSeemAnchors(($id ?: path_to_id($path, FILE_TABLE)), 'include') : '') .
				we_http_util::getHTTP(we_http_util::getServerUrl(true), $realPath);
			} catch (we_exception_http $e){
				echo $e->getMessage();
			}
			return '';
		}

		$fullPath = WEBEDITION_PATH . '..' . $realPath; //(symlink) webEdition always points to the REAL DOC-Root!
		if(!file_exists($fullPath) || !is_file($fullPath)){
//		t_e('include of', 'id:' . $id . ',path:' . $path . ',name:' . $name, ' doesn\'t exist');
			return '';
		}
		//check Customer-Filter on static documents
		if(we_modules_info::isActive(we_modules_info::CUSTOMER) && $id){
			$filter = we_customer_documentFilter::getFilterByIdAndTable($id, FILE_TABLE);

			if($filter->accessForVisitor($id, true) != we_customer_documentFilter::ACCESS){
				return '';
			}
		}

		if($isSeemode && $seeMode && ($id || $path)){
			echo we_gui_SEEM::getSeemAnchors(($id ?: path_to_id($path, FILE_TABLE)), 'include');
		}

		return $fullPath;
	}

	public static function setBackVar(int $we_unique): void{
		self::$backvars[$we_unique] = [
			'we_doc' => clone(we_base_WE::getWeDoc()),
			'currentDocID' => we_base_WE::$template['currentDocID'],
			'we_transaction' => we_base_WE::$transaction,
			'postTagName' => self::$postTagName,
			'editmode' => we_base_WE::$editmode,
			'WE_IS_DYN' => isset($GLOBALS['WE_IS_DYN']) ? 1 : 0,
			'we_cmd' => we_base_request::_(we_base_request::RAW, 'we_cmd', ''),
		];

		if(isset($GLOBALS['WE_IS_DYN'])){
			unset($GLOBALS['WE_IS_DYN']);
		}
		self::$postTagName = '';
	}

	/**
	 * function of we:include
	 * @param int $we_unique
	 * @param bool $merge
	 */
	public static function resetBackVar(int $we_unique = -1, bool $merge = false): void{
		$keys = array_keys(self::$backvars);
		$index = ($we_unique === -1 ? end($keys) : $we_unique);
		$curDoc = we_base_WE::$we_doc;
		$parentDoc = clone(self::$backvars[$index]['we_doc']);
		we_base_WE::setWeDoc($parentDoc);
		if($merge){
			$parentDoc->elements = array_merge($parentDoc->elements, $curDoc->elements);
		}

		$_REQUEST['we_cmd'] = self::$backvars[$index]['we_cmd'];
		self::$postTagName = self::$backvars[$index]['postTagName'];
		we_base_WE::$editmode = self::$backvars[$index]['editmode'];
		we_base_WE::$template['currentDocID'] = self::$backvars[$index]['currentDocID'];
		we_base_WE::$transaction = self::$backvars[$index]['we_transaction'];

		if(self::$backvars[$index]['WE_IS_DYN']){
			$GLOBALS['WE_IS_DYN'] = 1;
		} else if(isset($GLOBALS['WE_IS_DYN'])){
			unset($GLOBALS['WE_IS_DYN']);
		}
		unset(self::$backvars[$index]);
	}

	public static function newBackVarID(): int{
		$new = self::$we_unique++;
		self::$backvars[$new] = [];
		return $new;
	}

	public static function isIncludeActive(): bool{
		return !empty(self::$backvars);
	}

	protected static function getRealPath(int $id): string{
		$doc = we_base_WE::getWeDoc();
		switch($doc->ContentType){
			case we_base_ContentTypes::WEDOCUMENT:
				break; //don't include any unknown document
			case we_base_ContentTypes::TEMPLATE:
				if(in_array($doc->EditPageNr, [we_base_constants::WE_EDITPAGE_PREVIEW, we_base_constants::WE_EDITPAGE_PREVIEW_TEMPLATE])){
					break;
				}
			default:
				return '';
		}
		$tmp = getHash('SELECT Path,ContentType FROM ' . FILE_TABLE . ' WHERE ID=' . $id . ' AND IsPublished=1', null);
		$realPath = $tmp ? (string) $tmp['Path'] : '';
		$ct = $tmp ? (string) $tmp['ContentType'] : '';
		switch($ct){
			case we_base_ContentTypes::WEDOCUMENT:
			case we_base_ContentTypes::JS:
			case we_base_ContentTypes::CSS:
			case we_base_ContentTypes::HTML:
			case we_base_ContentTypes::TEXT:
			case we_base_ContentTypes::XML:
				return $realPath;
			default:
				return '';
		}
	}

	protected static function edit(array $attribs): void{
		$type = self::getAttribute('kind', $attribs, we_base_link::TYPE_ALL, we_base_request::STRING);
		$nameOrig = self::getAttribute('_name_orig', $attribs, '', we_base_request::STRING);
		$description = self::getAttribute('description', $attribs, g_l('weTag', '[include][include_file]'), we_base_request::RAW);

		echo '<table class="we we_include weEditTable"><tr><td>' . $description . '</td></tr><tr><td>' .
		we_weTag_util::tag('href', ['name' => $nameOrig, 'rootdir' => self::getAttribute('rootdir', $attribs, '/', we_base_request::FILE), 'startid' => self::getAttribute('startid', $attribs, 0, we_base_request::INT),
			'type' => $type, 'size' => self::getAttribute('size', $attribs, 50, we_base_request::UNIT)]) .
		'</td></tr></table>';
	}

}
