<?php
declare(strict_types=1);

/* webEdition CMS
 *
 * $Rev: aacab26fe68a $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-05-05 10:25 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

abstract class we_tag_printVersion extends we_weTag_tag{

	public static function tag(array $attribs = [], string $content = '', bool $internal = false): string{
		if(($foo = self::attributeMissing($attribs, ['tid' => false], __CLASS__))){
			return $foo;
		}

		$tid = self::getAttribute('tid', $attribs, 0, we_base_request::INT);
		$triggerID = self::getAttribute('triggerid', $attribs, self::getAttribute('triggerID', $attribs, 0, we_base_request::INT, true), we_base_request::INT);
		$docAttr = self::getAttribute('doc', $attribs, self::getAttribute('type', $attribs, '', we_base_request::STRING, true), we_base_request::STRING);
		$link = self::getAttribute('Link', $attribs, self::getAttribute('link', $attribs, true, we_base_request::BOOL), we_base_request::BOOL, true);

		$doc = self::getDocForTag($docAttr);
		try{
			$id = we_base_WE::getWeObj()->ID;
		} catch (Exception $ex){
			$id = $doc->ID;
		}


		$query_string = [];

		$hideQuery = [WEOBJ_PARAM, 'tid', 'id', 'pv_tid', 'pv_id', 'we_cmd', 'responseText', 'we_mode', 'btype', SESSION_NAME, 'PHPSESSID'];
		if(isset($_SESSION)){
			$hideQuery[] = session_name();
		}
		if(isset($_REQUEST)){
			$tmp = array_merge($_GET, $_POST);
			foreach($tmp as $k => $v){
				$k = strip_tags($k);
				if((!is_array($v)) && (!in_array($k, $hideQuery, true))){
					$query_string[$k] = strip_tags($v);
				}
			}
		}

		if($doc instanceof we_contents_objectFile){
			//objects are always shown by a dynamic page
			$data = [
				WEOBJ_PARAM => $id,
				'we_objectTID' => $tid
			];
			$query_string[WEOBJ_PARAM] = $id;
			$query_string['we_objectTID'] = $tid;
			$query_string['jwt'] = we_helpers_jwt::encode(we_serialize($data, SERIALIZE_JSON), sha1(SECURITY_ENCRYPTION_KEY));
		} else {
			$triggerID = $triggerID ?: ($doc->IsDynamic ? $doc->ID : 0);
			if($triggerID || $doc->IsDynamic){
				$data = [
					'pv_id' => $id,
					'pv_tid' => $tid,
				];
				$query_string['pv_id'] = $id;
				$query_string['pv_tid'] = $tid;
				$query_string['jwt'] = we_helpers_jwt::encode(we_serialize($data, SERIALIZE_JSON), sha1(SECURITY_ENCRYPTION_KEY));
			} else {
				return $content;
			}
		}

		$attribs['href'] = ($triggerID ? id2path($triggerID) : we_http_util::getScriptName()) . '?' . http_build_query($query_string);

		return ($link ?
			self::htmlTag('a', self::removeAttribs($attribs, ['tid', 'triggerID', 'triggerid', 'doc', 'type', 'link', 'Link']), $content, true) :
			$attribs['href']);
	}

}
