<?php
declare(strict_types=1);

/* webEdition CMS
 *
 * $Rev: 9f63b61cab1f $
 * $Author: Lukas Imhof $
 * $Date: 2020-06-02 14:20 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

abstract class we_tag_video extends we_weTag_tag{

	public static function tag(array $attribs = [], string $content = '', bool $internal = false): string{
		$lid = self::getAttribute('id', $attribs, 0, we_base_request::INT);
		$name = self::getAttribute('name', $attribs, '', we_base_request::STRING);
		if(!($lid || $name) && ($foo = self::attributeMissing($attribs, ['name' => false, 'id' => false], __CLASS__))){
			return $foo;
		}
		self::$doc = we_base_WE::getWeDoc();
		$showcontrol = self::getAttribute('showcontrol', $attribs, true, we_base_request::BOOL);
		$showvideo = self::getAttribute('showvideo', $attribs, true, we_base_request::BOOL);

		$id = $name ? (self::$doc->getElementI($name) ?: $lid) : $lid;

		if(isset($attribs['showcontrol']) && !$showcontrol && $lid){//bug 6433: später wird so ohne weiteres gar nicht mehr auf die id zurückgegriffen
			$attribs['id'] = $lid; //siehe korrespondierende Änderung in we:document::getField
			$attribs['showcontrol'] = $showcontrol; //sicherstellen das es boolean ist
		}
		$fname = 'we_' . self::$doc->Name . '_video[' . $name . '#' . we_contents_base::ETYPE_NUMBER . ']';

		$out = (we_base_WE::$editmode && !$showvideo ?
			'' :
			(string) self::$doc->getField(self::removeAttribs($attribs, ['showvideo', 'parentid', 'startid', 'showcontrol']), 'video') );

		if($showcontrol && we_base_WE::$editmode){
			$startid = self::getAttribute('startid', $attribs, 0, we_base_request::INT);
			$parentid = self::getAttribute('parentid', $attribs, 0, we_base_request::INT);
			// Create "Edit" button

			$button = ($id ?
					we_html_button::create_button(we_html_button::OPEN_TO_EDIT, "javascript:WE().layout.weEditorFrameController.openDocument(WE().consts.tables.FILE_TABLE,$id,WE().consts.contentTypes.VIDEO)") :
				// disable edit_image_button
				we_html_button::create_button(we_html_button::OPEN_TO_EDIT, "#", '', "", true)) .
				we_html_button::create_button('fa:btn_edit_video,fa-lg fa-hand-o-right,fa-lg fa-file-video-o', "javascript:we_cmd('we_selector_document',{id:'" . ($id ?: $startid) . "',table:WE().consts.tables.FILE_TABLE,targetID:'" . $fname . "',jsCmd:'reload_hot_editpage',rootDirID:" . $parentid . ", filter:WE().consts.contentTypes.VIDEO, open_doc:" . (we_base_permission::hasPerm([
						we_base_perm::WORK_WORKSPACE_CAN_SELECT_OTHER_FILES]) ? 0 : 1) . "})");

			// Create "Delete/Clear" button
			$clear_button = we_html_button::create_button(we_html_button::TRASH, "javascript:we_cmd('remove_image', '" . $name . "')");

			return '<table class="we we_video weEditTable">
	<tr>
		<td>' . $out . '<input type="hidden" name="' . $fname . '" value="' . self::$doc->getElementI($name) . '" /></td>
	</tr>
	<tr>
		<td style="text-align:center">' . $button . $clear_button . '</td></tr></table>';
		}
		//	When in SEEM - Mode add edit-Button to tag - textarea
		return $out;
	}

}
