<?php
declare(strict_types=1);

/* webEdition CMS
 *
 * $Rev: 3cfe184d65ae $
 * $Author: Lukas Imhof $
 * $Date: 2020-04-03 18:18 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

abstract class we_tag_write extends we_weTag_tag{
	const EDIT_DOC_PARAM = 'we_editDocument_ID';
	const EDIT_OBJ_PARAM = 'we_editObject_ID';
	public const OK = 0;
	public const ERR_INTERN = 1;
	public const ERR_NOT_ALLOWED_EDIT = 4;
	public const ERR_PROTECTION = 5;
	public const ERR_DUPLICATE_NAME = 8;
	public const ERR_WRITE = 7;

	/** @var array<string,array<string,int>> */
	public static $status = [
		'object' => [],
		'document' => [],
	];

	/** @var array<string,we_document_webEdition> */
	public static $docElements = [];

	/** @var array<string,we_contents_objectFile> */
	public static $objElements = [];

	public static function tag(array $attribs = [], string $content = '', bool $internal = false): string{
		$type = self::getAttribute('type', $attribs, 'document', we_base_request::STRING);

		switch($type){
			case 'object':
				if(($foo = self::attributeMissing($attribs, ['classid' => false], __CLASS__))){
					return $foo;
				}
				break;
			default:
				$type = 'document'; //make sure type is known!
				if(($foo = self::attributeMissing($attribs, ['doctype' => false], __CLASS__))){
					return $foo;
				}
				break;
		}

		$name = self::getAttribute('formname', $attribs, (we_tag_form::$name ?: we_base_constants::DEFAULT_FORM), we_base_request::STRING);
		$publish = self::getAttribute('publish', $attribs, false, we_base_request::BOOL);
		$triggerid = self::getAttribute('triggerid', $attribs, 0, we_base_request::INT);
		$categories = self::getAttribute('categories', $attribs, [], we_base_request::WEFILELISTA);
		$classid = self::getAttribute('classid', $attribs, 0, we_base_request::INT);
		$protected = self::getAttribute('protected', $attribs, false, we_base_request::BOOL);
		$admin = self::getAttribute('admin', $attribs, '', we_base_request::STRING);
		$mail = self::getAttribute('mail', $attribs, [], we_base_request::EMAILLISTA); //FIXME: email_list
		$mailfrom = self::getAttribute('mailfrom', $attribs, '', we_base_request::EMAIL);
		$forceedit = self::getAttribute('forceedit', $attribs, false, we_base_request::BOOL) && !empty($_SESSION['webuser']['registered']);
		$workspaces = self::getAttribute('workspaces', $attribs, [], we_base_request::INTLISTA);
		$objname = preg_replace('/[^a-z0-9_-]/i', '', self::getAttribute('name', $attribs, '', we_base_request::STRING));
		$onduplicate = ($objname === '' ? we_exim_ExIm::COLLISION_OVERWRITE : self::getAttribute('onduplicate', $attribs, 'increment', we_base_request::STRING));
		$onpredefinedname = self::getAttribute('onpredefinedname', $attribs, 'appendto', we_base_request::STRING);
		$workflowname = self::getAttribute('workflowname', $attribs, '', we_base_request::STRING);
		$workflowuserid = self::getAttribute('workflowuserid', $attribs, 0, we_base_request::INT);
		$searchable = self::getAttribute('searchable', $attribs, true, we_base_request::BOOL);
		$language = self::getAttribute('language', $attribs, '', we_base_request::STRING);

		$doworkflow = ($workflowname && $workflowuserid);

		//init with ok
		if(!we_base_request::bool('edit_' . $type)){
			self::$status[$type][$name] = 9;
			return '';
		}
		$catIds = path_to_idA($categories, CATEGORY_TABLE);
		switch($type){
			case 'document':
				$tid = self::getAttribute('tid', $attribs, 0, we_base_request::INT);
				$doctype = self::getAttribute('doctype', $attribs, '', we_base_request::STRING);
				$id = we_base_request::int(self::EDIT_DOC_PARAM, 0);
				$ok = we_document_webEdition::initDocument($name, $tid, $doctype, $catIds, $id, true);
				break;
			case 'object':
				$parentid = self::getAttribute('parentid', $attribs, 0, we_base_request::INT);
				$id = we_base_request::int(self::EDIT_OBJ_PARAM, 0);

				if(!fB('SELECT 1 FROM ' . OBJECT_TABLE . ' WHERE ID=' . intval($classid))){
					self::$status['object'][$name] = self::ERR_INTERN;
					t_e('Table ' . intval($classid) . ' does not exist!');
					return '';
				}
				if(!$id || !fB('SELECT 1 FROM ' . OBJECT_FILES_TABLE . ' WHERE ID=' . $id . ' AND TableID=' . intval($classid))){
					self::$status['object'][$name] = self::ERR_INTERN;
					t_e('Object ' . $id . ' is no element of class ' . intval($classid) . '!');
					return '';
				}
				try{
					$ok = we_contents_objectFile::initObject(intval($classid), $name, $catIds, intval($parentid), $id, true);
				} catch (we_exception_base $e){
					t_e($e);
					self::$status['object'][$name] = self::ERR_INTERN;
					return '';
				}
				break;
			default:
				self::$status[$type][$name] = self::ERR_INTERN;
				return '';
		}

		$isOwner = !empty($_SESSION['webuser']['registered']) && isset($_SESSION['webuser']['ID']) && (
			($protected && ($_SESSION['webuser']['ID'] == $ok->WebUserID))
			);

		$isAdmin = !empty($_SESSION['webuser']['registered']) && $admin && !empty($_SESSION['webuser'][$admin]);

		$isNew = (($ok->ID == 0) ? ($admin/* only if this field is used */ ? $isAdmin : true) : false); //FR #8411

		if(!($isAdmin || $isNew || $isOwner || $forceedit)){
			self::$status[$type][$name] = self::ERR_NOT_ALLOWED_EDIT;
			return '';
		}

		$doWrite = true;
		//$newObject = ($GLOBALS['we_'.$type][$name]->ID) ? false : true;
		if($protected){
			if(!isset($_SESSION['webuser']['ID']) || !isset($_SESSION['webuser']['registered']) || !$_SESSION['webuser']['registered']){
				self::$status[$type][$name] = self::ERR_PROTECTION;
				return '';
			}
			if(!$ok->WebUserID){
				$ok->WebUserID = $_SESSION['webuser']['ID'];
			}
		}
		self::$status[$type][$name] = self::OK;
		self::checkAndCreateBinary(($type === 'document' ? self::$docElements[$name] : self::$objElements[$name]));

		//FIXME: we should probably use checkFieldsOnSave?!
		$ok->checkPathDiffAndCreate();
		if(!$objname){
			$ok->correctDoublePath();
		}
		$WE_DOC_SAVE = we_base_WE::$we_doc;
		if($ok instanceof we_document_webEdition){
			we_base_WE::setWeDoc($ok);
		}
		$ok->IsSearchable = $searchable;
		switch($language){
			case 'self':
			case 'top':
				$docLanguage = self::getDocForTag($language);
				$language = $docLanguage->Language;
				unset($docLanguage);
		}
		if($language){
			$ok->Language = $language;
		} else {
			//preset lang
			$ok->initLanguageFromParent();
		}
		$ok->Path = $ok->getPath();

		if(we_modules_info::isActive(we_modules_info::OBJECT) && $type === 'object'){
			$obj = we_tag_write::$objElements[$name];
			if($workspaces){
				$obj->setWorkspaces($workspaces);
			}
			$objname = self::getObjName($obj, $objname, $onpredefinedname);
			$doWrite = self::correctObjName($obj, $objname, 'object', $name, $onduplicate);
		}

		if($doWrite){
			$ret = $ok->we_save();
			if($ret && $publish && !$doworkflow){
				$ret = $ok->we_publish();
			}

			if($ret){
				if($doworkflow){
					$wf_text = $workflowname . '  ';
					switch($type){
						default:
						case 'document':
							$wf_text .= 'Document ID: ' . we_base_WE::$we_doc->ID;
							$tab = FILE_TABLE;
							break;
						case 'object':
							$wf_text .= 'Object ID: ' . we_base_WE::$we_doc->ID;
							$tab = OBJECT_FILES_TABLE;
							break;
					}
					try{
						$workflowID = we_workflow_utility::getWorkflowID($workflowname, $tab);
						if(!we_workflow_utility::insertDocInWorkflow(we_base_WE::$we_doc->ID, we_base_WE::$we_doc->Table, $workflowID, $workflowuserid, $wf_text)){
							t_e('error inserting document to workflow. Additional data:', we_base_WE::$we_doc->Table, $workflowID, $workflowuserid);
						}
					} catch (we_exception_base $e){
						//no workflow found
					}
				}
				$GLOBALS['we_object_write_ID'] = we_base_WE::$we_doc->ID;

				/**
				 * Fix #9818
				 * now we have to set the new document/object ID as request value to avoid
				 * createing more documents/objects by reload the webform (<we:form type="object | document">) by an user
				 */
				$requestVarName = 'we_edit' . ucfirst($type) . '_ID';
				$_REQUEST[$requestVarName] = we_base_WE::$we_doc->ID;
			} else {
				//save or publish failed
				$doWrite = false;
				self::$status[$type][$name] = self::ERR_WRITE;
			}
		}

		we_base_WE::setWeDoc(null);
		if($WE_DOC_SAVE){
			we_base_WE::setWeDoc($WE_DOC_SAVE);
			unset($WE_DOC_SAVE);
		}
		$_REQUEST['we_returnpage'] = $ok->getElementS('we_returnpage');

		if($doWrite && $mail){
			if(!$mailfrom){
				$mailfrom = 'dontReply@' . $_SERVER['SERVER_NAME'];
			}
			$path = $ok->Path;
			switch($type){
				case 'object':
					$classname = fS('SELECT Text FROM ' . OBJECT_TABLE . ' WHERE ID=' . intval($classid));
					$mailtext = sprintf(g_l('global', '[std_mailtext_newObj]'), $path, $classname) . "\n" .
						($triggerid ? id2path($triggerid) . '?' . WEOBJ_PARAM . '=' : 'ObjectID: ') .
						we_tag_write::$objElements[$name]->ID;
					$subject = g_l('global', '[std_subject_newObj]');
					break;
				default:
				case 'document':
					$mailtext = sprintf(g_l('global', '[std_mailtext_newDoc]'), $path) . "\n" . $ok->getHttpPath();
					$subject = g_l('global', '[std_subject_newDoc]');
					break;
			}
			$phpmail = new we_mail_mail([], $subject, we_mail_address::parse($mailfrom));
			$phpmail->addAddressList($mail);
			$phpmail->addTextPart($mailtext);
			$phpmail->buildMessage();
			$phpmail->send();
		}

		if(we_tag_sessionStart::$started){
			unset($_SESSION['weS']['we_' . $type . '_session_' . $name]); //fix #8051
		}
		return '';
	}

	protected static function getObjName(we_contents_objectFile $obj, string $objname, string $onpredefinedname): string{
		if($obj->Filename === ''){
			return $objname ?: strval(1 + fI('SELECT MAX(ID) AS ID FROM ' . OBJECT_FILES_TABLE));
		}
		switch($onpredefinedname){
			case 'appendto':
				return ($objname ? $obj->Filename . '_' . $objname : $obj->getText());
			case 'infrontof':
				return $objname . ($objname ? '_' . $obj->Filename : $obj->getText());
			case we_exim_ExIm::COLLISION_OVERWRITE:
				return $objname ?: $obj->Filename;
			default:
				return $objname;
		}
	}

	protected static function correctObjName(we_contents_objectFile $obj, string $objname, string $type, string $name, string $onduplicate): bool{
		$objexists = fI('SELECT ID FROM ' . OBJECT_FILES_TABLE . ' WHERE ParentID=' . $obj->ParentID . ' AND Filename="' . DB_WE::$inst->escape($objname) . '"');

		if(!$objexists){
			$obj->Filename = $objname;
			$obj->Path = $obj->getPath();
			return true;
		}
		switch($onduplicate){
			default:
			case 'abort':
				self::$status[$type][$name] = self::ERR_DUPLICATE_NAME;
				return false;
			case we_exim_ExIm::COLLISION_OVERWRITE:
				$obj->ID = $objexists;
				$obj->Filename = $objname;
				$obj->Path = $obj->getPath();
				return true;
			case 'increment':
				$z = 1;
				$footext = $objname . '_' . $z;
				while(fI('SELECT ID FROM ' . OBJECT_FILES_TABLE . ' WHERE ParentID=' . $obj->ParentID . ' AND Filename="' . DB_WE::$inst->escape($footext) . '"')){
					$footext = $objname . '_' . ( ++$z);
				}
				$obj->Filename = $footext;
				$obj->Path = $obj->getPath();
				return true;
		}
	}

	protected static function checkAndCreateBinary(we_document_base $doc): void{
		$webuserId = !empty($_SESSION['webuser']['registered']) && !empty($_SESSION['webuser']['ID']) ? $_SESSION['webuser']['ID'] : 0;
		$regs = [];

		$checks = [
			we_fileupload::USERINPUT_TYPE_BINARY => [
				'application/',
				we_base_ContentTypes::APPLICATION,
				'application',
				[]
			],
			we_fileupload::USERINPUT_TYPE_FLASHMOVIE => [
				we_base_ContentTypes::FLASH,
				we_base_ContentTypes::FLASH,
				'image',
				['width' => 'imgwidth',
					'height' => 'imgheight',
					'origwidth' => 'imgwidth',
					'origheight' => 'imgheight'
				]
			],
			we_fileupload::USERINPUT_TYPE_VIDEO => [
				'video/',
				we_base_ContentTypes::VIDEO,
				'video',
				['width' => 'width',
					'height' => 'height'
				]
			],
			we_fileupload::USERINPUT_TYPE_AUDIO => [
				'audio/',
				we_base_ContentTypes::AUDIO,
				'audio',
				[]
			],
			we_fileupload::USERINPUT_TYPE_IMG => [
				'image/',
				we_base_ContentTypes::IMAGE,
				'image',
				['width' => 'imgwidth',
					'height' => 'imgheight',
					'origwidth' => 'imgwidth',
					'origheight' => 'imgheight'
				]
			],
		];
		$matchType = $dataType = '';
		$moreAttribs = [];
		foreach($_REQUEST as $key => $dataID){
			$contentType = '';
			foreach($checks as $check => $checkData){
				if(preg_match('|^' . $check . '(.*)$|', $key, $regs)){
					list($matchType, $contentType, $dataType, $moreAttribs) = $checkData;
					break;
				}
			}
			if(!$contentType || !isset($_SESSION[$dataID])){
				continue;
			}
			$name = $regs[1];
			$id = empty($_SESSION[$dataID]['id']) ? 0 : intval($_SESSION[$dataID]['id']);
			$contentType = !empty($_SESSION[$dataID]['type']) && $_SESSION[$dataID]['type'] === we_base_ContentTypes::PDF ? we_base_ContentTypes::PDF : $contentType;

			if(!empty($_SESSION[$dataID]['doDelete'])){
				if($id){
					$document = we_base_ContentTypes::getObject($contentType);
					assert($document instanceof we_document_base);
					$document->initByID($id);
					if($document->WebUserID == $webuserId){
						//everything ok, now delete
						we_base_delete::deleteEntry($id, FILE_TABLE);
						$doc->setElementI($name, 0);
					}
				}
			} elseif(isset($_SESSION[$dataID]['serverPath'])){
				if(strpos($_SESSION[$dataID]['type'], $matchType) === 0){
					$document = we_base_ContentTypes::getObject($contentType);
					assert($document instanceof we_document_files);
					if($id){
						// document has already an image
						// so change binary data
						$document->initByID($id);
					} else {
						$document->setParentID($_SESSION[$dataID]['parentid']);
					}

					$document->Table = FILE_TABLE;
					$document->ContentType = $contentType; // needed to switch between otherDocument and PDF
					$document->Publish_Date = new DateTime();
					$document->WebUserID = $webuserId;
					$document->Filename = $_SESSION[$dataID]['fileName'];
					$document->Extension = $_SESSION[$dataID]['extension'];
					$document->Path = $document->getPath();
					$document->setElementS('data', $_SESSION[$dataID]['serverPath'], $dataType);
					$document->setElementI('filesize', $_SESSION[$dataID]['size'], 'attrib');
					foreach($moreAttribs as $eName => $sKey){
						$document->setElementI($eName, $_SESSION[$dataID][$sKey], 'attrib');
					}

					$document->we_save();

					if(!empty($_SESSION[$dataID]['unique'])){
						$document->Filename = str_replace($_SESSION[$dataID]['unique'], strval($document->ID), $document->Filename);
					} else {
						$t = explode('_', $document->Filename);
						$t[1] = $document->ID;
						$document->Filename = implode('_', $t);
					}

					$document->Path = $document->getPath();
					$document->we_save();

					$doc->setElementI($name, $document->ID);
				}
			}
			unset($_SESSION[$dataID]);
		}
	}

}
