<?php
declare(strict_types=1);

/**

 * webEdition CMS
 *
 * $Rev: 5b14d86c1d11 $
 * $Author: Lukas Imhof $
 * $Date: 2020-02-21 17:35 +0100 $
 */
class we_tw_userInput extends we_weTag_twbase{

	public function __construct(string $tagName){
		parent::__construct($tagName);
		$this->NeedsEndTag = false;
		$this->Module = we_modules_info::CUSTOMER;
		$this->Description = g_l('weTag', '[' . $tagName . '][description]', true, true);

		$name = new we_weTag_textAttribute('name', true, '');
		$property = new we_weTag_selectAttribute('property', we_weTag_selectAttribute::getTrueFalse());
		$checked = new we_weTag_selectAttribute('checked', we_weTag_selectAttribute::getTrueFalse());
		$editable = new we_weTag_selectAttribute('editable', we_weTag_selectAttribute::getTrueFalse());
		$removefirstparagraph = new we_weTag_selectAttribute('removefirstparagraph', we_weTag_selectAttribute::getTrueFalse());
		$size = new we_weTag_textAttribute('size');
		$maxlength = new we_weTag_textAttribute('maxlength');
		$format = new we_weTag_textAttribute('format');
		$value = new we_weTag_textAttribute('value');
		$values = new we_weTag_textAttribute('values');
		$hidden = new we_weTag_selectAttribute('hidden', we_weTag_selectAttribute::getTrueFalse());
		$currentdate = new we_weTag_selectAttribute('currentdate', we_weTag_selectAttribute::getTrueFalse());
		$cols = new we_weTag_textAttribute('cols');
		$rows = new we_weTag_textAttribute('rows');
		$pure = new we_weTag_selectAttribute('pure', we_weTag_selectAttribute::getTrueFalse());
		$width = new we_weTag_textAttribute('width');
		$height = new we_weTag_textAttribute('height');
		$bgcolor = new we_weTag_textAttribute('bgcolor');
		$class = new we_weTag_textAttribute('class');
		$style = new we_weTag_textAttribute('style');
		$classes = new we_weTag_textAttribute('classes');
		$wysiwyg = new we_weTag_selectAttribute('wysiwyg', we_weTag_selectAttribute::getTrueFalse());
		$commands = new we_weTag_choiceAttribute('commands', we_wysiwyg_editor::getEditorCommands(true), false, true, '');
		$contextmenu = new we_weTag_choiceAttribute('contextmenu', we_wysiwyg_editor::getEditorCommands(true), false, true, '');
		$editorcss = new we_weTag_selectorAttribute('editorcss', FILE_TABLE, we_base_ContentTypes::CSS, false, '');
		$ignoredocumentcss = new we_weTag_selectAttribute('ignoredocumentcss', we_weTag_selectAttribute::getTrueFalse());
		$parentid = new we_weTag_selectorAttribute('parentid', FILE_TABLE, we_weTag_selectorAttribute::FOLDER, true);
		$quality = new we_weTag_selectAttribute('quality', [
			new we_weTag_option('0'),
			new we_weTag_option('1'),
			new we_weTag_option('2'),
			new we_weTag_option('3'),
			new we_weTag_option('4'),
			new we_weTag_option('5'),
			new we_weTag_option('6'),
			new we_weTag_option('7'),
			new we_weTag_option('8'),
			new we_weTag_option('9'),
			new we_weTag_option('10'),
		]);
		$keepratio = new we_weTag_selectAttribute('keepratio', we_weTag_selectAttribute::getTrueFalse());
		$maximize = new we_weTag_selectAttribute('maximize', we_weTag_selectAttribute::getTrueFalse());
		$bordercolor = new we_weTag_textAttribute('bordercolor');
		$checkboxstyle = new we_weTag_textAttribute('checkboxstyle');
		$checkboxclass = new we_weTag_textAttribute('checkboxclass');
		$inputstyle = new we_weTag_textAttribute('inputstyle');
		$inputclass = new we_weTag_textAttribute('inputclass');
		$checkboxtext = new we_weTag_textAttribute('checkboxtext');
		$doc = new we_weTag_selectAttribute('doc', [
			new we_weTag_option('self'),
			new we_weTag_option('top'),
			], false, '');
		$minyear = new we_weTag_textAttribute('minyear');
		$maxyear = new we_weTag_textAttribute('maxyear');
		/* $thumbnail = new we_weTag_sqlRowAttribute('thumbnail', THUMBNAILS_TABLE, false, 'Text', '', '', '');
		  $thumbid = new we_weTag_sqlRowAttribute('thumbnailid', THUMBNAILS_TABLE, false, 'ID', 'Text', 'Text');
		 */

		$this->TypeAttribute = new we_weTag_typeAttribute('type', [
			new we_weTag_option('textinput', false, '', [$name, $property, $editable, $size, $maxlength, $value,
				$class, $style], [$name]),
			new we_weTag_option('textarea', false, '', [$name, $property, $editable, $value, $cols, $rows, $width, $height, $bgcolor, $class, $style, /* $hideautobr, */
				$wysiwyg, $wysiwyg, $ignoredocumentcss, $editorcss, $commands, $contextmenu, $classes], [$name]),
			new we_weTag_option('checkbox', false, '', [$name, $property, $checked, $editable], [$name]),
			new we_weTag_option('radio', false, '', [$name, $property, $checked, $editable, $value], [$name]),
			new we_weTag_option('choice', false, '', [$name, $property, $editable, $size, $maxlength, $value, $values, $class, $style], [$name]),
			new we_weTag_option('select', false, '', [$name, $property, $editable, $size, $value, $values, $class, $style], [$name]),
			new we_weTag_option('hidden', false, '', [$name, $property, $value], [$name]),
			new we_weTag_option('print', false, '', [$name, $property], [$name]),
			new we_weTag_option('date', false, '', [$name, $property, $editable, $currentdate, $format, $value, $minyear, $maxyear, $hidden], [$name]),
			new we_weTag_option('password'),
			new we_weTag_option('img', false, '', [$name, $editable, $size, $value, $width, $height, /* $thumbnail, $thumbid, */ $parentid, $quality, $keepratio, $maximize,
				$bordercolor, $checkboxstyle, $checkboxclass, $inputstyle, $inputclass, $checkboxtext], [$name, $parentid]),
			new we_weTag_option('flashmovie', false, '', [$name, $editable, $size, $value, $width, $height, $parentid, $quality, $keepratio, $maximize, $bordercolor,
				$checkboxstyle, $checkboxclass, $inputstyle, $inputclass, $checkboxtext], [$name, $parentid]),
			new we_weTag_option('video', false, '', [$name, $editable, $size, $value, $width, $height, $parentid, $checkboxstyle, $checkboxclass, $inputstyle, $inputclass,
				$checkboxtext], [$name, $parentid]),
			new we_weTag_option('audio', false, '', [$name, $editable, $size, $value, $parentid, $checkboxstyle, $checkboxclass, $inputstyle, $inputclass, $checkboxtext], [
				$name, $parentid]),
			new we_weTag_option('binary', false, '', [$name, $editable, $size, $value, $parentid, $bordercolor, $checkboxstyle, $checkboxclass, $inputstyle, $inputclass,
				$checkboxtext], [$name, $parentid]),
			new we_weTag_option('country', false, '', [$name, $size, $class, $style, $doc, $value], [$name]),
			new we_weTag_option('language', false, '', [$name, $size, $class, $style, $doc, $value], [$name])
			], true, '');

		$this->Attributes = [
			$name,
			$property,
			$checked,
			$editable,
			$removefirstparagraph,
			$size, $maxlength, $format, $value, $values, $hidden, $currentdate, $cols,
			$rows, $pure, $width, $height, $bgcolor, $class, $style, $wysiwyg, $ignoredocumentcss, $editorcss, $commands, $contextmenu, $classes,
			$parentid, $quality, $keepratio, $maximize, /* $thumbnail, $thumbid, */ $bordercolor,
			$checkboxstyle, $checkboxclass, $inputstyle, $inputclass, $checkboxtext, $doc, $minyear, $maxyear];
	}

}
