<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 4f156532507d $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-05-06 23:12 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_users_frames extends we_modules_frame{
	/** @var we_users_view */
	protected $View;

	/** @var we_tree_users */
	protected $Tree;

	public function __construct(){
		parent::__construct();
		$this->frameset = WEBEDITION_DIR . 'we_cmd.php?we_cmd[0]=we_showMod&mod=' . we_modules_info::USERS;
		$this->module = we_modules_info::USERS;
		$this->Tree = new we_tree_users($this->jsCmd);
		$this->showTreeFooter = true;
		$this->View = new we_users_view();
	}

	protected function getHTMLCmd(): string{
		if(($pid = we_base_request::int('pid', -1)) < 0){
			//use this to get js code
			return $this->getHTMLDocument(we_html_element::body(), $this->jsCmd->getCmds());
		}

		$offset = we_base_request::int('offset', 0);
		$this->jsCmd->addCmd('loadTree', ['clear' => !$pid, 'items' => we_tree_users::getItems($pid, $offset, $this->Tree->default_segment)]);

		return $this->getHTMLDocument(
				we_html_element::body([], we_html_element::form(['name' => we_base_constants::WE_FORM], we_html_element::hiddens(['pnt' => "cmd",
							'cmd' => "no_cmd"])
					)
				)
		);
	}

	/* use parent
	  function getHTMLLeft(): string{}
	 *
	 */

	protected function getHTMLTreeFooter(): string{
		return $this->getHTMLSearchTreeFooter();
	}

	protected function getHTMLEditorHeader(): string{
		if(we_base_request::bool('home')){//FIXME: find one working condition
			return parent::getHTMLEditorHeader();
		}
		$user_object = $_SESSION['user_session_data'];
		return $this->getHTMLDocument(we_html_element::body(['onresize' => 'weTabs.setFrameSize()', 'onload' => 'weTabs.setFrameSize()', 'id' => 'eHeaderBody']), $user_object->formHeader(we_base_request::int('tab', 0)));
	}

	protected function getHTMLEditorBody(): string{
		if(we_base_request::bool('home')){
			return $this->View->getHomeScreen();
		}
		//$weSuggest = we_gui_suggest::getInstance();

		$user_object = ( $_SESSION['user_session_data'] ?? new we_users_user());

		$js = $this->View->getJSProperty();
		$tab = we_base_request::int('tab', 0);
		$content = we_html_element::hiddens([
				'cmd' => '',
				'tab' => strval($tab),
				'oldtab' => strval($tab),
				'obj_name' => $user_object->Name,
				'uid' => strval($user_object->ID),
				'ctype' => htmlspecialchars(we_base_request::string("ctype", '')),
				'ctable' => htmlspecialchars(we_base_request::string('ctable', '')),
				'sd' => '0']);

		if($user_object){
			if(($oldTab = we_base_request::int('oldtab', -1)) > -1){

				$user_object->preserveState($oldTab);
				$_SESSION['user_session_data'] = $user_object;
			}
			if(($start = we_base_request::int('seem_start_file', -1)) > -1){
				$_SESSION['save_user_seem_start_file'][we_base_request::int('uid')] = $start;
			}
			$content .= $user_object->formDefinition($this->jsCmd, $tab);
		}

		$form = we_html_element::form(['name' => we_base_constants::WE_FORM,
				'method' => 'post',
				'autocomplete' => 'off',
				'onsubmit' => 'return false'
				], $content);
		return $this->getHTMLDocument(we_html_element::body(['class' => 'weEditorBody', 'onload' => 'loaded=1;', 'onunload' => 'doUnload()'], $form), we_html_element::jsScript(JS_DIR . 'multiIconBox.js') . $js);
	}

	function getHTMLSearch(): string{
		$keyword = we_base_request::_(we_base_request::RAW, 'keyword', "");
		$arr = explode(' ', strToLower($keyword));
		$DB_WE = DB_WE::$inst;

		$array_and = [];
		$array_or = [];
		$array_not = [];

		for($i = 0; $i < count($arr); $i++){
			switch($arr[$i]){
				case 'not':
					$i++;
					$array_not[] = $arr[$i];
					break;
				case 'or':
					$i++;
					$array_or[] = $arr[$i];
					break;
				case 'and':
					$i++;
				//no break
				default:
					$array_and[] = $arr[$i];
					break;
			}
		}
		//FIXME: use persistent_slots to determine all fields
		$searchStr = '(First LIKE "%###WE###%" OR Second LIKE "%###WE###%" OR username LIKE "%###WE###%" OR Address LIKE "%###WE###%" OR City LIKE "%###WE###%" OR State LIKE "%###WE###%" OR Country LIKE "%###WE###%" OR Telephone LIKE "%###WE###%" OR Fax LIKE "%###WE###%" OR Description LIKE "%###WE###%")';

		$condition = '';
		foreach($array_and as $value){
			$value = $DB_WE->escape($value);
			$condition .= ($condition ? ' AND ' : '') .
				str_replace('###WE###', $value, $searchStr);
		}
		foreach($array_or as $value){
			$value = $DB_WE->escape($value);
			$condition .= ($condition ? ' OR ' : '') .
				str_replace('###WE###', $value, $searchStr);
		}
		foreach($array_not as $value){
			$value = $DB_WE->escape($value);
			$condition .= ($condition ? ' AND NOT ' : '') .
				str_replace('###WE###', $value, $searchStr);
		}
		$all = $DB_WE->getAllFirstq('SELECT ID,CONCAT(Salutation," ",First," ",Second," (",username,")") AS Name FROM ' . USER_TABLE . ($condition ? ' WHERE ' . $condition : '') . ' ORDER BY username', false);

		if($all){
			$select = '<select name="search_results" size="20" style="width:520px;height:220px;" ondblclick="top.we_cmd(\'check_user_display\',document.we_form.search_results.value); top.close();">';
			foreach($all as $id => $name){
				$select .= '<option value="' . $id . '">' . $name . '</option>';
			}
			$select .= '</select>';
		} else {
			$select = '<div style="background-color:white;width:520px;height:220px;"/>';
		}
		$buttons = we_html_button::position_yes_no_cancel(
				we_html_button::create_button(we_html_button::OPEN_TO_EDIT, "javascript:WE().util.dialogTop(window).we_cmd('check_user_display',document.we_form.search_results.value); if(document.we_form.search_results.value){WE().util.dialogTop(window).doClose()}"), '', we_html_button::create_button(we_html_button::CANCEL, "javascript:WE().util.dialogTop(window).doClose();")
		);

		$content = we_html_tools::formElementTable(g_l('modules_users', '[search_for]'), we_html_tools::textInput('keyword', $keyword, 0, [], 'text', '485px'), we_html_button::create_button(we_html_button::SEARCH, "javascript:document.we_form.submit();")
			) . '<div style="height:20px;"></div>' .
			we_html_tools::formElementTable(g_l('modules_users', '[search_result]'), $select);

		return $this->getHTMLDocument(we_html_element::body(['class' => 'weEditorBody'], we_html_element::form(['name' => we_base_constants::WE_FORM,
						'method' => 'post'], we_html_element::hiddens([
							'mod' => 'users',
							'pnt' => 'search']) .
						we_html_tools::dialogLayout($content, g_l('modules_users', '[search]'), $buttons))
				), we_html_element::jsScript(WE_JS_MODULES_DIR . 'users/users_view.js'));
	}

	public function getHTML(string $what = '', int $mode = 0, int $step = 0): string{
		switch($what){
			case self::FOOTER:
				return $this->getHTMLEditorFooter([
						'main' => [
							we_html_button::FOOTER_SAVE => [[we_base_perm::USER_GROUP_NEW, we_base_perm::USER_NEW, we_base_perm::USER_SAVE, we_base_perm::USER_GROUP_SAVE], ['save_user']],
						],
						'optional' => [
							we_html_button::FOOTER_TRASH => [[we_base_perm::USER_DELETE, we_base_perm::USER_GROUP_DELETE], ['delete_user']]
						]
				]);
			default:
				return parent::getHTML($what, $mode, $step);
		}
	}

}
