<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 7c1ecdc5ac50 $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-02-08 17:57 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

/**
 * Class we_widget()
 *
 * Use this class to add a widget to the Cockpit.
 */
abstract class we_widget_base{
	const WIDTH_SMALL = 202;
	const WIDTH_LARGE = 432;

	/** @var array */
	static $json = [];

	abstract public function __construct(string $curID = '', array $aProps = []);

	/**
	 * @return array{string,array{string,string}}
	 */
	abstract public function getInsertDiv(int $iCurrId, array $aProps, we_base_jsCmd $jsCmd): array;

	/**
	 * @return array<string,mixed>
	 */
	public static function getDefaultConfig(): array{
//overite
		return [];
	}

	public static function showDialog(): void{
//overite
	}

	abstract public function showPreview(): void;

	static function getDialogPrefs(array $dynVar = []): array{
		$dynVar['_sObjId'] = we_base_request::string('we_cmd', '', 0);
		$jsFile = we_html_element::jsScript(JS_DIR . 'widgets/dlg_prefs.js', '', ['id' => 'loadVarDlg_prefs', 'data-prefs' => setDynamicVar($dynVar)]);

		$oSctCls = (new we_html_select([
				'name' => "sct_cls",
				'class' => 'defaultfont',
				'style' => "width:120px;border:#AAAAAA solid 1px"
				]))
			->insertOption(0, "white", g_l('cockpit', '[white]'))
			->insertOption(1, "lightCyan", g_l('cockpit', '[lightcyan]'))
			->insertOption(2, "blue", g_l('cockpit', '[blue]'))
			->insertOption(3, "green", g_l('cockpit', '[green]'))
			->insertOption(4, "orange", g_l('cockpit', '[orange]'))
			->insertOption(5, "yellow", g_l('cockpit', '[yellow]'))
			->insertOption(6, "red", g_l('cockpit', '[red]'));

		$oSelCls = (new we_html_table(['class' => 'default'], 1, 2))
			->setCol(0, 0, g_l('cockpit', '[bgcolor]'), ['width' => '130', 'class' => 'defaultfont'])
			->setCol(0, 1, $oSctCls->getHTML());

		return [$jsFile, $oSelCls, $oSctCls->getHTML()];
	}

	/**
	 * To add a widget give a unique id ($iId). Currently supported widget types ($sType) are Shortcuts (sct), RSS Reader (rss),
	 * Last modified (mfd), Users Online (usr), and Unpublished docs and objs (ubp).
	 *
	 * @param      string $iId
	 * @param      string $sType
	 * @param      string $oContent
	 * @param      array{string,string} $aLabel
	 * @param      string $sCls
	 * @param      bool $iRes
	 * @param      string $sCsv
	 * @param      int $w
	 * @param      bool $resize
	 * @return     string    Returns the we_html_table object
	 */
	static function create(string $iId, string $sType, string $oContent, array $aLabel = ['', ''], string $sCls = "white", bool $iRes = false, string $sCsv = '', int $w = 0, bool $resize = true): string{
		$oIco_prc = (new we_html_table([], 1, 3))
			->setCol(0, 0, '<span class="fa-stack" title="' . g_l('cockpit', '[properties]') . '" onclick="propsWidget(\'' . $sType . '\',\'' . $iId . '\',document.getElementById(\'' . $iId . '_csv\').value);this.blur();">
		  <i class="fa fa-align-justify"></i>
		  </span>'
			)
			->setCol(0, 1, '<span id="' . $iId . '_icon_resize" class="fa-stack" title="' . g_l('cockpit', ($iRes ? '[reduce_size]' : '[increase_size]')) . '" onclick="resizeWidget(\'' . $iId . '\');this.blur();">
		  <i class="fa fa-' . ($iRes ? 'compress' : 'expand' ) . '"></i>
		  </span>'
			)
			->setCol(0, 2, '<span title="' . g_l('cockpit', '[close]') . '" onclick="removeWidget(\'' . $iId . '\');this.blur();">
		  <i class="fa fa-close"></i>
		  </span>');
		$oIco_pc = (new we_html_table([], 1, 2))
			->setCol(0, 0, '<span title="' . g_l('cockpit', '[properties]') . '" onclick="propsWidget(\'' . $sType . '\',\'' . $iId . '\',document.getElementById(\'' . $iId . '_csv\').value);this.blur();">
		  <i class="fa fa-align-justify"></i>
		  </span>')
			->setCol(0, 1, '<span title="' . g_l('cockpit', '[close]') . '" onclick="removeWidget(\'' . $iId . '\');this.blur();">
		  <i class="fa fa-close"></i>
		  </span>');

		$sIco = ($sType != "_reCloneType_") ? ($resize ? $oIco_prc->getHtml() : $oIco_pc->getHtml()) :
			we_html_element::div(['id' => $iId . "_ico_prc", 'style' => "display:block;"], $oIco_prc->getHtml()) .
			we_html_element::div(['id' => $iId . "_ico_pc", 'style' => "display:none;"], $oIco_pc->getHtml());

		$oTb = (new we_html_table(['id' => $iId . "_tb", 'class' => 'widget_controls'], 1, 2))
			->setColAttributes(0, 0, ['id' => $iId . "_h", 'class' => 'dragBar'])
			->setCol(0, 0, we_html_element::div(['id' => $iId . '_lbl', 'class' => 'label widgetTitle',]))
			->setCol(0, 1, $sIco);

		if($iId != 'clone'){
			self::$json[] = [
				$iId,
				str_replace("'", "\'", $aLabel[0]),
				str_replace("'", "\'", $aLabel[1])
			];
		}
		return we_html_element::div(['id' => $iId . '_bx', 'class' => 'widget bgc_' . $sCls . ' ' . ($w > self::WIDTH_SMALL ? 'cls_expand' : 'cls_collapse')], $oTb->getHtml() .
				we_html_element::div(['id' => $iId . '_wrapper', 'class' => 'content'], we_html_element::div(['id' => $iId . "_content"], $oContent) .
					we_html_element::hidden($iId . '_prefix', $aLabel[0], $iId . '_prefix') .
					we_html_element::hidden($iId . '_postfix', $aLabel[1], $iId . '_postfix') .
					we_html_element::hidden($iId . '_res', strval($iRes), $iId . '_res') .
					we_html_element::hidden($iId . '_type', $sType, $iId . '_type') .
					we_html_element::hidden($iId . '_cls', $sCls, $iId . '_cls') .
					we_html_element::hidden($iId . '_csv', $sCsv, $iId . '_csv')
				)
		);
	}

	public static function getJson(): array{
		return self::$json;
	}

}
