<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_wysiwyg_ToolbarButton extends we_wysiwyg_ToolbarElement{
	function __construct(we_wysiwyg_editor $editor, string $cmd, string $scope = self::SCOPE_TOOLBAR, string $group = ''){
		parent::__construct($editor, $cmd, $scope, $group);
		$this->group = $group;
	}

	function hasProp(string $cmd = ''): bool{
		//FIXME use of a structred array may be used for help to.
		switch($this->cmd){
			case 'sizeselect':
			case 'blockselect':
			case 'styleselect':
				return $this->editor->setPlugin('weselects', parent::hasProp() || parent::hasProp('prop'));
			case 'bold':
			case 'italic':
			case 'underline':
			case 'subscript':
			case 'superscript':
			case 'strikethrough':
			case 'removeformat':
				return parent::hasProp() || parent::hasProp('prop');
			case 'abbr':
			case 'acronym':
			case 'lang':
			case 'del':
			case 'ins':
			case 'cite' :
				return $this->editor->setPlugin('wexhtmlxtras', parent::hasProp() || parent::hasProp('xhtmlxtras'));
			case 'ltr':
			case 'rtl':
				return $this->editor->setPlugin('directionality', parent::hasProp() || parent::hasProp('xhtmlxtras'));
			case 'forecolor':
			case 'backcolor':
				return parent::hasProp() || parent::hasProp('color');
			case 'alignleft':
			case 'aligncenter':
			case 'alignright':
			case 'alignjustify':
				return parent::hasProp() || parent::hasProp('align');
			case 'bullist':
			case 'numlist':
				if(parent::hasProp() || parent::hasProp('list')){
					$this->editor->setPlugin('lists', true);
					$this->editor->setPlugin('advlist', true);
					return true;
				}
				return false;
			case 'blockquote':
			case 'indent':
			case 'outdent':
				return parent::hasProp() || parent::hasProp('list');
			case 'link':
			case 'unlink':
			case 'anchor':
				if(parent::hasProp() || parent::hasProp('link') || parent::hasProp('insert')){
					$this->editor->setPlugin('anchor', true);
					$this->editor->setPlugin('welink', true);
					return true;
				}
				return false;
			case 'image':
				return $this->editor->setPlugin('weimage', parent::hasProp() || parent::hasProp('insert'));
			case 'hr':
				return parent::hasProp() || parent::hasProp('insert');
			case 'break':
				return $this->editor->setPlugin('webreak', parent::hasProp() || parent::hasProp('insert'));
			case 'gallery':
				return $this->editor->setPlugin('wegallery', (parent::hasProp() || parent::hasProp('insert')) && !$this->editor->getIsFrontend() && we_modules_info::isActive(we_modules_info::COLLECTION));
			case 'tableinsert':
			case 'tablemenu':
				if(parent::hasProp('table') || parent::hasProp('insert')){
					$this->editor->setPlugin('table', true);
					$this->editor->setPlugin('wetable', true);
					return true;
				}
				return false;
			case 'pastetext':
				return $this->editor->setPlugin('wepastetext', parent::hasProp() || parent::hasProp('essential'));
			case 'undo':
			case 'redo':
				return parent::hasProp() || parent::hasProp('essential');
			case 'charmap':
				return $this->editor->setPlugin('charmap', parent::hasProp() || parent::hasProp('essential'));
			case 'visualaid':
				if(parent::hasProp() || parent::hasProp('essential')){
					$this->editor->setPlugin('visualblocks', true);
					$this->editor->setPlugin('visualchars', true);
					$this->editor->setPlugin('wevisualaid', true);
					return true;
				}
				return false;
			case 'selectall':
				return parent::hasProp() || parent::hasProp('essential');
			case 'search':
				return $this->editor->setPlugin('searchreplace', parent::hasProp() || parent::hasProp('essential'));
			case 'fullscreen':
				if(parent::hasProp() || parent::hasProp('essential')){
					$this->editor->setPlugin('fullscreen', !$this->editor->getIsFrontend());
					$this->editor->setPlugin('wefullscreen', !$this->editor->getIsFrontend());
					return true;
				}
				return false;
			case 'editsource':
				if(parent::hasProp() || parent::hasProp('essential')){
				$this->editor->setPlugin('wesource', true);
				$this->editor->setPlugin('editsource', true);
				//$this->editor->setPlugin('code', true);
					return true;
				}
				return false;
			case 'template':
				return $this->editor->setPlugin('template', parent::hasProp() || parent::hasProp('advanced'));
			case 'codesample':
				return $this->editor->setPlugin('codesample', parent::hasProp() || parent::hasProp('advanced'));
			default:
				return false;
		}
	}

}
