#!/usr/bin/php
<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: ebc65f1f63f6 $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-02-28 01:02 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
/*
 * The script exports webEdition backup file to the given file
 * webEdition must be installed
 */
define('WE_DOCUMENT_ROOT', (string) realpath(__DIR__ . '/../../../'));
require_once('../custom/conf/cliConfig.php'); // @phan-suppress-current-line PhanMissingRequireFile
// CONFIGURATION BEGINS ---------------------------------------------------------

/**
 * Path to backup file
 */
$_backup_filename = WE_DOCUMENT_ROOT . '/weBackup_daily';

// export details

/**
 * export webEdition Core Data (Documents, Templates and Navigation)
 */
$_REQUEST = [
	'handle_core' => true,
	/**
	 * export binary data
	 */
	'handle_binary' => true,
	/**
	 * export version data
	 */
	'handle_versions' => true,
	/**
	 * export version binary files
	 */
	'handle_versions_binarys' => true,
	/**
	 * export user data
	 */
	'handle_user' => true,
	/**
	 * export customer data
	 */
	'handle_customer' => true,
	/**
	 * export shop data
	 */
	'handle_shop' => true,
	/**
	 * export workflow data
	 */
	'handle_workflow' => true,
	/**
	 * export user data
	 */
	'handle_todo' => true,
	/**
	 * export newsletter data
	 */
	'handle_newsletter' => true,
	/**
	 * export temporary data
	 */
	'handle_temporary' => true,
	/**
	  /**
	 * export objects and classes
	 */
	'handle_object' => true,
	/**
	 * export scheduler data
	 */
	'handle_schedule' => true,
	/**
	 * export settings and preferences
	 */
	'handle_settings' => true,
	/**
	 * export configuration data
	 */
	'handle_configuration' => true,
	/**
	 * export webEdition export data
	 */
	'handle_export' => true,
	/**
	 * export voting data
	 */
	'handle_voting' => true,
	/**
	 * export extern fi�es
	 */
	'handle_extern' => false,
// be user friendly :-)
	'verbose' => true,
];

// CONFIGURATION ENDS ---------------------------------------------------------
// we want to see errors
ini_set('display_errors', '1');
error_reporting(E_ALL);

//use we-error handler; ignore if logging is disabled!
if(!isset($_SERVER['SERVER_NAME'])){
	$_SERVER['SERVER_NAME'] = $SERVER_NAME;
}

// include needed libraries
require_once(WE_DOCUMENT_ROOT . '/webEdition/we/include/we.inc.php');

if(!defined('WE_ERROR_SHOW')){
	define('WE_ERROR_SHOW', 1);
}
if(!defined('WE_ERROR_LOG')){
	define('WE_ERROR_LOG', 1);
}

we_base_errorHandler::start(false);

abstract class recoverBackup{

	const HELP = 'Usage: recoverBackup.php [options] [file]
Options:
  -v, --verbose              Verbosely list files processed

  --help                     Prints out this help

 Options what to recover:
  --all                      Recover everything except external files
  --core                     Documents and templates
  --versions                 Versions
  --versions_binarys         Versions binarys
  --binary                   Binary data
  --user                     User data
  --customer                 Customer data
  --shop                     Shop data
  --workflow                 Workflow data
  --newsletter               Newsletter data
  --temporary                Temporary data (unpublished data)
  --object                   Objects and Classes
  --schedule                 Scheduler data
  --settings                 Settings
  --configuration            Configuration Data
  --export                   webEdition-Export Data
  --voting                   Voting data
  --extern                   Extern files (only use this if you have
                             enough memory and cpu power)
';

	public static function _checkAll(bool $flag = true): void{
		$_REQUEST['handle_core'] = $flag;
		$_REQUEST['handle_binary'] = $flag;
		$_REQUEST['handle_versions'] = $flag;
		$_REQUEST['handle_versions_binarys'] = $flag;
		$_REQUEST['handle_user'] = $flag;
		$_REQUEST['handle_customer'] = $flag;
		$_REQUEST['handle_shop'] = $flag;
		$_REQUEST['handle_workflow'] = $flag;
		$_REQUEST['handle_todo'] = $flag;
		$_REQUEST['handle_newsletter'] = $flag;
		$_REQUEST['handle_temporary'] = $flag;
		$_REQUEST['handle_object'] = $flag;
		$_REQUEST['handle_schedule'] = $flag;
		$_REQUEST['handle_settings'] = $flag;
		$_REQUEST['handle_configuration'] = $flag;
		$_REQUEST['handle_export'] = $flag;
		$_REQUEST['handle_voting'] = $flag;
	}

	public static function getOptions($args){
// Short options
		$short_opts = 'v';

// Long options
		$long_opts = [
			'all',
			'core',
			'versions',
			'versions_binarys',
			'binary',
			'user',
			'customer',
			'shop',
			'workflow',
			'todo',
			'newsletter',
			'temporary',
			'object',
			'schedule',
			'settings',
			'configuration',
			'export',
			'voting',
			'extern',
			'verbose',
			'help'
		];

// Convert the arguments to options - check for the first argument
		return ($_SERVER['argv'] && realpath($_SERVER['argv'][0]) == __FILE__ ?
				Console_Getopt::getOpt($args, $short_opts, $long_opts) :
				Console_Getopt::getOpt2($args, $short_opts, $long_opts));
	}

	public static function doRecover(): void{
		// knock out identifiation and permissions
		$_SESSION['user']['isWeSession'] = true;
		$_SESSION['perms'] = ['ADMINISTRATOR' => true];
		$_SESSION['user']['Username'] = 1;


// Define exit codes for errors
		define('NO_ARGS', 10);
		define('INVALID_OPTION', 11);

// Reading the incoming arguments - same as $argv
		$args = Console_Getopt::readPHPArgv();


// Make sure we got them (for non CLI binaries)
		if(PEAR::isError($args)){
			fwrite(STDERR, $args->getMessage() . "\n");
			exit(NO_ARGS);
		}

		$options = self::getOptions($args);

// Check the options are valid
		if(PEAR::isError($options)){
			fwrite(STDERR, $options->getMessage() . "\n");
			fwrite(STDERR, self::HELP . "\n");
			exit(INVALID_OPTION);
		}
		if($args){
			$_REQUEST['verbose'] = false;
			self::_checkAll(false);
			$_REQUEST['handle_extern'] = false;
		}

		foreach($options[0] as $opt){
			switch($opt[0]){
				case '--all':
					self::_checkAll(true);
					break;

				case 'v':
				case '--verbose':
					$_REQUEST['verbose'] = true;
					break;

				case '--help':
					print self::HELP;
					exit(0);

				default:
					$_REQUEST['handle_' . preg_replace('/^--/', '', $opt[0])] = true;
			}
		}


// check if no option is checked

		$__optionsSelected = false;

		foreach($_REQUEST as $_k => $_v){
			if(substr($_k, 0, 7) == "handle_"){
				if($_v){
					$__optionsSelected = true;
					break;
				}
			}
		}

// if no option is checked, then check the dafaults
		if($__optionsSelected === false){
			$_REQUEST['handle_core'] = true;
		}


		if(isset($options[1][0])){
			$GLOBALS['_backup_filename'] = $options[1][0];
		}

		$_REQUEST['backup_select'] = basename($GLOBALS['_backup_filename']);
		if($GLOBALS['_backup_filename'] != WE_DOCUMENT_ROOT . WE_BACKUP_DIR . $_REQUEST['backup_select']){
			copy($GLOBALS['_backup_filename'], WE_DOCUMENT_ROOT . WE_BACKUP_DIR . $_REQUEST['backup_select']);
		}

		if(!we_backup_preparer::prepareImport()){
			print we_backup_preparer::getErrorMessage();
			return;
		}
		$settings = we_backup_settings::i();
		/* @phan-var we_backup_settings $settings */

		if($_REQUEST['verbose']){
			print "\nImporting from " . $GLOBALS['_backup_filename'] . "...\n";
		}
		while(($settings->offset < $settings->offset_end)){
			if($_REQUEST['verbose']){
				print '-';
			}
			if(!we_backup_import::import()){
				break;
			}
		}

		if($_REQUEST['verbose']){
			print "\nUpdate...\n";
		}

		we_updater::doUpdate('internal');

		we_backup_settings::clearSessionData();

		if($_REQUEST['verbose']){
			print "\nDone\n";
		}
	}

}

recoverBackup::doRecover();
