/* global top, WE, entries */

/**
 * webEdition CMS
 *
 * $Author: Marc Krämer <marc.kraemer@aldente-it.de> $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
'use strict';

function drawNewFolder() {
	WE().util.dialogTop(window).unselectAllFiles();
	WE().util.dialogTop(window).fscmd.location.replace(WE().util.dialogTop(window).queryString(WE().consts.selectors.NEWFOLDER, WE().util.dialogTop(window).fileSelect.data.currentDir));
}

function RenameFolder(id) {
	WE().util.dialogTop(window).unselectAllFiles();
	WE().util.dialogTop(window).fscmd.location.replace(WE().util.dialogTop(window).queryString(WE().consts.selectors.RENAMEFOLDER, WE().util.dialogTop(window).fileSelect.data.currentDir, "", id));
}

function showPreview(id) {
	if (WE().util.dialogTop(window).document.getElementById('fspreviewDiv')) {
		if (!id) {
			WE().util.dialogTop(window).document.getElementById('fspreviewDiv').innerHTML = "";
			return;
		}
		WE().util.rpc(WE().util.dialogTop(window).queryString(WE().consts.selectors.PREVIEW, id), '', function (weResponse) {
			var prev = WE().util.dialogTop(window).document.getElementById('fspreviewDiv'),
				html = '<table class="default previewTable" style="width:100%">',
				previewFields,
				i;

			prev.innerHTML = "";
			if (!weResponse.Success) {
				return;
			}
			setInfoSize();
			weWriteBreadCrumb(weResponse.DataArray.path);
			if (weResponse.DataArray.imgPreview) {
				html += '<tr><td colspan="2" class="image">' + weResponse.DataArray.imgPreview + '</td></tr>';
			}
			previewFields = weResponse.DataArray.previewFields;
			for (var part in previewFields) {
				if (previewFields[part].data.length) {
					html += '<tr><td colspan="2" class="headline">' + previewFields[part].headline + '</td></tr>';
					for (i = 0; i < previewFields[part].data.length; i++) {
						html += '<tr>' + (!previewFields[part].data[i].caption ? '<td colspan="2">' : '<td>' + previewFields[part].data[i].caption + ': </td><td>') + previewFields[part].data[i].content + '</td></tr>';
					}
				}
			}

			prev.innerHTML = html + '</table>';
		});
		//WE().util.dialogTop(window).fspreview.location.replace(WE().util.dialogTop(window).queryString(WE().consts.selectors.PREVIEW, id));
	}
}

function setInfoSize() {
	var infoElem;
	if ((infoElem = document.getElementById("info"))) {
		infoElem.style.height = document.body.clientHeight - (document.getElementById("previewpic") ? 160 : 0) + "px";
	}
}

function weWriteBreadCrumb(BreadCrumb) {
	if (top.document.getElementById("fspath")) {
		top.document.getElementById("fspath").innerHTML = BreadCrumb;
	}
}

function doClick(id, ct) {
	if (WE().util.dialogTop(window).document.getElementById('fspreviewDiv')) {
		WE().util.dialogTop(window).document.getElementById('fspreviewDiv').innerHTML = "";
	}
	if (ct == 1) {
		if (WE().util.dialogTop(window).metaKeys.doubleClick) {
			selectFile(id);
			setDir(id);
			window.setTimeout(function () {
				WE().util.dialogTop(window).metaKeys.doubleClick = false;
			}, 400);
		}
	} else if (WE().util.dialogTop(window).fileSelect.data.currentID == id && (!WE().util.dialogTop(window).metaKeys.ctrl)) {
		if (WE().util.dialogTop(window).fileSelect.options.userCanRenameFolder) {
			WE().util.dialogTop(window).RenameFolder(id);
		} else {
			selectFile(id);
		}

	} else if (WE().util.dialogTop(window).fileSelect.options.multiple) {
		if (WE().util.dialogTop(window).metaKeys.shift) {
			var oldid = WE().util.dialogTop(window).fileSelect.data.currentID;
			var currendPos = WE().util.dialogTop(window).getPositionByID(id);
			var firstSelected = WE().util.dialogTop(window).getFirstSelected();

			if (currendPos > firstSelected) {
				WE().util.dialogTop(window).selectFilesFrom(firstSelected, currendPos);
			} else if (currendPos < firstSelected) {
				WE().util.dialogTop(window).selectFilesFrom(currendPos, firstSelected);
			} else {
				WE().util.dialogTop(window).selectFile(id);
			}
			WE().util.dialogTop(window).fileSelect.data.currentID = oldid;

		} else if (!WE().util.dialogTop(window).metaKeys.ctrl) {
			WE().util.dialogTop(window).selectFile(id);
		} else if (WE().util.dialogTop(window).isFileSelected(id)) {
			WE().util.dialogTop(window).unselectFile(id);
		} else {
			WE().util.dialogTop(window).selectFile(id);
		}
	} else {
		selectFile(id);
	}
	if (WE().util.dialogTop(window).metaKeys.ctrl) {
		WE().util.dialogTop(window).metaKeys.ctrl = 0;
	}
	if (WE().util.dialogTop(window).metaKeys.shift) {
		WE().util.dialogTop(window).metaKeys.shift = 0;
	}
}

function setDir(id) {
	var e = WE().util.dialogTop(window).getEntry(id);
	showPreview(id);
	WE().util.dialogTop(window).fscmd.location.replace(WE().util.dialogTop(window).queryString(WE().consts.selectors.SETDIR, id));
	WE().util.dialogTop(window).document.getElementById('fspath').innerHTML = e.path;
}

function selectFile(id) {
	var a = WE().util.dialogTop(window).document.getElementsByName("fname")[0];
	if (id) {
		if (id === WE().util.dialogTop(window).fileSelect.data.currentID) {
			return;
		}

		showPreview(id);
		var e = WE().util.dialogTop(window).getEntry(id);
		if (a.value != e.text &&
			a.value.indexOf(e.text + ",") === -1 &&
			a.value.indexOf("," + e.text + ",") === -1 &&
			a.value.indexOf("," + e.text + ",") === -1) {

			a.value = a.value ?
				(a.value + "," + e.text) :
				e.text;
		}
		if (WE().util.dialogTop(window).fsbody.document.getElementById("line_" + id)) {
			WE().util.dialogTop(window).fsbody.document.getElementById("line_" + id).classList.add("selected");
		}
		WE().util.dialogTop(window).fileSelect.data.currentPath = e.path;
		WE().util.dialogTop(window).fileSelect.data.currentID = id;
		WE().util.dialogTop(window).fileSelect.data.we_editDirID = 0;
	} else {
		a.value = "";
		WE().util.dialogTop(window).fileSelect.data.currentPath = "";
		WE().util.dialogTop(window).fileSelect.data.we_editDirID = 0;
	}
}

function elementSelected() {
	return true;
}

function addEntry(ID, text, isFolder, path, modDate, contentType) {
	entries.push({
		ID: ID,
		text: text,
		isFolder: isFolder,
		path: path,
		modDate: modDate,
		contentType: contentType
	});
}

function writeBodyDir(d, newText, withModDate) {
	var body = (WE().util.dialogTop(window).fileSelect.data.makeNewFolder ?
		'<tr class="newEntry">' +
		'<td class="selectoricon">' + WE().util.getTreeIcon(WE().consts.contentTypes.FOLDER_COMMON, false) + '</td>' +
		'<td class="filename"><input type="hidden" name="we_FolderText" value="' + newText + '" /><input onMouseDown="WE().util.dialogTop(window).metaKeys.inputClick=true" name="we_FolderText_tmp" type="text" value="' + newText + '" class="wetextinput" /></td>' +
		(withModDate ? '<td class="selector moddate">' + WE().consts.g_l.fileselector.date_format + '</td>' : '') +
		'<th class="selector remain"></th></tr>' :
		'');

	for (var i = 0; i < entries.length; i++) {
		body += '<tr id="line_' + entries[i].ID + '" class="' + ((entries[i].ID == WE().util.dialogTop(window).fileSelect.data.currentID && (!WE().util.dialogTop(window).fileSelect.data.makeNewFolder)) ? 'selected' : '') + '" ' + ((WE().util.dialogTop(window).fileSelect.data.we_editDirID || WE().util.dialogTop(window).fileSelect.data.makeNewFolder) ? '' : ' onclick="return selectorOnClick(event,' + entries[i].ID + ');"') + (entries[i].isFolder ? ' onDblClick="return selectorOnDblClick(' + entries[i].ID + ');"' : '') + '>' +
			'<td class="selector selectoricon">' + WE().util.getTreeIcon(entries[i].contentType, false) + '</td>' +
			(WE().util.dialogTop(window).fileSelect.data.we_editDirID == entries[i].ID ?
				'<td class="selector filename"><input type="hidden" name="we_FolderText" value="' + entries[i].text + '" /><input onMouseDown="WE().util.dialogTop(window).metaKeys.inputClick=true" name="we_FolderText_tmp" type="text" value="' + entries[i].text + '" class="wetextinput" style="width:100%" />' :
				'<td class="selector cutText directory" title="' + entries[i].text + '">' + entries[i].text
				) +
			'</td>' + (withModDate ? '<td class="selector moddate">' + entries[i].modDate + '</td>' : '') +
			'<td class="remain"></td></tr>';
	}

	d.innerHTML = '<form name="we_form" target="fscmd" method="post" action="' + WE().util.dialogTop(window).fileSelect.options.formtarget + '" ' + (WE().util.dialogTop(window).fileSelect.data.makeNewFolder ? 'onsubmit="document.we_form.we_FolderText.value=document.we_form.we_FolderText_tmp.value;return true;"' : '') + '>' +
		'<table class="selector sort"><thead><tr>' + WE().util.dialogTop(window).fileSelect.options.selectorHeader + '</tr></thead><tbody>' + body + '</table></form>';
	if (WE().util.dialogTop(window).fileSelect.data.makeNewFolder || WE().util.dialogTop(window).fileSelect.data.we_editDirID) {
		WE().util.dialogTop(window).fsbody.document.we_form.we_FolderText_tmp.focus();
		WE().util.dialogTop(window).fsbody.document.we_form.we_FolderText_tmp.select();
	}
	window.fsbody.$('table.sort').tablesorter();
}

function writeBody(d) {
	writeBodyDir(d, WE().consts.g_l.fileselector.new_folder_name, true);
}

function queryString(what, id, o, we_editDirID) {
	if (!o) {
		o = WE().util.dialogTop(window).fileSelect.data.order;
	}
	return WE().util.dialogTop(window).fileSelect.options.formtarget +
		'&we_cmd[what]=' + what +
		'&we_cmd[rootDirID]=' + WE().util.dialogTop(window).fileSelect.options.rootDirID +
		'&we_cmd[open_doc]=' + WE().util.dialogTop(window).fileSelect.options.open_doc +
		'&we_cmd[table]=' + WE().util.dialogTop(window).fileSelect.options.table +
		'&we_cmd[id]=' + id +
		(o ? ("&we_cmd[order]=" + o) : "") +
		(we_editDirID ? ("&we_cmd[we_editDirID]=" + we_editDirID) : "");
}

function weonclick(e) {
	if (WE().util.dialogTop(window).document.getElementById('fspreviewDiv')) {
		WE().util.dialogTop(window).document.getElementById('fspreviewDiv').innerHTML = "";
	}
	if (WE().util.dialogTop(window).fileSelect.data.makeNewFolder || WE().util.dialogTop(window).fileSelect.data.we_editDirID) {
		if (!WE().util.dialogTop(window).metaKeys.inputClick) {
			WE().util.dialogTop(window).fileSelect.data.makeNewFolder = WE().util.dialogTop(window).fileSelect.data.we_editDirID = false;
			document.we_form.we_FolderText.value = document.we_form.we_FolderText_tmp.value;
			document.we_form.submit();
		} else {
			WE().util.dialogTop(window).metaKeys.inputClick = false;
		}
	} else {
		WE().util.dialogTop(window).metaKeys.inputClick = false;
		if (e.altKey || e.metaKey || e.ctrlKey) {
			WE().util.dialogTop(window).metaKeys.ctrl = true;
		}
		if (e.shiftKey) {
			WE().util.dialogTop(window).metaKeys.shift = true;
		}
		if (WE().util.dialogTop(window).fileSelect.options.multiple) {
			if (!WE().util.dialogTop(window).metaKeys.shift && !WE().util.dialogTop(window).metaKeys.ctrl) {
				WE().util.dialogTop(window).unselectAllFiles();
			}
		} else {
			WE().util.dialogTop(window).unselectAllFiles();
		}
	}
}
